# -*- coding: latin-1 -*-
###############################################################################
# Copyright © 2018 NetApp, Inc. All Rights Reserved.
#
# CONFIDENTIALITY NOTICE:  THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
# NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
# EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
###############################################################################
from urllib3.util import parse_url

from envparse import env
import builtins
import json
import requests
import tenacity

TOKEN_URL_CONFIG_KEY = 'token_url'
JWKS_URL_CONFIG_KEY = 'jwks_url'
CONFIG_URL_CONFIG_KEY = 'config_url'
TESTING_MODE_KEY = 'testing_mode'


def _read_text(path: str) -> str:
    with builtins.open(path, 'r', encoding='utf8') as data_file:
        data = data_file.read()
    return data


def _read_json(path: str) -> str:
    """
    Read a text file in JSON format
    :param path: The file to read
    :return: The file's contents decoded from JSON
    """
    data = _read_text(path)
    return json.loads(data)


class Config:
    AUTH_CLIENT_ID = env('AUTH_CLIENT_ID', default='unknown')
    AUTH_CONFIG_PATH = env('AUTH_CONFIG_PATH', default='/sf/etc/auth/config.json')
    AUTH_SECRET_PATH = env('AUTH_SECRET_PATH', default='/sf/etc/auth/{0}.secret'.format(AUTH_CLIENT_ID))

    @staticmethod
    def getClientSecret() -> str:
        secret = _read_text(Config.AUTH_SECRET_PATH)
        return secret

    @staticmethod
    def getTokenUrl() -> str:
        data = _read_json(Config.AUTH_CONFIG_PATH)
        return data[TOKEN_URL_CONFIG_KEY]

    @staticmethod
    def getJwksUrl() -> str:
        data = _read_json(Config.AUTH_CONFIG_PATH)
        return data[JWKS_URL_CONFIG_KEY]

    @staticmethod
    def getJwksData() -> str:
        jwks_url = Config.getJwksUrl()
        r = tenacity.Retrying(wait=tenacity.wait_fixed(1), stop=tenacity.stop_after_attempt(5))
        data = r.call(lambda: requests.get(jwks_url, verify=False).text)
        return data

    @staticmethod
    def getConfigurationUrl() -> str:
        data = _read_json(Config.AUTH_CONFIG_PATH)
        return data[CONFIG_URL_CONFIG_KEY]

    @staticmethod
    def getTestingMode() -> bool:
        data = _read_json(Config.AUTH_CONFIG_PATH)
        return data[TESTING_MODE_KEY]

    @staticmethod
    def getAboutUrl() -> str:
        token_url = parse_url(Config.getTokenUrl())
        return "{}://{}/auth/about".format(token_url.scheme, token_url.netloc)
