# -*- coding: latin-1 -*-
###############################################################################
# Copyright © 2018 NetApp, Inc. All Rights Reserved.
#
# CONFIDENTIALITY NOTICE:  THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
# NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
# EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
###############################################################################


import unittest
from unittest import TestCase
from unittest.mock import MagicMock, patch
from authtools.utils import is_valid_url, get_json_data
from requests.exceptions import HTTPError, ConnectionError


class TestAuthToolsUtils(TestCase):
    def setUp(self):
        pass

    def test_is_valid_url_fqdn_true(self):
        teststr = 'https://www.google.com'
        retval = is_valid_url(teststr)
        self.assertTrue(retval)

    def test_is_valid_url_ip_true(self):
        teststr = 'http://192.168.1.1:8080'
        retval = is_valid_url(teststr)
        self.assertTrue(retval)

    def test_is_valid_url_localhost_true(self):
        teststr = 'http://localhost'
        retval = is_valid_url(teststr)
        self.assertTrue(retval)

    def test_is_valid_url_ipv6_true(self):
        teststr = 'http://[2001:4860:0:2001::68]/'
        retval = is_valid_url(teststr)
        self.assertTrue(retval)

    def test_is_valid_url_no_schema_false(self):
        teststr = 'www.microsoft.com'
        retval = is_valid_url(teststr)
        self.assertFalse(retval)

    def test_is_valid_url_invalid_schema_false(self):
        teststr = 'blah://www.microsoft.com'
        retval = is_valid_url(teststr)
        self.assertFalse(retval)

    def test_is_valid_url_no_netloc_false(self):
        teststr = 'http://'
        retval = is_valid_url(teststr)
        self.assertFalse(retval)

    def test_is_valid_url_invalid_chars_false(self):
        teststr = 'http://www.netapp*&^%$.com'
        retval = is_valid_url(teststr)
        self.assertFalse(retval)

    @patch('authtools.utils.requests.get')
    def test_get_json_data_succeeds(self, mock_get):
        mock_get.return_value = MagicMock()
        mock_get.return_value.json.return_value = {'key': 'value'}

        testUrl = 'https://localhost:5000/.well-known/openid-configuration'
        data = get_json_data(testUrl)
        self.assertTrue(data)

    @patch('authtools.utils.requests.get')
    def test_get_json_data_bad_status_fails(self, mock_get):
        mock_get.return_value = MagicMock()
        mock_get.return_value.raise_for_status.side_effect = HTTPError()

        testUrl = 'https://localhost:5000/.well-known/openid-configuration'
        with self.assertRaises(HTTPError):
            get_json_data(testUrl)

    @patch('authtools.utils.requests.get')
    def test_get_json_data_invalid_json_fails(self, mock_get):
        mock_get.return_value = MagicMock()
        mock_get.return_value.json.side_effect = ValueError()

        testUrl = 'https://localhost:5000/.well-known/openid-configuration'
        with self.assertRaises(ValueError):
            get_json_data(testUrl)

    @patch('authtools.utils.requests.get')
    def test_get_json_data_connection_error_fails(self, mock_get):
        mock_get.side_effect = ConnectionError()

        testUrl = 'https://localhost:5000/.well-known/openid-configuration'
        with self.assertRaises(ConnectionError):
            get_json_data(testUrl)


if __name__ == '__main__':
    unittest.main()
