# -*- coding: latin-1 -*-
###############################################################################
# Copyright © 2018 NetApp, Inc. All Rights Reserved.
#
# CONFIDENTIALITY NOTICE:  THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION OF
# NETAPP, INC. USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE PRIOR
# EXPRESS WRITTEN PERMISSION OF NETAPP, INC.
###############################################################################


from rfc3986 import uri_reference
import requests

from authtools import Config

ALLOWED_SCHEMES = ['http', 'https']


def is_valid_url(url):
    """
    Check if a specified URL is a valid URL for authentication purposes (discovery, redirection, etc.)
    A valid URL must contain both a scheme and network location, and scheme must be either HTTP or HTTPS.
    :param url: The URL to check for validity
    :return: True if the URL is valid, otherwise False
    """

    url_ref = uri_reference(url)
    if not url_ref.is_valid(require_scheme=True, require_authority=True):
        return False

    if url_ref.scheme not in ALLOWED_SCHEMES:
        return False

    return True


def get_json_data(url):
    """
    Get JSON data from an unsecured URL such as is used by OpenID discovery
    :param url: The URL to retrieve JSON data from
    :return: The JSON data returned from the source, converted to Python dict/array
    """
    response = requests.get(url, verify=False)
    response.raise_for_status()
    return response.json()
