"""Camelot enums."""
import aenum


class ResultStatus(aenum.Enum):
    """Test Result Status Enum.

    These statuses are used by test cases and suites.

    pending
        The test has been instantiated, but not yet run.
    running
        The test is currently running.
    skipped
        The test was skipped due to settings in the suite config.
    passed
        The test has finished executing and was successful.
    failed
        The body of the test encountered an exception. Ideally, this status indicates that the
        test case detected a bug.
    error
        An exception occurred outside of the test body. This generally means that the test case
        or test suite failed to properly setup or teardown. Ideally, this status indicates a problem
        with resources or the test case itself as opposed to a product bug.
    cancelled
        The test case was cancelled during execution via SIGINT
    """
    pending = 'pending'
    running = 'running'
    skipped = 'skipped'
    passed = 'passed'
    failed = 'failed'
    error = 'error'
    cancelled = 'cancelled'


class CaseTransition(aenum.Enum):
    """Contain the enum for the transitions."""

    setup = 'Setup'
    test = 'Test'
    teardown = 'Teardown'
    post_test_assertions = 'Post test assertions'


class LoggingLevel(aenum.Enum):
    """Logging levels as defined in the suite config file."""
    info = 'INFO'
    debug = 'DEBUG'
    warning = 'WARNING'
    error = 'ERROR'
    critical = 'CRITICAL'


class SaveState(aenum.Enum):
    """Used to specify under what conditions state from tests should be saved."""
    on_fail = 'on_fail'
    always = 'always'
    never = 'never'
