"""Formatting tools for camelot."""
import re

FMT_SUITE_SETUP = 'Test Suite: Starting Setup of {name}'
FMT_SUITE_TEARDOWN = 'Test Suite: Starting Teardown of {name}'

FMT_TEST_CASE_SKIP = 'Test Case {index}: Skipping {name}'
FMT_TEST_CASE_SETUP = 'Test Case {index}: Starting Setup of {name}'
FMT_TEST_CASE_PRECONDITIONS = 'Test Case {index}: Starting Preconditions of {name}'
FMT_TEST_CASE_TEST = 'Test Case {index}: Starting Test of {name}'
FMT_TEST_CASE_POST_TEST_ASSERTIONS = 'Test Case {index}: Starting Post-test Assertions of {name}'
FMT_TEST_CASE_POST_TEST_EXECUTION = 'Test Case {index}: Starting Post-test Execution of {name}'
FMT_TEST_CASE_TEARDOWN = 'Test Case {index}: Starting Teardown of {name}'

FMT_TEST_CASE_STATUS = 'Test Case {index}: {name} returned status {status}'
FMT_TEST_STEP = 'Starting Step {index} for {name}: {action}\nExpect: {expect}'
FMT_TEST_CASE_TRANSITION = 'Camelot {status} in {name} {transition}: {exc}'

# Create the regular expressions. Move these to the parser?
_name_re = r'(?P<name>\S+)'
_index_re = r'(?P<index>\d+)'
_RE_PREFIX = r'\n'

RE_SUITE_SETUP = re.compile(_RE_PREFIX + FMT_SUITE_SETUP.format(name=_name_re))
RE_SUITE_TEARDOWN = re.compile(_RE_PREFIX + FMT_SUITE_TEARDOWN.format(name=_name_re))
RE_TEST_CASE_SKIP = re.compile(_RE_PREFIX + FMT_TEST_CASE_SKIP.format(index=_index_re, name=_name_re))
RE_TEST_CASE_SETUP = re.compile(_RE_PREFIX + FMT_TEST_CASE_SETUP.format(index=_index_re, name=_name_re))
RE_TEST_CASE_TEST = re.compile(_RE_PREFIX + FMT_TEST_CASE_TEST.format(index=_index_re, name=_name_re))
RE_TEST_CASE_TEARDOWN = re.compile(
        _RE_PREFIX + FMT_TEST_CASE_TEARDOWN.format(index=_index_re, name=_name_re))
RE_TEST_CASE_STATUS = re.compile(FMT_TEST_CASE_STATUS.format(
        index=_index_re, name=_name_re, status=r'(?P<status>\S+)'))
RE_LOG_STEP = re.compile(r'{}\n-----'.format(FMT_TEST_STEP.format(index=_index_re,
                                                                  name=_name_re,
                                                                  action='(?P<action>.*?)',
                                                                  expect='(?P<expect>.*?)')))


def banner(msg, banner_char='=', banner_width=105):
    """Make a banner log.

    :param str msg: Message that will be inside banner
    :param str banner_char: The banner character used, def '='
    :param int banner_width: The width of the banner
    :return: Message surrounded by a banner
    :rtype: str
    """
    output = '\n\n{}\n{}\n{}\n'.format(banner_char * banner_width,
                                       msg,
                                       banner_char * banner_width)
    return output


def step_banner(msg):
    """Make a banner for steps with '$'s.

    :param str msg: Message that will be inside banner
    :return: Message surrounded by a banner
    :rtype: str
    """
    return banner(msg, banner_char='-')


def format_time_delta(time_delta):
    """Get the time delta in string format: DD:HH:MM:SS.SSSS.

    :param datetime.timedelta time_delta: The time delta you wish to stringify
    :return: string time delta
    :rtype: str
    """
    total_seconds = time_delta.total_seconds()
    days, remainder = divmod(total_seconds, 60 * 60 * 24)
    days = int(days)
    hours, remainder = divmod(remainder, 60 * 60)
    hours = int(hours)
    minutes, seconds = divmod(remainder, 60)
    minutes = int(minutes)
    return '{:02d}:{:02d}:{:02d}:{:07.4f}'.format(days, hours, minutes, seconds)
