"""Helper functions for Camelot."""

import abc
import inspect
import os

import autologger
import six

_log = autologger.get_sflogging(__name__)

INDENT_STR = '|'


def fmt_called_from(stack):
    """Format the called-from stack for logging.

    :param list stack: stack object from inspect.stack()
    :rtype: str
    """
    caller = inspect.getframeinfo(stack[1][0])
    folder, file_name = os.path.split(caller.filename)
    _, folder = os.path.split(folder)
    return '{}:{}'.format(os.path.join(folder, file_name), caller.lineno)


@six.add_metaclass(abc.ABCMeta)
class PrettyLines(object):
    """Abstract class for pretty formatting an object."""

    @abc.abstractmethod
    def pretty_lines(self, indent=''):
        """Override to get the pretty lines with an indent.

        :param str indent: String used to indent lines
        :rtype: list[str]
        """

    def pretty_fmt(self):
        """Get the pretty string.

        :rtype: str
        """
        return '\n'.join(self.pretty_lines())


def add_indent(indent):
    """Add one to the indent.

    :param str indent: String to use for indentation
    :rtype: str
    """
    if not indent:
        return INDENT_STR
    else:
        return indent + indent[0]
