"""Tests for the log filter in the Valence driver."""
import os
import unittest

import mock

from camelot.driver import (
    _filter_log,
    vl_filter,
)
from valence import (
    models,
    unittest_utils,
)

DATA_DIR = unittest_utils.get_data_dir(__file__)


class TestFilterAndCreateLog(unittest.TestCase):

    @mock.patch('shutil.copyfile')
    def test_filter_and_create_log(self, mock_copyfile):
        log_filter = models.CamelotLogFilter.get_mock_object()
        log_filter.exclude_regexes = ['valence.camelot']
        log_filter.match_regexes = ['Test Case']
        log_filter.output_log = 'test.log'
        log_filter.create_file = True
        log_path = os.path.join(DATA_DIR, 'test.log')
        mock_copyfile = mock.Mock()
        _filter_log.filter_and_create_log(log_filter=log_filter, test_log_path=log_path)
        assert log_filter.output_log == 'new.test.log'

    @mock.patch('shutil.copyfile')
    def test_create_log_create_file_False(self, mock_copyfile):
        log_filter = models.CamelotLogFilter.get_mock_object()
        log_filter.create_file = False
        log_path = os.path.join(DATA_DIR, 'test.log')
        _filter_log.filter_and_create_log(log_filter=log_filter, test_log_path=log_path)

    def test_process_input_line(self):
        input_line = 'Test filter line'
        exclude_regexes = ['Test']
        match_regexes = ['Case']
        filtered_line = _filter_log.process_input_line(input_line, match_regexes=match_regexes,
                                                       exclude_regexes=exclude_regexes)
        assert filtered_line is None


class TestVlFilter(unittest.TestCase):

    @mock.patch('camelot.driver._filter_log.filter_and_create_log')
    @mock.patch('sys.stdin.isatty')
    def test_filter_and_create_logs(self, mock_isatty, mock_filter_and_create_log):
        args = mock.Mock()
        mock_isatty.return_value = True
        vl_filter.filter_and_create_logs(args)

    @mock.patch('camelot.driver._filter_log.process_input_line')
    @mock.patch('sys.stdout.write')
    @mock.patch('sys.stdin.readline')
    @mock.patch('sys.stdin.isatty')
    def test_filter_and_create_logs_stream(self, mock_isatty, mock_readline, mock_write,
                                           mock_filter_and_create_log):
        args = ''
        mock_isatty.return_value = False
        mock_write.return_value = None
        mock_readline.side_effect = ['a line', None]
        vl_filter.filter_and_create_logs(args)
