import unittest

import mock
from requests import HTTPError

from camelot.driver import _main
from valence import (
    models,
    unittest_utils,
)


class TestPostMethodHipChat(unittest.TestCase):
    """Unit tests for the post method of hip chat."""
    room_name = '1858'
    message = 'Hello'
    color = 'random'
    token = 'abc'

    hipchat = models.HipChatAPI.create(
        room_id_or_name='foo',
        token='bar',
    )

    config = models.SuiteConfig.create(
        stop_on_failure=True,
        store_report=False,
        hipchat=hipchat
    )

    resources = models.SuiteResources.get_mock_object()
    failing_suite = unittest_utils.suites.FailingSuite(resources, config)

    def mock_response(
            self,
            status=200,
            raise_for_status=None):

        mock_resp = mock.Mock()
        mock_resp.raise_for_status = mock.Mock()
        if raise_for_status:
            mock_resp.raise_for_status.side_effect = raise_for_status
        mock_resp.status_code = status

        return mock_resp

    @mock.patch('camelot.driver._main.requests.post')
    def test_post_hipchat_method(self, mock_requests):

        mock_resp = self.mock_response()
        mock_requests.return_value = mock_resp

        response = _main._hipchat_notify(token=self.token,
                                         room_id_or_name=self.room_name,
                                         message=self.message,
                                         color=self.color)
        assert response == 200

        mock_req = mock.Mock()
        mock_req._main._hipchat_notify(token=self.token,
                                       room_id_or_name=self.room_name,
                                       message=self.message,
                                       color=self.color)

        mock_req._main._hipchat_notify.assert_called_with(token=self.token,
                                                          room_id_or_name=self.room_name,
                                                          message=self.message,
                                                          color=self.color)

    @mock.patch('camelot.driver._main.requests.post')
    def test_run_suite_from_module_with_httperror(self, mock_hipchat_notify):
        _main._hipchat_notify = mock.MagicMock(side_effect=HTTPError)
        _main._post_to_hipchat(self.failing_suite)

    @mock.patch('camelot.driver._main.requests.post')
    def test_run_suite_from_module_with_systemexit(self, mock_hipchat_notify):
        _main._hipchat_notify = mock.MagicMock(side_effect=SystemExit)
        with self.assertRaises(SystemExit):
            _main._post_to_hipchat(self.failing_suite)
