import json
import os
import unittest

import at2_sdk.at2
import mock
from mock import mock_open

from camelot.driver import _main

CWD = os.path.split(os.path.abspath(__file__))[0]
DATA_DIR = os.path.join(CWD, 'data')

with open(os.path.join(DATA_DIR, 'logs-task-with-camelot.json')) as f:
    LOGS_TASK_WITH_CAMELOT = json.load(f)

with open(os.path.join(DATA_DIR, 'logs-task-without-camelot.json')) as f:
    LOGS_TASK_WITHOUT_CAMELOT = json.load(f)

with open(os.path.join(DATA_DIR, 'logs-task-with-skipped-camelot.json')) as f:
    LOGS_TASK_WITH_SKIPPED_CAMELOT = json.load(f)


class TestLogs(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.sample_log = ['and I', 'am feeling', 'a little peculiar']

    @mock.patch.object(at2_sdk.at2.LegacyAt2Api, 'authenticate')
    @mock.patch.object(at2_sdk.at2.LegacyAt2Api, 'get_task_instance_by_id')
    @mock.patch('requests.Session.get')
    def test_get_logs_with_camelot(self, mock_get, mock_get_task_instance_by_id, mock_auth):
        m = mock_open()
        task = LOGS_TASK_WITH_CAMELOT
        mock_auth.return_value = True
        mock_response = mock.MagicMock()
        mock_get.return_value = mock_response
        mock_response.iter_lines.return_value = (n for n in self.sample_log)
        with mock.patch('camelot.driver._main.open', m, create=True):
            mock_get_task_instance_by_id.return_value = task
            _main.get_task_logs(task)
        expected_file_path = os.path.join(os.getcwd(), 'task_log_' +
                                          str(task['taskInstanceID']) + '.txt')
        m.assert_called_with(expected_file_path, 'w')
        handle = m()
        for l in self.sample_log:
            handle.write.has_calls(mock.call(l))

    @mock.patch.object(at2_sdk.at2.LegacyAt2Api, 'authenticate')
    @mock.patch.object(at2_sdk.at2.LegacyAt2Api, 'get_task_instance_by_id')
    def test_get_logs_without_camelot(self, mock_get_task_instance_by_id, mock_auth):
        m = mock_open()
        mock_auth.return_value = True
        task = LOGS_TASK_WITHOUT_CAMELOT
        with mock.patch('camelot.driver._main.open', m, create=True):
            mock_get_task_instance_by_id.return_value = task
            _main._download_task_logs(task)
            assert m.call_count == 0

    @mock.patch.object(at2_sdk.at2.LegacyAt2Api, 'authenticate')
    @mock.patch.object(at2_sdk.at2.LegacyAt2Api, 'get_task_instance_by_id')
    def test_get_logs_skipped_camelot(self, mock_get_task_instance_by_id, mock_auth):
        m = mock_open()
        mock_auth.return_value = True
        task = LOGS_TASK_WITH_SKIPPED_CAMELOT
        with mock.patch('camelot.driver._main.open', m, create=True):
            mock_get_task_instance_by_id.return_value = task
            _main._download_task_logs(task)
            assert m.call_count == 0
