import json
import os
import unittest

import mock
import pytest

from camelot import driver
from camelot.driver import _main
from valence import models

CWD = os.path.split(os.path.abspath(__file__))[0]
DATA_DIR = os.path.join(CWD, 'data')

with open(os.path.join(DATA_DIR, 'pull-resources.json')) as f:
    RESOURCES_DATA = json.load(f, encoding='utf-8')

with open(os.path.join(DATA_DIR, 'pull-task.json')) as f:
    TASK_DATA = json.load(f, encoding='utf-8')

with open(os.path.join(DATA_DIR, 'create-camelot-suite-resource-step.json')) as f:
    SUITE_RES_CONF = json.load(f, encoding='utf-8')


class TestPull(unittest.TestCase):
    def setUp(self):
        self.resources = RESOURCES_DATA
        self.task = TASK_DATA

    @pytest.mark.timeout(1)
    def test_get_resources(self):
        resources = _main._get_task_resources(self.task)
        assert len(resources.serialize()['nodes']) == len(self.resources['nodes'])

    @pytest.mark.timeout(1)
    def test_no_setup_cluster(self):
        for i, s in enumerate(self.task['taskInstanceSteps']):
            if s['stepName'] == 'setup_cluster':
                self.task['taskInstanceSteps'][i] = SUITE_RES_CONF
        resources = _main._get_task_resources(self.task)
        # technically Ips
        assert resources.serialize()['clusters'] == self.resources['clusters']

    @pytest.mark.timeout(1)
    def test_no_build(self):
        with open(os.path.join(DATA_DIR, 'pull-task-no_build.json')) as f:
            task = json.load(f)
        with open(os.path.join(DATA_DIR, 'pull-resources-no_build.json')) as f:
            expected = json.load(f)
        resources = _main._get_task_resources(task)

        assert len(resources.serialize()['nodes']) == len(expected['nodes'])

    @pytest.mark.timeout(1)
    def test_camelot(self):
        with open(os.path.join(DATA_DIR, 'pull-task-camelot-resources.json')) as f:
            task = json.load(f)
        with open(os.path.join(DATA_DIR, 'pull-camelot-resources.json')) as f:
            expected = json.load(f)
        resources = _main._get_task_resources(task)
        assert len(resources.serialize()['nodes']) == len(expected['nodes'])

    @pytest.mark.timeout(1)
    def test_networks(self):
        resources = _main._get_task_resources(self.task)
        node_dms = resources.node_dms
        networks = _main._get_networks(node_dms)
        assert isinstance(networks[0], models.NetworkSuiteResource)
        assert networks[0].model == node_dms[0].network_profile

    @mock.patch('valence.functional_models.Cluster.get_cluster_info')
    @mock.patch('valence.functional_models.Cluster.list_all_nodes')
    @mock.patch('valence.functional_models.Cluster.get_current_version')
    def test_get_docker_resources(self, mock_get_version, mock_list_nodes, mock_get_cluster):
        cluster_info = models.Cluster.get_mock_object()
        mock_get_cluster.return_value = cluster_info
        node = models.ActiveNode.get_mock_object()
        mock_list_nodes.return_value = [node]
        version = models.SfVersion('8.0')
        mock_get_version.return_value = version
        mvip = '0.0.0.0:10000'
        resources = driver.get_docker_resources(mvip)
        cluster_dm = resources.cluster_dms[0]
        assert cluster_dm.mvip == mvip
        assert cluster_dm.api_endpoint == version
        assert resources.node_dms[0].mip == node.mip
