#!/usr/bin/env python
"""Tool to filter Valence logs."""
from __future__ import print_function

import argparse
import sys

from camelot.driver import (
    _filter_log,
    _main,
)


def filter_and_create_logs(args):
    """Wrapper to run all the filters on the given log file or stream input.

    :param argparse.Namespace args: Parsed command line arguments
    """
    suite_config_dm = _main.get_config_dm(None)

    # When used as a pipe we check stdin for data and just apply the first log filter
    if not sys.stdin.isatty():
        # pylint: disable=unsubscriptable-object
        exclude_regexes = suite_config_dm.log_filters[0].exclude_regexes
        match_regexes = suite_config_dm.log_filters[0].match_regexes
        while True:
            line = sys.stdin.readline()
            if not line:
                break
            filtered_line = _filter_log.process_input_line(line, match_regexes=match_regexes,
                                                           exclude_regexes=exclude_regexes)
            if filtered_line:
                sys.stdout.write(filtered_line)
    else:
        for log_filter in suite_config_dm.log_filters:  # pylint: disable=not-an-iterable
            _filter_log.filter_and_create_log(log_filter, args.log)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Filter and create new camelot test logs')

    parser.add_argument('--log', '-l', help='The original log file')
    parser.set_defaults(func=filter_and_create_logs)

    arguments = parser.parse_args()
    arguments.func(arguments)
