"""Utilities and constants for posting to groot."""
import json
import traceback

import autologger
import requests

GROOT_URL = 'http://grootwdmysqlv01.one.den.solidfire.net:800'
REPORT_ENDPOINT = '/report'
CASES_ENDPOINT = '/cases'
SUITES_ENDPOINT = '/suites'
FAULTS_ENDPOINT = '/cluster_faults'

_log = autologger.get_sflogging(__name__)


def post_to_groot(location, payload):
    """Post a json payload to groot. Returns True on success and False on failure.

    The location should be something like ``cluster_faults/{case_instance_id}``

    :param str location: location to store the json record in groot.
    :param dict|list payload: the payload to store, must be a dictionary of only
        basic types (``json.dumps(payload)`` must work)
    """

    try:
        requests.post(
            GROOT_URL + location,
            data={'json_payload': json.dumps(payload)}
        )
    # Why so broad?  Because it shouldn't stop testing.
    # pylint: disable=broad-except
    except Exception:
        _log.debug('Exception raised when trying to post to Groot.  Please contact '
                   'Tim Jacobs to resolve.\nTRACEBACK:\n' + traceback.format_exc())
