"""ONTAP Data Model."""

import sf_models
from sf_models import types

from camelot.models import (
    at2_constants,
    at2_enums,
)


class ResourceOntap(sf_models.Base):
    """AT2 Ontap Resource Model."""
    username = types.StringType(required=True)
    password = types.StringType(required=True)
    cyclortp_username = types.StringType(serialized_name=at2_constants.CYCLORTP_USERNAME)
    cyclortp_password = types.StringType(serialized_name=at2_constants.CYCLORTP_PASSWORD)
    cluster_name = types.StringType(required=True)
    node_names = types.ListType(types.StringType, required=True,
                                serialized_name=at2_constants.NODE_NAMES)
    model = types.EnumType(at2_enums.OntapModel, required=True)
    resource_id = types.IntIdType(required=True, serialized_name=at2_constants.RESOURCE_ID)
    location_id = types.IntType(required=True, serialized_name=at2_constants.LOCATION_ID)
    node_management_port = types.EnumType(at2_enums.OntapPorts, required=True,
                                          serialized_name=at2_constants.NODE_MANAGEMENT_PORT)
    cluster_management_port = types.EnumType(at2_enums.OntapPorts, required=True,
                                             serialized_name=at2_constants.CLUSTER_MANAGEMENT_PORT)
    cluster_ports = types.StringListType(types.EnumType(at2_enums.OntapPorts), required=True,
                                         serialized_name=at2_constants.CLUSTER_PORTS)
    netmask = types.IPv4Type(required=True)
    cluster_lif = types.IPv4Type(required=True, serialized_name=at2_constants.CLUSTER_LIF)
    intercluster_lifs = types.StringListType(types.IPv4Type, required=True,
                                             serialized_name=at2_constants.INTERCLUSTER_LIFS)
    intercluster_ports = types.StringListType(types.EnumType(at2_enums.OntapPorts), required=True,
                                              serialized_name=at2_constants.INTERCLUSTER_PORTS)
    data_ports = types.StringListType(types.EnumType(at2_enums.OntapPorts), required=True,
                                      serialized_name=at2_constants.DATA_PORTS)
    node_lifs = types.StringListType(types.IPv4Type, required=True,
                                     serialized_name=at2_constants.NODE_LIFS)
    node_sps = types.StringListType(types.IPv4Type, required=True,
                                    serialized_name=at2_constants.NODE_SPS)
    svm_node_lifs = types.StringListType(types.IPv4Type, required=True,
                                         serialized_name=at2_constants.SVM_NODE_LIFS)

    resource_type_name = types.StringType(serialized_name=at2_constants.RESOURCE_TYPE_NAME)

    def __repr__(self):
        return '{}<resource_id={}, location_id={}, model={}>'.format(
            self.__class__.__name__, self.resource_id, self.location_id, self.model)
