import pytest

from camelot import _delay_exception
from valence import unittest_utils


class TestExceptionDelayer(unittest_utils.MockHistoryUnitTest):

    def setUp(self):
        super(TestExceptionDelayer, self).setUp()
        self.exc_delayer = _delay_exception.ExceptionDelayer('test')

    def test_delay_multiple_exceptions(self):
        with self.exc_delayer.delay_exception():
            _ = 1/0
        with self.exc_delayer.delay_exception():
            d = {}
            _ = d['not a key']
        output = self.exc_delayer.pretty_format()
        assert 'ZeroDivisionError' in output
        assert 'KeyError' in output

    def test_has_errors(self):
        assert not self.exc_delayer.has_errors()
        with self.exc_delayer.delay_exception():
            _ = 1/0
        assert self.exc_delayer.has_errors()

    def test_clear_errors(self):
        with self.exc_delayer.delay_exception():
            _ = 1/0
        assert self.exc_delayer.has_errors()
        self.exc_delayer.clear_errors()
        assert not self.exc_delayer.has_errors()

    def test_delay_specific_exception(self):
        with self.exc_delayer.delay_exception(exception_classes=(ZeroDivisionError, )):
            _ = 1/0
        with pytest.raises(KeyError):
            with self.exc_delayer.delay_exception(exception_classes=(ZeroDivisionError, )):
                d = {}
                _ = d['not a key']
