"""Test camelot.formatter."""
import os
import unittest

from camelot import _formatter
from valence import unittest_utils

DATA_DIR = unittest_utils.get_data_dir(__file__)

with open(os.path.join(DATA_DIR, 'passing_suite.log')) as log:
    passing_suite_log = log.read()


class TestFormatterRegexes(unittest.TestCase):
    """Test the regular expressions in formatter."""
    def test_log_step_re(self):
        matches = _formatter.RE_LOG_STEP.findall(passing_suite_log)
        # There's 2 steps in this log file
        assert len(matches) == 2

        # Match step 0
        match = _formatter.RE_LOG_STEP.search(passing_suite_log)
        assert match.group('index') == '0'
        assert match.group('name') == 'PassingCase'
        assert match.group('action') == 'Assert True'
        assert match.group('expect') == 'Pass'

    def test_suite_setup_re(self):
        matches = _formatter.RE_SUITE_SETUP.findall(passing_suite_log)
        assert len(matches) == 1
        # Suite name
        assert matches[0] == 'PassingSuite'

    def test_suite_teardown_re(self):
        matches = _formatter.RE_SUITE_TEARDOWN.findall(passing_suite_log)
        assert len(matches) == 1
        # Suite name
        assert matches[0] == 'PassingSuite'

    def test_case_test(self):
        match = _formatter.RE_TEST_CASE_TEST.search(passing_suite_log)
        assert match.group('name') == 'PassingCase'

    def test_case_setup(self):
        match = _formatter.RE_TEST_CASE_SETUP.search(passing_suite_log)
        assert match

    def test_case_teardown(self):
        match = _formatter.RE_TEST_CASE_TEARDOWN.search(passing_suite_log)
        assert match

    def test_case_status(self):
        match = _formatter.RE_TEST_CASE_STATUS.search(passing_suite_log)
        assert match.group('status') == 'passed'
        assert match.group('name') == 'PassingCase'
