import re
import unittest

from camelot import (
    _history,
    _record,
)


def divide_by_zero():
    h = _history.History('division')
    h.record('this is a function, awesome!')
    try:
        h.record('dividing by ZERO')
        x = 1 / 0
    except ZeroDivisionError:
        exc = _record.ExcInfo(history=h)
        exc.reraise()
    return x


def count_numbers():
    h = _history.History('counting numbers')
    h.record('1')
    h.record('2')
    h.record('3')
    try:
        divide_by_zero()
    except ZeroDivisionError:
        h.record("STILL can't divide by zero???", exc_info=True)
    return h


class TestHistory(unittest.TestCase):
    def test_create(self):
        h = _history.History('history')
        expected_pat = r'# NO HISTORY FOR history\[.*\]'
        assert re.match(expected_pat, h.pretty_fmt())

    def test_embed(self):
        h = _history.History('counting words')
        h.record('one', log=True)
        h.record('two')
        try:
            1/0
        except ZeroDivisionError:
            h.record("can't divide by zero???", exc_info=True)

        count_h = count_numbers()
        h.record('number counting', history=count_h)
        h.record('finished with counting I guess')
        h.pretty_fmt()
