"""Unittests for the test result."""

import datetime

import pytz

import camelot
from valence import models
from valence.unittest_utils import (
    helpers,
    suites,
)


class TestSuiteResult(helpers.MockHistoryUnitTest):

    def setUp(self):
        super(TestSuiteResult, self).setUp()
        self.config = models.SuiteConfig(dict(skip_indices=[2], stop_on_failure=False))
        self.resources = models.SuiteResources()
        self.passing_suite = suites.PassingTwoTestSuite(self.resources, self.config)
        self.multi_suite = suites.MultiErrorFailingSuite(self.resources, self.config)
        # suite does [pass, fail, pass(skipped), setup_error, teardown_error]

    def test_repr(self):
        self.passing_suite.execute()
        assert camelot.ResultStatus.passed.value.upper() in repr(self.passing_suite.result)

    def test_unique_result_on_execute(self):
        self.passing_suite.execute()
        first_result = self.passing_suite.result
        self.passing_suite.execute()
        assert first_result is not self.passing_suite.result

    def test_test_run_properties(self):
        self.multi_suite.execute()
        assert len(self.multi_suite.result.failed_test_runs) == 1
        assert len(self.multi_suite.result.skipped_test_runs) == 1
        assert len(self.multi_suite.result.errored_test_runs) == 1
        assert len(self.multi_suite.result.passed_test_runs) == 1
        assert len(self.multi_suite.result.pending_test_runs) == 1
        assert self.multi_suite.result.status is camelot.ResultStatus.error is self.multi_suite.status

    def test_reports(self):
        self.passing_suite.execute()
        self.multi_suite.execute()
        assert isinstance(self.passing_suite.result.generate_report(), dict)
        assert isinstance(self.multi_suite.result.generate_report(), dict)

    def test_no_endtime(self):
        result = camelot.SuiteResult('name', 'full-name', self.resources, self.config, [])
        result.start_time = datetime.datetime.now(pytz.utc)
        assert result.execution_time.total_seconds() == 0
