"""Ipython startup file for Valence.

To use, put this in ~/.ipython/profile_default/startup/ by using a symlink or copying the file.
"""

# pylint: skip-file

_valence_imported = False
try:
    import os
    if os.environ.get('VIRTUAL_ENV'):
        import valence
        _valence_imported = True
except ImportError:
    pass

if _valence_imported:
    from valence import camelot
    from camelot import driver
    from valence import enums
    from valence import functional_models
    from valence import models
    from valence import selectors
    from valence import vl_helpers
    from valence import vl_components
    import dev_utils


def setup_valence(resources_file=None, attempt_docker_sf_node=True):
    """Instantiates commonly used valence objects.

    :param str resources_file: Path to resources.json file. If None, look under the vl_data directory
         in the project directory (requires use of virtualenvwrapper).
    :param bool attempt_docker_sf_node: If True, use a docker-sfnode instance if available
    """
    global cluster_dms
    global cluster_fms
    global node_dms
    global node_fms
    global client_dms
    global client_fms
    global suite_resources

    # For single cluster/node/client configurations
    global cluster_dm
    global cluster_fm
    global node_dm
    global node_fm
    global client_dm
    global client_fm

    shell_init = dev_utils.ShellInitializer(resources_file=resources_file,
                                            attempt_docker_sf_node=attempt_docker_sf_node)

    cluster_fms = shell_init.cluster_fms
    cluster_dms = [fm.cluster_dm for fm in cluster_fms]
    node_fms = shell_init.node_fms
    node_dms = [fm.node_dm for fm in node_fms]
    client_fms = shell_init.client_fms
    client_dms = [fm.client_dm for fm in client_fms]

    # For single cluster/node/client configurations
    if len(cluster_dms) > 0:
        cluster_fm = cluster_fms[0]
        cluster_dm = cluster_fm.cluster_dm
    if len(node_dms) > 0:
        node_fm = node_fms[0]
        node_dm = node_fm.node_dm
    if len(client_dms) > 0:
        client_fm = client_fms[0]
        client_dm = client_fm.client_dm

    suite_resources = shell_init.suite_resources
    if not suite_resources:
        dev_utils.print_red('WARNING: No resources.json found!')


if _valence_imported:
    print('\nCurrent python path is using the following...')
    print('Valence: {}'.format(valence.__path__[0]))
    setup_valence()

del _valence_imported
