# Aliases and shell functions. To use, source this file in your .bashrc or equivalent file.

# Detect OS
uname | grep -q "Darwin"
if [ $? -eq 0 ]; then
    ON_MAC=1
else
    ON_MAC=0
fi

# Simulate pbpaste/pbcopy when not on Mac
if [[ $ON_MAC == 0 ]];
then
    alias pbcopy="xclip -selection clipboard"
    alias pbpaste="xclip -selection clipboard -o"
fi

pcd() {
    # Change to the folder of a python package.
    # Usage: pcd valence.models
    py_path=`python -c "import $1, os; print(os.path.dirname($1.__file__))" |grep -v DEBUG 2>/dev/null`
	if [ $? -ne 0 ]; then
		echo "$1 is not a python module"
	else
		echo "Changing to $py_path"
		cd $py_path
	fi
}

rmproject() {
    # Removes a project and associated virtualenvs. Opposite of mkproject.
    # Usage: rmproject project_name
    project_name=$1
    project_dir=$PROJECT_HOME/$project_name
    if [ $project_name ]; then
        echo "Running: rm -rf $project_dir"
        rm -rf $project_dir
        echo "Running: rmvirtualenv $project_name"
        rmvirtualenv $project_name
        rmvirtualenv $project_name-py3
        echo "Running: rmvirtualenv $project_name-py3"
    else
        echo "Please provide a project name"
    fi
}

pywhich() {
    # Print location of python package or module
    # Usage: pywhich valence_setup
    python -c "import $1; print($1.__file__)"
}

until_fail() {
    # Run a command in a loop until it exits nonzero
    # Usage: until_fail py.test
	$@
	while [ $? -eq 0 ]; do
		$@
	done
}

# Oh-My-Zsh like Git Aliases
alias gfm="git fetch origin master:master"  # Shortcut for git checkout master; git pull; git checkout -
alias gfr="git fetch origin release:release"  # Shortcut for git checkout release; git pull; git checkout -
alias gam="git commit -a --amend --no-edit --date=now"  # Amend previous commit without editing the message

# Mercurial
alias hl="hg pull -u"
