"""Node API Module."""
from sf_api import (
    _base,
    api_constants,
)


class NodeApi(_base.BaseJsonRpcClient):
    """SolidFire Node API Generic class.

    :param str mip: The management IP of Node
    :param str username: username of a node user, e.g "admin"
    :param str password: password of a node user, e.g "admin"
    :param sf_api.SfVersion version: version of the node
    :param bool authenticate: If true, authenticate resource in constructor (default=True)
    :param typing.Iterable[Any] retry_sf_exceptions_override: SolidFire exceptions that all
        methods will retry on. If provided, this will override the retry faults for all methods.
    :param int retry_count: Number of times to retry before raising an exception
    :param int port: Optional port number
    """

    def __init__(self, mip, username=None, password=None, version=None, authenticate=True,
                 retry_sf_exceptions_override=None, retry_count=2, port=api_constants.NODE_API_PORT):
        super(NodeApi, self).__init__(mip, username, password, version, authenticate,
                                      retry_sf_exceptions_override=retry_sf_exceptions_override,
                                      retry_count=retry_count, port=port)

    @property
    def mip(self):
        return self.ip
