"""Constants of SolidFire exceptions to retry on."""

from sf_api import (
    api_errors,
    enums,
)

# Generally nonfatal API method faults, due to transient Zookeeper error
SF_XDB_RETRIES = (
    api_errors.DBOperationTimeout,
    api_errors.DBConnectionLoss,
    api_errors.DBPathExists,
    api_errors.DBVersionMismatch,
    api_errors.DBSessionExpired,
    api_errors.DBSessionMoved,
    api_errors.DBClosing,
    api_errors.DBInvalidState,
    api_errors.DBMultiCommitFailed,
    api_errors.DaemonStatusChangeFailed
)

# Faults that should trigger a retry of an API method for only specific methods

_SNAPSHOT_TRANSIENT_EXCEPTIONS = (api_errors.NotReadyForIo, api_errors.RecvTimeout,
                                  api_errors.SnapshotFailed, api_errors.NoHandler,
                                  api_errors.IsNotPrimary, api_errors.NotPrimary,
                                  api_errors.FreezeTimeout)

cm = enums.ClusterMethod

METHOD_RETRY_SF_EXCEPTIONS = {
    # TODO: uncomment these methods as they are added
    # cm.compute_volume_checksum: (errors.NotReadyForIo, errors.RecvTimeout),
    # cm.read_slice_lba: (errors.RecvTimeout,),
    # cm.read_slice_lba_checksum: (errors.RecvTimeout,),
    cm.clone_volume: _SNAPSHOT_TRANSIENT_EXCEPTIONS,
    cm.create_snapshot: _SNAPSHOT_TRANSIENT_EXCEPTIONS,
    # cm.rollback_to_snapshot: _SNAPSHOT_TRANSIENT_EXCEPTIONS,
    cm.create_group_snapshot: _SNAPSHOT_TRANSIENT_EXCEPTIONS,
    # cm.rollback_to_group_snapshot: _SNAPSHOT_TRANSIENT_EXCEPTIONS,
    # cm.clone_multiple_volumes: _SNAPSHOT_TRANSIENT_EXCEPTIONS,
    # cm.start_bulk_volume_read: _SNAPSHOT_TRANSIENT_EXCEPTIONS,
    # cm.start_bulk_volume_write: _SNAPSHOT_TRANSIENT_EXCEPTIONS,
    # cm.set_network_config: (errors.NetworkConfigCommitFailed,),
    # cm.get_ensemble_config: (errors.DBRuntimeInconsistency, errors.DBConnectionLoss)
    cm.list_cluster_faults: (api_errors.Shutdown,),
}
