"""Enums for SolidFire API."""
import sf_models
from sf_models import at2_enums
from sf_models.enums import *  # pylint: disable=unused-wildcard-import,wildcard-import

from sf_api import (
    api_constants,
    sf_version,
)

# Backward compatibility
BondMode = at2_enums.BondMode


class AccountStatus(sf_models.ApiEnum):
    """Used by status field in Account data model."""
    active = api_constants.ACTIVE
    locked = api_constants.LOCKED
    removed = api_constants.REMOVED


class AccountType(sf_models.ApiEnum):
    """Used by account_type field on Account data model."""
    standard = api_constants.STANDARD
    vvol = api_constants.VVOL


class ApiExceptionType(aenum.Enum):
    """Standard API exception types."""
    param_conflict = 'apiParamConflict'
    param_enum_invalid = 'apiParamEnumInvalid'
    param_exceeded_max = 'apiParamExceededMax'
    param_exceeded_min = 'apiParamExceededMin'
    param_invalid = 'apiParamInvalid'
    param_missing = 'apiParamMissing'
    param_null = 'apiParamNull'
    param_type_invalid = 'apiParamTypeInvalid'
    param_value_does_not_exist = 'apiParamValueDoesNotExist'
    param_value_invalid = 'apiParamValueInvalid'


class ApiTestResult(sf_models.ApiEnum):
    """Used by result field on various [Test...] data models in the node API."""
    failed = 'Failed'
    passed = 'Passed'
    success = 'Success'

    def is_passed(self):
        """Helper method to determine if enum is a passed value.

        This makes it easy to check if the enum is passed without needing to import the enums package.

        :rtype: bool
        """
        return self is self.passed or self is self.success


class AsyncResultStatus(sf_models.ApiEnum):
    """Used by status field on AsyncResult data model."""
    complete = api_constants.COMPLETE
    failed = api_constants.FAILED
    running = api_constants.RUNNING


class AsyncResultType(sf_models.ApiEnum):
    """Used by result_type field on AsyncResult data model."""
    bs_check = 'BSCheck'
    clone = 'Clone'
    drive_add = 'DriveAdd'
    drive_removal = 'DriveRemoval'
    drive_secure_erase = 'DriveSecureErase'
    fake = 'Fake'  # Fake AsyncResultType for AsyncHandler
    fill_slice = 'FillSlice'
    garbage_collection = 'GarbageCollection'
    group_clone = 'GroupClone'
    rtfi_pending_node = 'RtfiPendingNode'
    set_cluster_structure = 'SetClusterStructure'


class AuthConfigType(ApiEnum):
    """Defines possible config data which will be accessed or modified by the element auth container."""
    element = 'element'
    mnode = 'mNode'


class BinStatus(sf_models.ApiEnum):
    """Used to determine what state a bin assignment is in."""
    active = 'bsActive'
    pending_active = 'bsPendingActive'
    pending_removal_active = 'bsPendingRemovalActive'
    pending_removal_dead = 'bsPendingRemovalDead'
    pending_removal_dead_from_active = 'bsPendingRemovalDeadFromActive'
    updating = 'bsUpdating'
    updating_no_sync = 'bsUpdatingNoSync'
    updating_from_active = 'bsUpdatingFromActive'


class BiosVendor(sf_models.ApiEnum):
    """Defines the available vendors used by SolidFire."""
    netapp = 'NetApp'
    solidfire = 'SolidFire'
    ami = 'American Megatrends Inc.'


class BondFailOverMac(sf_models.ApiEnum):
    """Used by bond-fail_over_mac in NetworkInterfaceDetails data model."""
    active = 'Active'
    none = 'None'


class BondReselectStatus(sf_models.ApiEnum):
    """Used for the bond_primary_reselect field in NetworkInterfaceDetails data model."""
    failure = 'Failure'


class BulkVolumeJobType(sf_models.ApiEnum):
    """Used by type field on BulkVolumeJob data model."""
    read = 'read'
    write = 'write'


class ChassisType(sf_models.ApiEnum):
    """Used by platformInfo model (used in node models).

    C220M4 are sanitas/cisco nodes
    """
    C220M4 = 'C220M4'
    QS52B = 'QS52B'
    R620 = 'R620'
    R630 = 'R630'
    SFVIRT = 'SFVIRT'
    SMBigTwin = 'SMBigTwin'


class ClusterAccessLevel(sf_models.ApiEnum):
    """Used by the access field on the ClusterAdmin data model."""
    accounts = 'accounts'
    administrator = 'administrator'
    cluster_admins = 'clusterAdmins'
    drives = 'drives'
    nodes = 'nodes'
    read = 'read'
    reporting = 'reporting'
    repositories = 'repositories'
    volumes = 'volumes'
    write = 'write'


class ClusterAuthMethod(sf_models.ApiEnum):
    """Used by the auth_method field on the ClusterAdmin data model."""
    cluster = 'Cluster'
    ldap = 'Ldap'
    idp = 'Idp'


class ClusterConstants(sf_models.ApiEnum):
    """ClusterConstants enumeration."""
    access_token_duration = 'cAccessTokenDuration'
    allow_drive_type_modification_of_slot_num = 'cAllowDriveTypeModificationOfSlotNum'
    artificial_memory_severity = 'cArtificialMemorySeverity'
    artificial_memory_usage_bytes = 'cArtificialMemoryUsageBytes'
    auth_session_inactivity_duration = 'cAuthSessionInactivityDuration'
    bin_sync_headroom_percent = 'cBinSyncHeadroomPercent'
    bin_sync_new_service_write_mib_per_second = 'cBinSyncNewServiceWriteMiBPerSecond'
    bin_sync_write_mib_per_second = 'cBinSyncWriteMiBPerSecond'
    block_service_format_override = 'cBServiceFormatOverride'
    bs_check_interval_seconds = 'cBSCheckIntervalSeconds'
    bs_check_pause_msec = 'cBSCheckPauseMSec'
    bs_first_time_write_ratio = 'cBSFirstTimeWriteRatio'
    bs_incorporate_unusable_space_in_used_bytes = 'cBSIncorporateUnusableSpaceInUsedBytes'
    bs_op_queue_read_threads = 'cBSOpQueueReadThreads'
    bs_recycle_at_pct = 'cBSRecycleAtPct'
    bs_unusable_space_percent = 'cBSUnusableSpacePercent'
    cluster_bs_check_mode = 'cClusterBSCheckMode'
    convert_to_paging_debounce_delay_seconds = 'cConvertToPagingDebounceDelaySeconds'
    disable_bin_assignment_updates = 'cDisableBinAssignmentUpdates'
    disable_pcache_clean_files = 'cDisablePCacheCleanFiles'
    dont_cache_ear_auth_keys = 'cDontCacheEARAuthKeys'
    duplicated_metadata_on_disk = 'cDuplicatedMetadataOnDisk'
    enable_auto_install = 'cEnableAutoInstall'
    enable_configurable_protection_domains = 'cEnableConfigurableProtectionDomains'
    enable_protection_domains = 'cEnableMultiNodeZones'
    enable_space_based_rebalancing = 'cEnableSpaceBasedRebalancing'
    enable_suppression_for_bin_assignment_events = 'cEnableSuppressionForBinAssignmentEvents'
    enable_updating_no_sync = 'cEnableUpdatingNoSync'
    event_slice_lbas_max_mismatches = 'cEventSliceLBAsMaxMismatches'
    force_bin_sync_without_balance_check = 'cForceBinSyncWithoutBalanceCheck'
    force_paging_sublist_count = 'cForcePagingSublistCount'
    force_python_bin_assignment_algorithm = 'cForcePythonBinAssignmentAlgorithm'
    gc_interval = 'cGCInterval'
    gc_status_polling_interval = 'cGCStatusPollingInterval'
    include_custom_pd_in_api = 'cIncludeCustomPDInPublicApiResults'
    max_auth_sessions = 'cMaxAuthSessionsPerUser'
    max_auth_sessions_per_user = 'cMaxAuthSessionsForCluster'
    max_time_between_successful_gcs_without_fault = 'cMaxTimeBetweenSuccessfulGCsWithoutFault'
    max_volumes_per_multi_api = 'cMaxVolumesPerMultiAPI'
    merkle_check_time = 'cMerkleCheckTime'
    merkle_checker_every_time = 'cMerkleCheckEveryTime'
    merkle_idle_cycles = 'cMerkleIdleCycles'
    panic_on_mismatched_lbas = 'cPanicOnMismatchedLbas'
    pcache_enable_active_file_flush = 'cPCacheEnableActiveFileFlush'
    pcache_ignore_min_flush_size_interval_seconds = 'cPCacheIgnoreMinFlushSizeIntervalSeconds'
    scheduler_interval_seconds = 'cSchedulerIntervalSec'
    set_bs_op_queue_read_threads_based_on_bin_coupling = 'cSetBSOpQueueReadThreadsBasedOnBinCoupling'
    skip_gc = 'cSkipGC'
    slice_balance_enable_auto_timer_iops_rebalancing = 'cSliceBalanceEnableAutoTimerIopsRebalancing'
    ss_health_monitor_interval_for_permanence = 'cSSHealthMonitorHealthSamplesConsideredPermanent'
    ssl_certificate_experation_warning_days = 'cSslCertificateExpirationWarningDays'
    start_merkle_check_from_qos = 'cStartMerkleCheckFromQos'
    time_until_bs_is_killed_msec = 'cTimeUntilBSIsKilledMSec'
    usable_drive_capacity_bytes = 'cUsableDriveCapacityBytes'
    usable_drive_capacity_percent = 'cUsableDriveCapacityPercent'
    use_metadata_full_threshold_algorithm = 'cUseMetadataFullThresholdAlgorithm'

    @classmethod
    def fields_in_version(cls, version):
        """Returns the list of enum fields valid for the specified version.

        :param sf_models.SfVersion version: SolidFire API version.
        :rtype: list
        """
        fields = super(ClusterConstants, cls).fields_in_version(version)
        if version < sf_version.SfVersion('11.0'):
            fields.remove(cls.artificial_memory_severity)
            fields.remove(cls.block_service_format_override)
            fields.remove(cls.force_paging_sublist_count)
            fields.remove(cls.pcache_enable_active_file_flush)
            fields.remove(cls.pcache_ignore_min_flush_size_interval_seconds)
        if version < sf_version.SfVersion('11.3'):
            fields.remove(cls.access_token_duration)
            fields.remove(cls.include_custom_pd_in_api)
        if version < sf_version.SfVersion('11.7'):
            fields.remove(cls.bs_first_time_write_ratio)
            fields.remove(cls.dont_cache_ear_auth_keys)
        if version < sf_version.SfVersion('12.0'):
            fields.remove(cls.convert_to_paging_debounce_delay_seconds)
            fields.remove(cls.duplicated_metadata_on_disk)
            fields.remove(cls.enable_suppression_for_bin_assignment_events)
            fields.remove(cls.force_python_bin_assignment_algorithm)
            fields.remove(cls.gc_status_polling_interval)
            fields.remove(cls.max_time_between_successful_gcs_without_fault)
            fields.remove(cls.auth_session_inactivity_duration)
            fields.remove(cls.max_auth_sessions)
            fields.remove(cls.max_auth_sessions_per_user)
            fields.remove(cls.usable_drive_capacity_bytes)
            fields.remove(cls.usable_drive_capacity_percent)
            fields.remove(cls.use_metadata_full_threshold_algorithm)
        return fields


class ClusterEventMessage(sf_models.ApiEnum):
    """The message field on ClusterEvent data model.

    The data model still uses StringType for message and does not validate against this enum.
    """
    unhealthy_block_service_added = 'Unhealthy block service added'
    skipping_bin_assignment_and_clearing_kill_services = (
        'Skipping Bin Assignments and clearing kill services: '
        'at least one bin has no active copies')
    block_data_assignment_change_detected = 'Block data assignment change detected. '


class ClusterEventType(sf_models.ApiEnum):
    """Used by event_info_type field on ClusterEvent data model."""
    api_event = 'apiEvent'
    bin_assignments_event = 'binAssignmentsEvent'
    bin_sync_event = 'binSyncEvent'
    bs_check_even = 'bsCheckEvent'
    bs_kill_event = 'bSKillEvent'
    bulk_op_event = 'bulkOpEvent'
    clone_event = 'cloneEvent'
    cluster_event = 'clusterEvent'
    cluster_master_event = 'clusterMasterEvent'
    csum_event = 'cSumEvent'
    data_event = 'dataEvent'
    db_event = 'dbEvent'
    drive_event = 'driveEvent'
    encryption_at_rest_event = 'encryptionAtRestEvent'
    ensemble_event = 'ensembleEvent'
    fault_checker_event = 'faultCheckerEvent'
    fc_event = 'fibreChannelEvent'
    gc_event = 'gcEvent'
    ie_event = 'iEEvent'
    install_event = 'installEvent'
    iscsi_event = 'iSCSIEvent'
    limit_event = 'limitEvent'
    network_event = 'networkEvent'
    platform_hardware_event = 'platformHardwareEvent'
    pre_startup_event = 'preStartupEvent'
    remote_cluster_event = 'remoteClusterEvent'
    scheduler_event = 'schedulerEvent'
    service_event = 'serviceEvent'
    slice_event = 'sliceEvent'
    snmp_trap_event = 'snmpTrapEvent'
    stat_event = 'statEvent'
    ts_event = 'tSEvent'
    unexpected_exception = 'unexpectedException'
    upgrade_event = 'upgradeEvent'
    vasa_provider_event = 'vasaProviderEvent'


class ClusterFaultCode(sf_models.ApiEnum):
    """Used by code field on ClusterFault data model.

    Constants not defined in sf_api.api_constants.py because we should only reference these via enum.
    """
    authentication_service_fault = 'authenticationServiceFault'
    available_virtual_network_ip_addresses_low = 'availableVirtualNetworkIPAddressesLow'
    bad_encryption_at_rest_state = 'badEncryptionAtRestState'
    block_cluster_full = 'blockClusterFull'
    block_service_too_full = 'blockServiceTooFull'
    block_service_unhealthy = 'blockServiceUnhealthy'
    blocks_degraded = 'blocksDegraded'
    bmc_sel_ecc_error = 'bmcSelEccError'
    cluster_cannot_sync = 'clusterCannotSync'
    cluster_full = 'clusterFull'
    cluster_iops_are_over_provisioned = 'clusterIOPSAreOverProvisioned'
    disable_drive_security_failed = 'disableDriveSecurityFailed'
    disconnected_cluster_pair = 'disconnectedClusterPair'
    disconnected_remote_node = 'disconnectedRemoteNode'
    drive_available = 'driveAvailable'
    drive_failed = 'driveFailed'
    drive_wear_fault = 'driveWearFault'
    duplicate_cluster_master_candidates = 'duplicateClusterMasterCandidates'
    enable_drive_security_failed = 'enableDriveSecurityFailed'
    ensemble_degraded = 'ensembleDegraded'
    exception = 'exception'
    failed_space_too_full = 'failedSpaceTooFull'
    fan_sensor = 'fanSensor'
    fibre_channel_access_degraded = 'fibreChannelAccessDegraded'
    fibre_channel_access_unavailable = 'fibreChannelAccessUnavailable'
    fibre_channel_config = 'fibreChannelConfig'
    file_system_capacity_low = 'fileSystemCapacityLow'
    fips_drives_mismatch = 'fipsDrivesMismatch'
    fips_drives_out_of_compliance = 'fipsDrivesOutOfCompliance'
    fips_self_test_failure = 'fipsSelfTestFailure'
    hardware_config_mismatch = 'hardwareConfigMismatch'
    idp_certificate_expiration = 'IdpCertificateExpiration'
    inconsistent_bond_modes = 'inconsistentBondModes'
    inconsistent_interface_configuration = 'inconsistentInterfaceConfiguration'
    inconsistent_mtus = 'inconsistentMtus'
    inconsistent_routing_rules = 'inconsistentRoutingRules'
    inconsistent_subnet_masks = 'inconsistentSubnetMasks'
    incorrect_bond_port_count = 'incorrectBondPortCount'
    invalid_configured_fibre_channel_node_count = 'invalidConfiguredFibreChannelNodeCount'
    irq_balance_failed = 'irqBalanceFailed'
    kmip_certificate_fault = 'kmipCertificateFault'
    kmip_server_fault = 'kmipServerFault'
    memory_usage_threshold = 'memoryUsageThreshold'
    metadata_cluster_full = 'metadataClusterFull'
    mtu_check_failure = 'mtuCheckFailure'
    network_config = 'networkConfig'
    network_errors_exceed_threshold = 'networkErrorsExceedThreshold'
    no_available_virtual_network_ip_addresses = 'noAvailableVirtualNetworkIPAddresses'
    node_hardware_fault = 'nodeHardwareFault'
    node_offline = 'nodeOffline'
    not_using_lacp_bond_mode = 'notUsingLACPBondMode'
    ntp_server_unreachable = 'ntpServerUnreachable'
    ntp_time_not_in_sync = 'ntpTimeNotInSync'
    nvram_device_status = 'nvramDeviceStatus'
    power_supply_error = 'powerSupplyError'
    provisioned_space_too_full = 'provisionedSpaceTooFull'
    remote_rep_async_delay_exceeded = 'remoteRepAsyncDelayExceeded'
    remote_rep_cluster_full = 'remoteRepClusterFull'
    remote_rep_snapshot_cluster_full = 'remoteRepSnapshotClusterFull'
    remote_rep_snapshots_exceeded_limit = 'remoteRepSnapshotsExceededLimit'
    schedule_action_error = 'scheduleActionError'
    sensor_reading_failed = 'sensorReadingFailed'
    service_not_running = 'serviceNotRunning'
    slice_service_too_full = 'sliceServiceTooFull'
    slice_service_unhealthy = 'sliceServiceUnhealthy'
    ssh_enabled = 'sshEnabled'
    ssl_certificate_expiration = 'sslCertificateExpiration'
    stranded_capacity = 'strandedCapacity'
    temp_sensor = 'tempSensor'
    time_since_last_full_gc_exceeded_limit = 'timeSinceLastFullGCExceededLimit'
    unbalanced_mixed_nodes = 'unbalancedMixedNodes'
    unresponsive_service = 'unresponsiveService'
    upgrade = 'upgrade'
    virtual_network_config = 'virtualNetworkConfig'
    volumes_degraded = 'volumesDegraded'
    volumes_offline = 'volumesOffline'

    @classmethod
    def fields_in_version(cls, version):
        """Returns the list of enum fields valid for the specified version.

        :param sf_models.SfVersion version: SolidFire API version.
        :rtype: list
        """
        fields = super(ClusterFaultCode, cls).fields_in_version(version)
        if version > sf_version.SfVersion('11.0'):
            fields.remove(cls.unbalanced_mixed_nodes)
        return fields


class ClusterFaultSeverity(sf_models.ApiEnum):
    """Used by the severity field on ClusterFault data model."""
    best_practice = api_constants.BEST_PRACTICE
    critical = api_constants.CRITICAL
    error = api_constants.ERROR
    warning = api_constants.WARNING


class ClusterFaultStatus(sf_models.ApiEnum):
    """Used for faultTypes argument in ListClusterFaults."""
    all = 'all'
    current = 'current'
    resolved = 'resolved'


class ClusterFaultType(sf_models.ApiEnum):
    """Used by the type field on ClusterFault data model."""
    cluster = api_constants.CLUSTER
    drive = api_constants.DRIVE
    node = api_constants.NODE
    service = api_constants.SERVICE
    volume = api_constants.VOLUME


class ClusterHardwareInfoType(sf_models.ApiEnum):
    """Used for the hw_info parameter in GetClusterHardwareInfo."""
    all = 'all'
    drives = 'drives'
    nodes = 'nodes'


class ClusterMethod(sf_models.ApiEnum):
    """Methods that are part of the SolidFire Cluster API."""
    abort_gc = 'AbortGC'
    add_account = 'AddAccount'
    add_block_to_bs = 'AddBlockToBS'
    add_cluster_admin = 'AddClusterAdmin'
    add_idp_cluster_admin = 'AddIdpClusterAdmin'
    add_ldap_cluster_admin = 'AddLdapClusterAdmin'
    add_drives = 'AddDrives'
    add_initiators_to_volume_access_group = 'AddInitiatorsToVolumeAccessGroup'
    add_key_server_to_provider_kmip = 'AddKeyServerToProviderKmip'
    add_nodes = 'AddNodes'
    add_virtual_network = 'AddVirtualNetwork'
    add_volumes_to_volume_access_group = 'AddVolumesToVolumeAccessGroup'
    break_snapmirror_volume = 'BreakSnapMirrorVolume'
    break_snapmirror_volumes = 'BreakSnapMirrorVolumes'
    check_block = 'CheckBlock'
    check_proposed_node_additions = 'CheckProposedNodeAdditions'
    check_volume_standbys_assigned = 'CheckVolumeStandbysAssigned'
    clear_cluster_faults = 'ClearClusterFaults'
    clone_volume = 'CloneVolume'
    complete_cluster_pairing = 'CompleteClusterPairing'
    complete_volume_pairing = 'CompleteVolumePairing'
    compute_volume_checksum = 'ComputeVolumeChecksum'
    create_auth_session = 'CreateAuthSession'
    create_event = 'CreateEvent'
    create_group_snapshot = 'CreateGroupSnapshot'
    create_idp_auth_session = 'CreateIdpAuthSession'
    create_idp_configuration = 'CreateIdpConfiguration'
    create_initiators = 'CreateInitiators'
    create_key_provider_kmip = 'CreateKeyProviderKmip'
    create_key_server_kmip = 'CreateKeyServerKmip'
    create_multiple_volumes = 'CreateMultipleVolumes'
    create_public_private_key_pair = 'CreatePublicPrivateKeyPair'
    create_schedule = 'CreateSchedule'
    create_snapmirror_endpoint = 'CreateSnapMirrorEndpoint'
    create_snapshot = 'CreateSnapshot'
    create_volume = 'CreateVolume'
    create_volume_access_group = 'CreateVolumeAccessGroup'
    delete_auth_session = 'DeleteAuthSession'
    delete_auth_sessions_by_cluster_admin = 'DeleteAuthSessionsByClusterAdmin'
    delete_auth_sessions_by_username = 'DeleteAuthSessionsByUsername'
    delete_group_snapshot = 'DeleteGroupSnapshot'
    delete_idp_configuration = 'DeleteIdpConfiguration'
    delete_initiators = 'DeleteInitiators'
    delete_key_provider_kmip = 'DeleteKeyProviderKmip'
    delete_key_server_kmip = 'DeleteKeyServerKmip'
    delete_snapmirror_endpoint = 'DeleteSnapMirrorEndpoint'
    delete_snapshot = 'DeleteSnapshot'
    delete_volume = 'DeleteVolume'
    delete_volume_access_group = 'DeleteVolumeAccessGroup'
    delete_volumes = 'DeleteVolumes'
    demote_cluster_master = 'DemoteClusterMaster'
    disable_encryption_at_rest = 'DisableEncryptionAtRest'
    disable_idp_authentication = 'DisableIdpAuthentication'
    disable_standby_slice_assignments = 'DisableStandbySliceAssignments'
    enable_encryption_at_rest = 'EnableEncryptionAtRest'
    enable_feature = 'EnableFeature'
    enable_idp_authentication = 'EnableIdpAuthentication'
    enable_standby_slice_assignments = 'EnableStandbySliceAssignments'
    fill_slice = 'FillSlice'
    finish_upgrade = 'FinishUpgrade'
    get_account_by_id = 'GetAccountByID'
    get_active_tls_ciphers = 'GetActiveTlsCiphers'
    get_api = 'GetAPI'
    get_async_result = 'GetAsyncResult'
    get_auth_configuration = 'GetAuthConfiguration'
    get_client_certificate_sign_request = 'GetClientCertificateSignRequest'
    get_cluster_capacity = 'GetClusterCapacity'
    get_cluster_full_threshold = 'GetClusterFullThreshold'
    get_cluster_hardware_info = 'GetClusterHardwareInfo'
    get_cluster_info = 'GetClusterInfo'
    get_cluster_master_node_id = 'GetClusterMasterNodeID'
    get_cluster_stats = 'GetClusterStats'
    get_cluster_structure = 'GetClusterStructure'
    get_cluster_version_info = 'GetClusterVersionInfo'
    get_constants = 'GetConstants'
    get_database_entry = 'GetDatabaseEntry'
    get_default_qos = 'GetDefaultQoS'
    get_drive_hardware_info = 'GetDriveHardwareInfo'
    get_encryption_at_rest_info = 'GetEncryptionAtRestInfo'
    get_ensemble_config = 'GetEnsembleConfig'
    get_feature_status = 'GetFeatureStatus'
    get_fips_report = 'GetFipsReport'
    get_gc_status = 'GetGCStatus'
    get_hardware_info_overrides = 'GetHardwareInfoOverrides'
    get_idp_authentication_state = 'GetIdpAuthenticationState'
    get_immutable_values = 'GetImmutableValues'
    get_ipmi_info = 'GetIpmiInfo'
    get_key_provider_kmip = 'GetKeyProviderKmip'
    get_key_server_kmip = 'GetKeyServerKmip'
    get_limits = 'GetLimits'
    get_network_interface_status = 'GetNetworkInterfaceStatus'
    get_node_stats = 'GetNodeStats'
    get_ntp_info = 'GetNtpInfo'
    get_protection_domain_layout = 'GetProtectionDomainLayout'
    get_raw_stats = 'GetRawStats'
    get_service_status = 'GetServiceStatus'
    get_slice_info = 'GetSliceInfo'
    get_ssl_certificate = 'GetSSLCertificate'
    get_supported_tls_ciphers = 'GetSupportedTlsCiphers'
    get_volume_efficiency = 'GetVolumeEfficiency'
    list_accounts = 'ListAccounts'
    list_active_nodes = 'ListActiveNodes'
    list_active_paired_volumes = 'ListActivePairedVolumes'
    list_all_nodes = 'ListAllNodes'
    list_async_results = 'ListAsyncResults'
    list_active_auth_sessions = 'ListActiveAuthSessions'
    list_auth_sessions_by_cluster_admin = 'ListAuthSessionsByClusterAdmin'
    list_auth_sessions_by_username = 'ListAuthSessionsByUsername'
    list_cluster_admins = 'ListClusterAdmins'
    list_cluster_faults = 'ListClusterFaults'
    list_current_cluster_admins = 'ListCurrentClusterAdmins'
    list_cluster_pairs = 'ListClusterPairs'
    list_database_children_data = 'ListDatabaseChildrenData'
    list_drive_stats = 'ListDriveStats'
    list_drives = 'ListDrives'
    list_events = 'ListEvents'
    list_expired_auth_sessions = 'ListExpiredAuthSessions'
    list_group_snapshots = 'ListGroupSnapshots'
    list_idp_configurations = 'ListIdpConfigurations'
    list_initiators = 'ListInitiators'
    list_key_providers_kmip = 'ListKeyProvidersKmip'
    list_key_servers_kmip = 'ListKeyServersKmip'
    list_node_stats = 'ListNodeStats'
    list_pending_active_nodes = 'ListPendingActiveNodes'
    list_pending_nodes = 'ListPendingNodes'
    list_protection_domain_levels = 'ListProtectionDomainLevels'
    list_protection_domains = 'ListProtectionDomains'
    list_schedules = 'ListSchedules'
    list_services = 'ListServices'
    list_snapmirror_endpoints = 'ListSnapMirrorEndpoints'
    list_snapshots = 'ListSnapshots'
    list_sync_jobs = 'ListSyncJobs'
    list_virtual_networks = 'ListVirtualNetworks'
    list_volume_access_groups = 'ListVolumeAccessGroups'
    list_volume_qos_histograms = 'ListVolumeQoSHistograms'
    list_volume_stats = 'ListVolumeStats'
    list_volume_stats_by_volume = 'ListVolumeStatsByVolume'
    list_volumes = 'ListVolumes'
    modify_account = 'ModifyAccount'
    modify_cluster_admin = 'ModifyClusterAdmin'
    modify_cluster_full_threshold = 'ModifyClusterFullThreshold'
    modify_group_snapshot = 'ModifyGroupSnapshot'
    modify_initiator = 'ModifyInitiator'
    modify_initiators = 'ModifyInitiators'
    modify_key_server_kmip = 'ModifyKeyServerKmip'
    modify_schedule = 'ModifySchedule'
    modify_snapmirror_endpoint = 'ModifySnapMirrorEndpoint'
    modify_snapshot = 'ModifySnapshot'
    modify_virtual_network = 'ModifyVirtualNetwork'
    modify_volume = 'ModifyVolume'
    modify_volume_access_group = 'ModifyVolumeAccessGroup'
    modify_volume_pair = 'ModifyVolumePair'
    modify_volumes = 'ModifyVolumes'
    move_primaries_away_from_node = 'MovePrimariesAwayFromNode'
    promote_cluster_master = 'PromoteClusterMaster'
    purge_all_deleted_volumes = 'PurgeAllDeletedVolumes'
    purge_deleted_volume = 'PurgeDeletedVolume'
    purge_deleted_volumes = 'PurgeDeletedVolumes'
    read_slice_lba = 'ReadSliceLBA'
    read_slice_lba_checksum = 'ReadSliceLBAChecksum'
    reassign_slice = 'ReassignSlice'
    rebalance_slices = 'RebalanceSlices'
    recover_dead_volumes = 'RecoverDeadVolumes'
    remove_account = 'RemoveAccount'
    remove_block_from_bs = 'RemoveBlockFromBS'
    remove_cluster_admin = 'RemoveClusterAdmin'
    remove_cluster_pair = 'RemoveClusterPair'
    remove_drives = 'RemoveDrives'
    remove_initiators_from_volume_access_group = 'RemoveInitiatorsFromVolumeAccessGroup'
    remove_key_server_from_provider_kmip = 'RemoveKeyServerFromProviderKmip'
    remove_nodes = 'RemoveNodes'
    remove_ssl_certificate = 'RemoveSSLCertificate'
    remove_virtual_network = 'RemoveVirtualNetwork'
    remove_volume_pair = 'RemoveVolumePair'
    remove_volumes_from_volume_access_group = 'RemoveVolumesFromVolumeAccessGroup'
    reset_constants = 'ResetConstants'
    reset_supplemental_tls_ciphers = 'ResetSupplementalTlsCiphers'
    resize_drives = 'ResizeDrives'
    restore_deleted_volume = 'RestoreDeletedVolume'
    rollback_to_group_snapshot = 'RollbackToGroupSnapshot'
    rollback_to_snapshot = 'RollbackToSnapshot'
    secure_erase_drives = 'SecureEraseDrives'
    set_auth_configuration = 'SetAuthConfiguration'
    set_cluster_structure = 'SetClusterStructure'
    set_constants = 'SetConstants'
    set_database_entry = 'SetDatabaseEntry'
    set_hardware_info_overrides = 'SetHardwareInfoOverrides'
    set_ntp_info = 'SetNtpInfo'
    set_protection_domain_layout = 'SetProtectionDomainLayout'
    set_protection_domain_layout_chassis_override = 'SetProtectionDomainLayoutChassisOverride'
    set_protection_domains = 'SetProtectionDomains'
    set_ssl_certificate = 'SetSSLCertificate'
    set_supplemental_tls_ciphers = 'SetSupplementalTlsCiphers'
    shutdown = 'Shutdown'
    start_cluster_bs_check = 'StartClusterBSCheck'
    start_cluster_pairing = 'StartClusterPairing'
    start_gc = 'StartGC'
    start_upgrade = 'StartUpgrade'
    start_volume_pairing = 'StartVolumePairing'
    stop_cluster_bs_check = 'StopClusterBSCheck'
    test_key_provider_kmip = 'TestKeyProviderKmip'
    test_key_server_kmip = 'TestKeyServerKmip'
    update_auth_session = 'UpdateAuthSession'
    update_idp_configuration = 'UpdateIdpConfiguration'


class ClusterPairStatus(sf_models.ApiEnum):
    """Used by the status field on the ClusterPair data model."""
    connected = 'Connected'
    connecting = 'Connecting'
    disconnected = 'Disconnected'
    misconfigured = 'Misconfigured'
    unconfigured = 'Unconfigured'


class ClusterStage(sf_models.ApiEnum):
    """Used by fullness field on GetClusterFullThreshold result model."""
    stage1 = 'stage1Happy'
    stage2 = 'stage2Aware'
    stage3 = 'stage3Low'
    stage4 = 'stage4Critical'
    stage5 = 'stage5CompletelyConsumed'


class CompressionAlgorithm(sf_models.ApiEnum):
    """Used by compAlgorithim field for fill_slice."""
    deflate = 'csDEFLATE'
    lzf = 'caLZF'
    none = 'caNone'
    snappy = 'caSNAPPY'
    wkdm = 'caWKdm'


class DriveEncryptionCapability(sf_models.ApiEnum):
    """Used by encryption_capable field in DriveInfo data model."""
    fips = 'fips'
    none = 'none'
    sed = 'sed'


class DriveStatus(sf_models.ApiEnum):
    """Used by status field on Drive data model."""
    active = api_constants.ACTIVE
    assigned = 'assigned'
    available = api_constants.AVAILABLE
    erasing = api_constants.ERASING
    failed = api_constants.FAILED
    locked = api_constants.LOCKED
    removed = api_constants.REMOVED
    removing = api_constants.REMOVING


class DriveType(sf_models.ApiEnum):
    """Used by type field on Drive data model."""
    block = api_constants.BLOCK
    slice = api_constants.SLICE
    unknown = api_constants.UNKNOWN
    volume = api_constants.VOLUME


class EarState(sf_models.ApiEnum):
    """Used by encryption_as_rest_state field on Cluster data model."""
    enabled = api_constants.ENABLED
    enabling = api_constants.ENABLING
    disabled = api_constants.DISABLED
    disabling = api_constants.DISABLING


class EnsembleMode(sf_models.ApiEnum):
    """EnsembleMode enumeration.

    Corresponds to the mode object in the ClusterEnsemble data model.
    """
    quorum = 'Quorum'


class EventQueueType(sf_models.ApiEnum):
    """Used by event_queue_type field on ListEvents data model."""
    standard = 'Standard'
    vvol = 'Vvol'


class FibreChannelPortState(sf_models.ApiEnum):
    """Used by state field on FibreChannelPort data model."""
    blocked = 'Blocked'
    bypassed = 'Bypassed'
    deleted = 'Deleted'
    diagnostics = 'Diagnostics'
    error = 'Error'
    linkdown = 'Linkdown'
    loopback = 'Loopback'
    not_present = 'NotPresent'
    offline = 'Offline'
    online = 'Online'
    unknown = 'Unknown'


class FipsDrivesStatus(sf_models.ApiEnum):
    """Used to determine what FIPS 140-2 compliance status GetFipsReport API show."""
    none = 'None'
    partial = 'Partial'
    ready = 'Ready'


class GcProgressPhase(sf_models.ApiEnum):
    collect = 'collect'
    discard = 'discard'
    prepare = 'prepare'


class GcStatus(sf_models.ApiEnum):
    aborted = 'aborted'
    complete = 'complete'
    completed_partial = 'completed-partial'
    failed = 'failed'
    in_progress = 'in-progress'
    none_recorded = 'none-recorded'  # GC has not yet run on this cluster
    unknown = 'unknown'  # CM has changed since the last GC


class GroupSnapshotStatus(sf_models.ApiEnum):
    """Used for status field on group snapshots."""
    done = 'done'
    preparing = 'preparing'


class HardwareBaseUnit(sf_models.ApiEnum):
    """Used by base_unit field on HardwareInfo data model."""
    degrees_c = 'C'
    rpm = 'RPM'


class IOFormat(sf_models.ApiEnum):
    """IOFormat enumeration.

    Corresponds to the format object in the StartBulkVolumeRead/Write requests.
    """
    native = 'native'
    uncompressed = 'uncompressed'


class JsonReport(aenum.Enum):
    """Enum for cluster JSON reports."""
    bins = 'bins.json'
    bs_disk_data = 'bsdiskdata.json'
    fconv = 'fconv.json'
    flushing = 'flushing.json'
    index = 'index.json'
    qos = 'qos.json'
    scsi_commands = 'scsiCommands.json'
    scsi_nexuses = 'scsiNexuses.json'
    slice_balance = 'slicebalance.json'
    slice_filesystem = 'slicefilesystem.json'
    slices = 'slices.json'
    snapmirror = 'snapmirror.json'
    timing = 'timing.json'


class LdapAuthType(sf_models.ApiEnum):
    """LdapAuthType enumeration.

    Corresponds to the auth_type object in the LdapConfig data model.
    Part of the GetLdapConfiguration response.
    """
    direct_bind = 'DirectBind'
    search_and_bind = 'SearchAndBind'


class LdapSearchType(sf_models.ApiEnum):
    """LdapSearchType enumeration.

    Corresponds to the group_search_type object in the LdapConfig data model.
    Part of the GetLdapConfiguration response.
    """
    active_directory = 'ActiveDirectory'
    member_dn = 'MemberDN'
    no_groups = 'NoGroups'


class MemoryUsageSeverity(sf_models.ApiEnum):
    """Low memory severity values.  See cArtificialMemorySeverity node API constant."""
    critical = 3
    error = 2
    ok = 0
    warning = 1


class MnodeServiceUpdateStatus(sf_models.ApiEnum):
    not_started = 'not started'
    in_progress = 'in progress'
    success = 'success'


class NetworkStatus(sf_models.ApiEnum):
    """Used by status field on NodeNetworkInterface data model."""
    down = 'Down'
    up = 'Up'
    up_and_running = 'UpAndRunning'


class NetworkType(sf_models.ApiEnum):
    """Used by type field on NodeNetworkInterface data model."""
    bond_master = 'BondMaster'
    bond_slave = 'BondSlave'
    docker = 'Docker'
    loopback = 'Loopback'
    physical = 'Physical'
    virtual_bond_master = 'VirtualBondMaster'
    virtual_physical = 'VirtualPhysical'
    virtual_vlan = 'VirtualVlan'  # for virtual bonded network interfaces on the cluster master
    vlan = 'Vlan'                 # for virtual bonded network interfaces


class NodeBinary(sf_models.ApiEnum):
    """Binaries on the node.

    Most of these can be restarted via RestartServices api call.
    """
    sfapp = 'sfapp'
    sfbasiciocheck = 'sfbasiciocheck'
    sfconfig = 'sfconfig'
    sfnetwd = 'sfnetwd'
    sfsvcmgr = 'sfsvcmgr'
    solidfire = 'solidfire'


class NodeMethod(sf_models.ApiEnum):
    """Methods that are part of the SolidFire Node API."""
    check_proposed_cluster = 'CheckProposedCluster'
    create_cluster = 'CreateCluster'
    create_standalone_cluster = 'CreateStandaloneCluster'
    create_support_bundle = 'CreateSupportBundle'
    delete_all_support_bundles = 'DeleteAllSupportBundles'
    delete_downloads = 'DeleteDownloads'
    download = 'Download'
    get_api = 'GetAPI'
    get_bootstrap_config = 'GetBootstrapConfig'
    get_cluster_config = 'GetClusterConfig'
    get_cluster_state = 'GetClusterState'
    get_config_extended = 'GetConfigExtended'
    get_drive_config = 'GetDriveConfig'
    get_ensemble_config = 'GetEnsembleConfig'
    get_hardware_config = 'GetHardwareConfig'
    get_hardware_info = 'GetHardwareInfo'
    get_network_config = 'GetNetworkConfig'
    get_network_interface_status = 'GetNetworkInterfaceStatus'
    get_node_constants = 'GetNodeConstants'
    get_node_fips_drives_report = 'GetNodeFipsDrivesReport'
    get_node_ssl_certificate = 'GetNodeSSLCertificate'
    get_startup_flags = 'GetStartupFlags'
    get_version_info = 'GetVersionInfo'
    install_package = 'InstallPackage'
    list_available_nodes = 'ListAvailableNodes'
    list_downloads = 'ListDownloads'
    list_mounted_file_systems = 'ListMountedFileSystems'
    list_network_interfaces = 'ListNetworkInterfaces'
    list_package_info = 'ListPackageInfo'
    read_link = 'ReadLink'
    reboot_node = 'RebootNode'
    remove_node_ssl_certificate = 'RemoveNodeSSLCertificate'
    reset_network_config = 'ResetNetworkConfig'
    reset_node = 'ResetNode'
    reset_node_constants = 'ResetNodeConstants'
    restart_services = 'RestartServices'
    rotate_logs = 'RotateLogs'
    set_active_package_version = 'SetActivePackageVersion'
    set_cluster_config = 'SetClusterConfig'
    set_config = 'SetConfig'
    set_config_extended = 'SetConfigExtended'
    set_network_config = 'SetNetworkConfig'
    set_network_interface_status = 'SetNetworkInterfaceStatus'
    set_node_constants = 'SetNodeConstants'
    set_node_ssl_certificate = 'SetNodeSSLCertificate'
    set_startup_flags = 'SetStartupFlags'
    start_rtfi = 'StartRtfi'
    test_hardware_config = 'TestHardwareConfig'
    uninstall_package = 'UninstallPackage'


class NodeService(sf_models.ApiEnum):
    """Services that can be manipulated via the kill command."""
    block = 'block'
    master = 'master'
    slice = 'slice'

    @property
    def is_drive_service(self):
        return self in (self.slice, self.block)


class NodeState(sf_models.ApiEnum):
    """Used by state field on NodeClusterConfig data model."""
    active = 'Active'
    available = 'Available'
    pending = 'Pending'
    pending_active = 'PendingActive'


class ProposedClusterErrorCode(sf_models.ApiEnum):
    """Defines error codes for CheckProposedCluster and CheckProposedNodeAdditions."""
    nodes_cluster_member = 'nodesClusterMember'
    nodes_connect_failed = 'nodesConnectFailed'
    nodes_no_capacity = 'nodesNoCapacity'
    nodes_query_failed = 'nodesQueryFailed'
    nodes_too_large = 'nodesTooLarge'
    non_fips_drives_capable = 'nonFipsDrivesCapable'
    non_fips_node_capable = 'nonFipsNodeCapable'


class ProtectionDomainType(sf_models.ApiEnum):
    """Defines possible protection domain levels on the cluster."""
    chassis = 'chassis'
    custom = 'custom'
    node = 'node'


class ProtectionScheme(sf_models.ApiEnum):
    """Used by Volumes; added in 11.0."""
    double_helix = 'doubleHelix'
    single_helix = 'singleHelix'
    triple_helix = 'tripleHelix'


class RemoteClusterSnapshotStatus(sf_models.ApiEnum):
    """Used for status field on snapshot replication status."""
    deleted = 'Deleted'
    not_present = 'NotPresent'
    present = 'Present'
    syncing = 'Syncing'
    unknown = 'Unknown'


class RemoteRepMode(sf_models.ApiEnum):
    """Defines different remote replication modes.

    Used by mode field on VolumePairRemoteReplication data model.
    """
    async_ = api_constants.ASYNC
    snapshots_only = api_constants.SNAPSHOTS_ONLY
    sync = api_constants.SYNC


class RemoteRepState(sf_models.ApiEnum):
    """Remove replication states.

    Used by SnapshotReplication and VolumePairRemoteReplication data models.
    """
    active = 'Active'
    idle = 'Idle'
    paused_cluster_full = 'PausedClusterFull'
    paused_disconnected = 'PausedDisconnected'
    paused_exceeded_max_snapshot_count = 'PausedExceededMaxSnapshotCount'
    paused_manual = 'PausedManual'
    paused_manual_remote = 'PausedManualRemote'
    paused_misconfigured = 'PausedMisconfigured'
    paused_qos = 'PausedQoS'
    paused_slow_link = 'PausedSlowLink'
    resuming_connected = 'ResumingConnected'
    resuming_data_transfer = 'ResumingDataTransfer'
    resuming_local_sync = 'ResumingLocalSync'
    resuming_rr_sync = 'ResumingRRSync'
    stopped_misconfigured = 'StoppedMisconfigured'
    unconfigured = 'Unconfigured'


class RestartServicesAction(sf_models.ApiEnum):
    """Used by action parameter of RestartServices call."""
    restart = 'restart'
    start = 'start'
    stop = 'stop'


class ScheduleStatus(sf_models.ApiEnum):
    """Used for the lastRunStatus field on a schedule."""
    failed = 'Failed'
    failure = 'Failure'
    success = 'Success'


class ScheduleType(sf_models.ApiEnum):
    """Used for schedule type field on a schedule.

    Only snapshot is supported at this time.
    """
    snapshot = 'Snapshot'


class ScsiState(sf_models.ApiEnum):
    """Used by scsi_state field on DriveInfo data model."""
    running = 'Running'
    unknown = 'Unknown'


class ServiceFormat(sf_models.ApiEnum):
    """Used to specify format of block services."""
    block_service_paging = 'BlockServicePaging'
    invalid = 'Invalid'
    standard = 'Standard'


class ServiceStatus(sf_models.ApiEnum):
    """Used by status field on Service data model."""
    decommissioned = 'decommissioned'
    healthy = 'healthy'
    migrating = 'migrating'
    migrating_healthy = 'migratingHealthy'
    migrating_immediate = 'migratingImmediate'
    unhealthy = 'unhealthy'


class ServiceType(sf_models.ApiEnum):
    """Used by service_type field on Service data model."""
    block = 'block'
    bulk_volume = 'bulkvolume'
    master = 'master'
    slice = 'slice'
    transport = 'transport'
    volume = 'volume'

    @property
    def is_drive_service(self):
        return self in (self.block, self.volume, self.slice)


class SfException(sf_models.ApiEnum):
    """Exception types returned by the SolidFire API.

    https://solidfire.fogbugz.com/f/cases/22763/Consolidate-exceptions-and-make-them-well-defined
    """
    account_id_does_not_exist = 'xAccountIDDoesNotExist'
    add_non_encryptable_nodes = 'xAddNonEncryptableNodes'
    add_non_fips_drives = 'xAddNonFipsDrives'
    add_non_fips_nodes = 'xAddNonFipsNodes'
    admin_id_does_not_exist = 'xAdminIDDoesNotExist'
    already_in_volume_access_group = 'xAlreadyInVolumeAccessGroup'
    api_not_permitted = 'xAPINotPermitted'
    assertion_failure = 'xAssertionFailure'
    async_handle_does_not_exist = 'xAsyncHandleDoesNotExist'
    backup_target_id_does_not_exist = 'xBackupTargetIDDoesNotExist'
    bin_sync_in_progress = 'xBinSyncInProgress'
    check_failure = 'xCheckFailure'
    cluster_pair_id_does_not_exist = 'xClusterPairIDDoesNotExist'
    conflicting_list_items = 'xConflictingListItems'
    conflicting_parameter_values = 'xConflictingParameterValues'
    csr_generation_failed = 'xCsrGenerationFailed'
    daemon_status_change_failed = 'xDaemonStatusChangeFailed'
    daemon_unknown_type = 'xDaemonUnknownType'
    db_bad_arguments = 'xDBBadArguments'
    db_closing = 'xDBClosing'
    db_connection_loss = 'xDBConnectionLoss'
    db_invalid_state = 'xDBInvalidState'
    db_multi_commit_failed = 'xDBMultiCommitFailed'
    db_no_server_response = 'xDBNoServerResponse'
    db_no_such_path = 'xDBNoSuchPath'
    db_operation_timeout = 'xDBOperationTimeout'
    db_path_exists = 'xDBPathExists'
    db_path_not_empty = 'xDBPathNotEmpty'
    db_runtime_inconsistency = 'xDBRuntimeInconsistency'
    db_session_expired = 'xDBSessionExpired'
    db_session_moved = 'xDBSessionMoved'
    db_version_mismatch = 'xDBVersionMismatch'
    drive_id_does_not_exist = 'xDriveIDDoesNotExist'
    duplicate_username = 'xDuplicateUsername'
    ekm_generic_error = 'xEkmGenericError'
    empty_broadcast = 'xEmptyBroadcast'
    encryption_at_rest_disabled = 'xEncryptionAtRestDisabled'
    ensemble_invalid_size = 'xEnsembleInvalidSize'
    exceeded_limit = 'xExceededLimit'
    file_not_found = 'xFileNotFound'
    file_transfer_failed = 'xFileTransferFailed'
    freeze_timeout = 'xFreezeTimeout'
    gc_already_running = 'xGCAlreadyRunning'
    gc_not_started_due_to_skip_gc_set = 'xGCNotStartedDueTocSkipGCSet'
    initiator_does_not_exist = 'xInitiatorDoesNotExist'
    invalid_account_name = 'xInvalidAccountName'
    invalid_api_param = 'xInvalidAPIParameter'
    invalid_certificate = 'xInvalidCertificate'
    invalid_cluster_state = 'xInvalidClusterState'
    invalid_drive_id = 'xInvalidDriveID'
    invalid_encryption_at_rest_state_transition = 'xInvalidEncryptionAtRestStateTransition'
    invalid_fips_drive_configuration = 'xInvalidFipsDriveConfiguration'
    invalid_lun = 'xInvalidLun'
    invalid_network_interface = 'xInvalidNetworkInterface'
    invalid_param = 'xInvalidParameter'
    invalid_param_type = 'xInvalidParameterType'
    invalid_qos_policy_name = 'xQoSPolicyInvalidName'
    invalid_regex = 'xInvalidRegex'
    invalid_schedule_parameters = 'xInvalidScheduleParameters'
    invalid_schedule_type = 'xInvalidScheduleType'
    invalid_snapshot_schedule = 'xInvalidSnapshotSchedule'
    invalid_value = 'xInvalidValue'
    invalid_volume_access_group_name = 'xInvalidVolumeAccessGroupName'
    is_not_primary = 'xIsNotPrimary'
    key_pair_in_use = 'xKeyPairInUse'
    key_provider_active = 'xKeyProviderActive'
    key_provider_configuration_error = 'xKeyProviderConfigurationError'
    key_provider_failed = 'xKeyProviderFailed'
    key_provider_unavailable = 'xKeyProviderUnavailable'
    key_server_already_exists = 'xKeyServerAlreadyExists'
    key_server_auth_failed = 'xKeyServerAuthFailed'
    key_server_conflict = 'xKeyServerConflict'
    key_server_failed = 'xKeyServerFailed'
    key_server_has_provider_assigned = 'xKeyServerHasProviderAssigned'
    key_server_invalid_auth = 'xKeyServerInvalidAuth'
    key_server_last_for_active_provider = 'xKeyServerLastForActiveProvider'
    key_server_ssl_error = 'xKeyServerSslError'
    key_server_unavailable = 'xKeyServerUnavailable'
    lun_in_use = 'xLunInUse'
    max_volumes_per_group_snapshot_exceeded = 'xMaxVolumesPerGroupSnapshotExceeded'
    mismatched_certificate_and_key = 'xMismatchedCertificateAndKey'
    missing_list_item = 'xMissingListItem'
    missing_param = 'xMissingParameter'
    network_config_commit_failed = 'xNetworkConfigCommitFailed'
    no_handler = 'xNoHandler'
    node_id_does_not_exist = 'xNodeIDDoesNotExist'
    non_fips_drives_present = 'xNonFipsDrivesPresent'
    non_fips_nodes_present = 'xNonFipsNodesPresent'
    not_in_volume_access_group = 'xNotInVolumeAccessGroup'
    not_primary = 'xNotPrimary'
    not_ready_for_io = 'xNotReadyForIo'
    object_does_not_exist = 'xObjectDoesNotExist'
    package_active = 'xPackageActive'
    package_already_installed = 'xPackageAlreadyInstalled'
    package_format_unsupported = 'xPackageFormatUnsupported'
    package_not_found = 'xPackageNotFound'
    package_removal_not_allowed = 'xPackageRemovalNotAllowed'
    package_validation_failed = 'xPackageValidationFailed'
    param_not_valid = 'xParameterNotValid'
    primary_cluster_admin = 'xPrimaryClusterAdmin'
    provider_verification_failed = 'xProviderVerificationFailed'
    qos_policy_id_does_not_exist = 'xQoSPolicyDoesNotExist'
    recv_timeout = 'xRecvTimeout'
    slice_service_selection_failed = 'xSliceServiceSelectionFailed'
    snapshot_failed = 'xSnapshotFailed'
    snapshot_id_does_not_exist = 'xSnapshotIDDoesNotExist'
    software_install_not_in_progress = 'xSoftwareInstallNotInProgress'
    software_install_package_not_valid = 'xSoftwareInstallPackageNotValid'
    ssl_keys_do_not_exist = 'xSslKeysDoNotExist'
    system_command_failure = 'xSystemCommandFailure'
    too_few_drives = 'xTooFewDrives'
    unbalanced_cluster = 'xUnbalancedCluster'
    unknown_account = 'xUnknownAccount'
    unknown_api_constant = 'xUnknownAPIConstant'
    unknown_api_method = 'xUnknownAPIMethod'
    unknown_block_id = 'xUnknownBlockID'
    unknown_enum_type = 'xUnknownEnumType'
    unknown_exception = 'xUnknownException'
    unknown_report = 'xUnknownReport'
    unknown_service_id = 'xUnknownServiceID'
    unrecognized_enum_string = 'xUnrecognizedEnumString'
    volume_access_group_id_does_not_exist = 'xVolumeAccessGroupIDDoesNotExist'
    volume_id_does_not_exist = 'xVolumeIDDoesNotExist'
    wait_for_network_interface_status_failed = 'xWaitForNetworkInterfaceStatusFailed'


class SfFeature(sf_models.ApiEnum):
    """Features that can be enabled via the EnableFeature API call."""
    fips = 'Fips'
    fips_drives = 'FipsDrives'
    snap_mirror = 'SnapMirror'
    vvols = 'Vvols'

    @classmethod
    def fields_in_version(cls, version):
        """Returns the list of enum fields valid for the specified version.

        :param sf_models.SfVersion version: SolidFire API version.
        :rtype: list
        """
        fields = super(SfFeature, cls).fields_in_version(version)
        if version < sf_version.SfVersion('11.5'):
            fields.remove(cls.fips_drives)
        return fields


class ShutdownOption(sf_models.ApiEnum):
    """ShutdownOption enumeration.

    Corresponds to the option object in the Shutdown request.
    """
    halt = 'halt'
    restart = 'restart'


class SliceAction(sf_models.ApiEnum):
    """Actions that reassign slice can take."""
    desired = 'Desired'
    prune = 'Prune'
    required = 'Required'


class SliceRole(sf_models.ApiEnum):
    """Different roles of a slice service."""
    dead_secondary = 'deadSecondary'
    live_secondary = 'liveSecondary'
    primary = 'primary'
    standby = 'standby'


class SliceStatus(sf_models.ApiEnum):
    """Used to determine what state a slice balance is in."""
    active = 'active'
    failed = 'failed'
    inactive = 'inactive'
    unassigned = 'unassigned'


class SnapMirrorAccessMode(sf_models.ApiEnum):
    """Used to define snapmirror volume access modes."""
    locked = 'locked'
    read_only = 'readOnly'
    read_write = 'readWrite'


class SnapshotStatus(sf_models.ApiEnum):
    """Used for status field on snapshots."""
    active = 'active'
    cloning = 'cloning'
    done = 'done'
    preparing = 'preparing'
    remote_syncing = 'remoteSyncing'
    unknown = 'unknown'


class SnmpCommunity(sf_models.ApiEnum):
    """Used by community field on SnmpNetworkInfo."""
    public = 'public'
    sf_alerts = 'solidfireAlerts'
    zabbix = 'zabbix'


class StandbyStates(sf_models.ApiEnum):
    """Standby states."""
    active = 'Active'
    failed = 'Failed'
    inactive = 'Inactive'


class SyncJobBranchType(sf_models.ApiEnum):
    """Used for branch_type field on SyncJob data mode."""
    snapshot = 'snapshot'
    volume = 'volume'


class SyncJobStage(sf_models.ApiEnum):
    """Used for stage field on SyncJob data mode.

    metadata
        Replication is in the process of determining what data needs to be
        transferred to the remote cluster. Status is not reported for this stage of the
        replication process.
    data
        Replication is in the process of transferring the bulk of the data to the
        remote cluster.
    whole
        Indicates backward compatibility of the slice for slice sync jobs.
    """
    data = 'data'
    metadata = 'metadata'
    whole = 'whole'


class SyncJobType(sf_models.ApiEnum):
    """Used for type field on SyncJob data mode."""
    block = 'block'
    clone = 'clone'
    remote = 'remote'
    slice = 'slice'


class SystemMemorySeverity(sf_models.ApiEnum):
    """Used by severity field on SystemMemory data model."""
    okay = 'OK'


class VolumeAccess(sf_models.ApiEnum):
    """Used by access field on Volume data model."""
    locked = api_constants.LOCKED
    read_only = api_constants.READ_ONLY
    read_write = api_constants.READ_WRITE
    replication_target = api_constants.REPLICATION_TARGET
    snap_mirror_target = api_constants.SNAPMIRROR_TARGET

    @classmethod
    def fields_in_version(cls, version):
        """Returns the list of enum fields valid for the specified version.

        :param sf_models.SfVersion version: SolidFire API version.
        :rtype: list
        """
        fields = super(VolumeAccess, cls).fields_in_version(version)
        if version < sf_version.SfVersion('10.0'):
            fields.remove(cls.snap_mirror_target)
        return fields


class VolumeStatus(sf_models.ApiEnum):
    """Used by status field on Volume data model."""
    active = api_constants.ACTIVE
    creating = api_constants.CREATING
    deleted = api_constants.DELETED
    init = api_constants.INIT
    snapshotting = api_constants.SNAPSHOTTING


class ZookeeperType(sf_models.ApiEnum):
    """Used by zookeeper mode field on Ensemble Config data model."""
    ensemble = 'ensemble'
    follower = 'follower'
    leader = 'leader'
    non_ensemble = 'non_ensemble'
    standalone = 'standalone'


class NvramVendor(sf_models.ApiEnum):
    """Defines the available nvram vendors used by SolidFire."""
    marvell = 'Marvell'
    micron = 'Micron'
    micron_gen2 = 'Micron_Gen2'
    radian = 'Radian'
    radian_rms300 = 'Radian_RMS300'
    smart = 'Smart'
    smart_gen2 = 'Smart_Gen2'

    def is_marvell(self):
        """Return true if enum is one of the Marvell enums.

        There's only one right now but it could be confusing not to have a function for Marvell.

        :rtype: bool
        """
        return self in [NvramVendor.marvell]

    def is_micron(self):
        """Return true if enum is one of the Micron enums.

        :rtype: bool
        """
        return self in [NvramVendor.micron, NvramVendor.micron_gen2]

    def is_radian(self):
        """Return true if enum is one of the Radian enums.

        :rtype: bool
        """
        return self in [NvramVendor.radian, NvramVendor.radian_rms300]

    def is_smart(self):
        """Return true if enum is one of the Smart enums.

        :rtype: bool
        """
        return self in [NvramVendor.smart, NvramVendor.smart_gen2]
