"""Cluster Constants Data model."""
import sf_models

from sf_api import (
    enums,
    sf_version,
)
from sf_api.models import types


class ClusterConstants(sf_models.Base):
    """Constants used for Cluster model.

    NOTE: serialize_when_none is False for this class.
    When a model attribute is None-valued, it will not be included in .serialize() output.
    """
    allow_drive_type_modification_of_slot_num = types.IntType(
        serialized_name=enums.ClusterConstants.allow_drive_type_modification_of_slot_num.value
    )
    bin_sync_headroom_percent = types.FloatType(
        serialized_name=enums.ClusterConstants.bin_sync_headroom_percent.value
    )
    bin_sync_new_service_write_mib_per_second = types.IntType(
        serialized_name=enums.ClusterConstants.bin_sync_new_service_write_mib_per_second.value
    )
    bin_sync_write_mib_per_second = types.IntType(
        serialized_name=enums.ClusterConstants.bin_sync_write_mib_per_second.value
    )
    bs_check_interval_seconds = types.IntType(
        serialized_name=enums.ClusterConstants.bs_check_interval_seconds.value
    )
    bs_check_pause_msec = types.IntType(
        serialized_name=enums.ClusterConstants.bs_check_pause_msec.value
    )
    bs_incorporate_unusable_space_in_used_bytes = types.BooleanType(
        serialized_name=enums.ClusterConstants.bs_incorporate_unusable_space_in_used_bytes.value
    )
    bs_op_queue_read_threads = types.IntType(
        serialized_name=enums.ClusterConstants.bs_op_queue_read_threads.value
    )
    bs_recycle_at_pct = types.FloatType(
        serialized_name=enums.ClusterConstants.bs_recycle_at_pct.value
    )
    bs_unusable_space_percent = types.IntType(
        serialized_name=enums.ClusterConstants.bs_unusable_space_percent.value
    )
    cluster_bs_check_mode = types.IntType(
        serialized_name=enums.ClusterConstants.cluster_bs_check_mode.value
    )
    disable_pcache_clean_files = types.BooleanType(
        serialized_name=enums.ClusterConstants.disable_pcache_clean_files.value
    )
    enable_auto_install = types.BooleanType(
        serialized_name=enums.ClusterConstants.enable_auto_install.value
    )
    enable_configurable_protection_domains = types.BooleanType(
        serialized_name=enums.ClusterConstants.enable_configurable_protection_domains.value
    )
    enable_protection_domains = types.BooleanType(
        serialized_name=enums.ClusterConstants.enable_protection_domains.value
    )
    enable_space_based_rebalancing = types.BooleanType(
        serialized_name=enums.ClusterConstants.enable_space_based_rebalancing.value
    )
    enable_updating_no_sync = types.BooleanType(
        serialized_name=enums.ClusterConstants.enable_updating_no_sync.value
    )
    event_slice_lbas_max_mismatches = types.IntType(
        serialized_name=enums.ClusterConstants.event_slice_lbas_max_mismatches.value
    )
    max_volumes_per_multi_api = types.IntType(
        serialized_name=enums.ClusterConstants.max_volumes_per_multi_api.value
    )
    merkle_check_time = types.IntType(
        serialized_name=enums.ClusterConstants.merkle_check_time.value
    )
    merkle_checker_every_time = types.IntType(
        serialized_name=enums.ClusterConstants.merkle_checker_every_time.value
    )
    merkle_idle_cycles = types.IntType(
        serialized_name=enums.ClusterConstants.merkle_idle_cycles.value
    )
    panic_on_mismatched_lbas = types.IntType(
        serialized_name=enums.ClusterConstants.panic_on_mismatched_lbas.value
    )
    pcache_ignore_min_flush_size_interval_seconds = types.IntType(
        serialized_name=enums.ClusterConstants.pcache_ignore_min_flush_size_interval_seconds.value
    )
    scheduler_interval_seconds = types.IntType(
        serialized_name=enums.ClusterConstants.scheduler_interval_seconds.value
    )
    skip_gc = types.BooleanType(
        serialized_name=enums.ClusterConstants.skip_gc.value
    )
    slice_balance_enable_auto_timer_iops_rebalancing = types.IntType(
        serialized_name=enums.ClusterConstants.slice_balance_enable_auto_timer_iops_rebalancing.value
    )
    ss_health_monitor_interval_for_permanence = types.IntType(
        serialized_name=enums.ClusterConstants.ss_health_monitor_interval_for_permanence.value
    )
    start_merkle_check_from_qos = types.IntType(
        serialized_name=enums.ClusterConstants.start_merkle_check_from_qos.value
    )
    time_until_bs_is_killed_msec = types.IntType(
        serialized_name=enums.ClusterConstants.time_until_bs_is_killed_msec.value
    )

    # Version 8.0 Additions
    disable_bin_assignment_updates = types.BooleanType(
        serialized_name=enums.ClusterConstants.disable_bin_assignment_updates.value,
        version_added=sf_version.SfVersion('8.0')
    )
    set_bs_op_queue_read_threads_based_on_bin_coupling = types.BooleanType(
        serialized_name=enums.ClusterConstants.set_bs_op_queue_read_threads_based_on_bin_coupling.value,
        version_added=sf_version.SfVersion('8.2')
    )

    # Version 9.0 Additions
    disable_pcache_clean_files = types.BooleanType(
        serialized_name=enums.ClusterConstants.disable_pcache_clean_files.value,
        version_added=sf_version.SfVersion('9.0')
    )

    # Version 10.0 Additions
    gc_interval = types.TimedeltaStringType(
        serialized_name=enums.ClusterConstants.gc_interval.value,
        version_added=sf_version.SfVersion('10.0')
    )

    ssl_certificate_experation_warning_days = types.IntType(
        serialized_name=enums.ClusterConstants.ssl_certificate_experation_warning_days.value,
        version_added=sf_version.SfVersion('10.0')
    )

    # Version 11.0 Additions
    artificial_memory_severity = types.EnumType(
        enum_class=enums.MemoryUsageSeverity,
        serialized_name=enums.ClusterConstants.artificial_memory_severity.value,
        version_added=sf_version.SfVersion('11.0')
    )
    block_service_format_override = types.EnumType(
        enum_class=enums.ServiceFormat,
        serialized_name=enums.ClusterConstants.block_service_format_override.value,
        version_added=sf_version.SfVersion('11.0')
    )
    convert_to_paging_debounce_delay_seconds = types.IntType(
        serialized_name=enums.ClusterConstants.convert_to_paging_debounce_delay_seconds.value,
        version_added=sf_version.SfVersion('11.0')
    )
    force_paging_sublist_count = types.IntType(
        serialized_name=enums.ClusterConstants.force_paging_sublist_count.value,
        version_added=sf_version.SfVersion('11.0')
    )
    pcache_enable_active_file_flush = types.BooleanType(
        serialized_name=enums.ClusterConstants.pcache_enable_active_file_flush.value,
        version_added=sf_version.SfVersion('11.0')
    )

    # Version 11.3 Additions
    access_token_duration = types.TimedeltaStringType(
        serialized_name=enums.ClusterConstants.access_token_duration.value,
        version_added=sf_version.SfVersion('11.3')
    )

    include_custom_pd_in_api = types.BooleanType(
        serialized_name=enums.ClusterConstants.include_custom_pd_in_api.value,
        version_added=sf_version.SfVersion('11.3')
    )

    # Version 11.7 Additions
    bs_first_time_write_ratio = types.FloatType(
        serialized_name=enums.ClusterConstants.bs_first_time_write_ratio.value,
        version_added=sf_version.SfVersion('11.7')
    )
    dont_cache_ear_auth_keys = types.BooleanType(
        serialized_name=enums.ClusterConstants.dont_cache_ear_auth_keys.value,
        version_added=sf_version.SfVersion('11.7')
    )

    # Version 12.0 Additions
    duplicated_metadata_on_disk = types.BooleanType(
        serialized_name=enums.ClusterConstants.duplicated_metadata_on_disk.value,
        version_added=sf_version.SfVersion('12.0')
    )
    enable_suppression_for_bin_assignment_events = types.BooleanType(
        serialized_name=enums.ClusterConstants.enable_suppression_for_bin_assignment_events.value,
        version_added=sf_version.SfVersion('12.0')
    )
    force_python_bin_assignment_algorithm = types.BooleanType(
        serialized_name=enums.ClusterConstants.force_python_bin_assignment_algorithm.value,
        version_added=sf_version.SfVersion('12.0')
    )
    gc_status_polling_interval = types.TimedeltaStringType(
        serialized_name=enums.ClusterConstants.gc_status_polling_interval.value,
        version_added=sf_version.SfVersion('12.0')
    )
    max_time_between_successful_gcs_without_fault = types.TimedeltaStringType(
        serialized_name=enums.ClusterConstants.max_time_between_successful_gcs_without_fault.value,
        version_added=sf_version.SfVersion('12.0')
    )

    auth_session_inactivity_duration = types.TimedeltaStringType(
        serialized_name=enums.ClusterConstants.auth_session_inactivity_duration.value,
        version_added=sf_version.SfVersion('12.0')
    )

    max_auth_sessions = types.IntType(
        serialized_name=enums.ClusterConstants.max_auth_sessions_per_user.value,
        version_added=sf_version.SfVersion('12.0')
    )

    max_auth_sessions_per_user = types.IntType(
        serialized_name=enums.ClusterConstants.max_auth_sessions.value,
        version_added=sf_version.SfVersion('12.0')
    )

    usable_drive_capacity_bytes = types.IntType(
        serialized_name=enums.ClusterConstants.usable_drive_capacity_bytes.value,
        version_added=sf_version.SfVersion('12.0')
    )

    usable_drive_capacity_percent = types.IntType(
        serialized_name=enums.ClusterConstants.usable_drive_capacity_percent.value,
        version_added=sf_version.SfVersion('12.0')
    )

    use_metadata_full_threshold_algorithm = types.BooleanType(
        serialized_name=enums.ClusterConstants.use_metadata_full_threshold_algorithm.value,
        version_added=sf_version.SfVersion('12.0')
    )

    class Options:
        serialize_when_none = False

    def __init__(self, *args, **kwargs):
        kwargs.update({'strict': False})
        super(ClusterConstants, self).__init__(*args, **kwargs)


class ClusterConstantsVerbose(ClusterConstants):
    class Options:
        serialize_when_none = True
