"""Mnode asset Data Models."""
import sf_models

from sf_api import enums
from sf_api.models import types


class HealthcheckStatusModel(sf_models.Base):
    status = types.EnumType(enum_class=enums.HealthcheckStatus)
    checks = types.ListType(types.ModelType(sf_models.Healthcheck), default=list)


class HealthcheckResultModel(sf_models.Base):
    """Total healthcheck result that returns both nodes and cluster checks.

    Corresponds to the healthchecks object in the PreUpgradeHealthCheck API response.
    """
    links = types.DictType(types.StringType(), serialized_name='_links')
    asset_id = types.UUIDType(required=True, serialized_name='assetId')
    compute_id = types.UUIDType(required=False, serialized_name='computeId')
    config = types.DictType(types.StringType(), default=dict)
    date_completed = types.DateTimeType(serialized_name='dateCompleted')
    date_created = types.DateTimeType(serialized_name='dateCreated')
    healthcheck_id = types.UUIDType(serialized_name='healthCheckId')
    state = types.EnumType(enum_class=enums.HealthcheckState)
    status = types.ModelType(HealthcheckStatusModel)
    storage_id = types.UUIDType(required=False, serialized_name='storageId')
    task_id = types.UUIDType(required=False, serialized_name='taskId')
