"""Protection Domains Data Models."""
import sf_models

from sf_api import enums
from sf_api.models import types


class ProtectionSchema(sf_models.Base):
    """Base protection schema data model."""
    protection_scheme = types.EnumType(enums.ProtectionScheme, required=True,
                                       serialized_name='protectionScheme')
    sustainable_failures_for_block_data = types.IntType(
        required=True, serialized_name='sustainableFailuresForBlockData')
    sustainable_failures_for_metadata = types.IntType(
        required=True, serialized_name='sustainableFailuresForMetadata')


class ProtectionDomainTolerance(sf_models.Base):
    """Protection domain tolerance data model."""
    sustainable_failures_for_ensemble = types.IntType(required=True,
                                                      serialized_name='sustainableFailuresForEnsemble')
    protection_scheme_tolerances = types.ListType(types.ModelType(ProtectionSchema), required=True,
                                                  serialized_name='protectionSchemeTolerances')


class ProtectionDomainResiliency(sf_models.Base):
    """Protection domain resilience data model."""
    sustainable_failures_for_ensemble = types.IntType(required=True,
                                                      serialized_name='sustainableFailuresForEnsemble')
    single_failure_threshold_bytes_for_block_data = types.IntType(
        required=True, serialized_name='singleFailureThresholdBytesForBlockData')
    protection_scheme_resiliencies = types.ListType(types.ModelType(ProtectionSchema), required=True,
                                                    serialized_name='protectionSchemeResiliencies')


class ProtectionDomainLevel(sf_models.Base):
    """ProtectionDomainLevel data model.

    Provides a model for the protection domain levels on a cluster.
    """
    protection_domain_type = types.EnumType(enums.ProtectionDomainType, required=True,
                                            serialized_name='protectionDomainType')
    tolerance = types.ModelType(ProtectionDomainTolerance, required=True)
    resiliency = types.ModelType(ProtectionDomainResiliency, required=True)
