"""Security Data Models."""
import sf_models

from sf_api import api_constants
from sf_api.models import types


class KeyProviderKmip(sf_models.Base):
    """KeyProviderKmip data model."""
    key_provider_name = types.StringType(
        required=True,
        serialized_name=api_constants.KEY_PROVIDER_NAME
    )
    key_provider_id = types.IntIdType(
        required=True,
        serialized_name=api_constants.KEY_PROVIDER_ID
    )
    key_provider_is_active = types.BooleanType(
        required=True,
        serialized_name=api_constants.KEY_PROVIDER_IS_ACTIVE
    )
    key_server_ids = types.ListType(
        types.IntIdType,
        required=True,
        serialized_name=api_constants.KEY_SERVER_IDS
    )
    kmip_capabilities = types.StringType(
        required=True,
        serialized_name=api_constants.KMIP_CAPABILITIES
    )

    def __repr__(self):
        return '{}<{}={}, {}={}, {}={}>'.format(
            self.__class__.__name__,
            'key_provider_name', self.key_provider_name,
            'key_provider_id', self.key_provider_id,
            'key_server_ids', self.key_server_ids
        )

    @property
    def id(self):
        """Shortened property for key_provider_id.

        :rtype: int
        """
        return self.key_provider_id

    @property
    def name(self):
        """Shortened property for key_provider_name.

        :rtype: str
        """
        return self.key_provider_name

    @name.setter
    def name(self, value):
        """Shortened setter property for key_provider_name.

        :param str value: Key provider name
        """
        self.key_provider_name = value

    @property
    def is_active(self):
        """Shortened property for key_provider_is_active.

        :rtype: bool
        """
        return self.key_provider_is_active


class KeyServerKmip(sf_models.Base):
    """KeyServerKmip data model."""
    key_provider_id = types.IntIdType(serialized_name=api_constants.KEY_PROVIDER_ID)
    kmip_assigned_provider_is_active = types.BooleanType(
        serialized_name=api_constants.KMIP_ASSIGNED_PROVIDER_IS_ACTIVE
    )
    kmip_ca_certificate = types.StringType(
        required=True,
        serialized_name=api_constants.KMIP_CA_CERTIFICATE
    )
    kmip_client_certificate = types.StringType(
        required=True,
        serialized_name=api_constants.KMIP_CLIENT_CERTIFICATE
    )
    kmip_key_server_hostnames = types.ListType(
        types.StringType,
        required=True,
        serialized_name=api_constants.KMIP_KEY_SERVER_HOSTNAMES
    )
    key_server_id = types.IntIdType(
        required=True,
        serialized_name=api_constants.KEY_SERVER_ID
    )
    kmip_key_server_name = types.StringType(
        required=True,
        serialized_name=api_constants.KMIP_KEY_SERVER_NAME
    )
    kmip_key_server_port = types.IntType(
        required=True,
        serialized_name=api_constants.KMIP_KEY_SERVER_PORT
    )

    def __repr__(self):
        return '{}<{}={}, {}={}>'.format(
            self.__class__.__name__,
            'kmip_key_server_name', self.kmip_key_server_name,
            'key_server_id', self.key_server_id
        )

    @property
    def id(self):
        """Shortened property for key_server_id.

        :rtype: int
        """
        return self.key_server_id

    @property
    def name(self):
        """Shortened property for kmip_key_server_name.

        :rtype: str
        """
        return self.kmip_key_server_name

    @name.setter
    def name(self, value):
        """Shortened setter property for kmip_key_server_name.

        :param str value: Key provider name
        """
        self.kmip_key_server_name = value

    @property
    def is_provider_active(self):
        """Shortened property for kmip_assigned_provider_is_active.

        :rtype: bool
        """
        return self.kmip_assigned_provider_is_active
