"""Snapmirror related data models."""

import sf_models

from sf_api import api_constants
from sf_api.models import types


class SnapMirrorEndpoint(sf_models.Base):
    """Snapmirror endpoint data model.

    Represents the remote SnapMirror storage systems communicating with the SolidFire Cluster.
    """
    snapmirror_endpoint_id = types.IntIdType(required=True,
                                             serialized_name=api_constants.SNAPMIRROR_ENDPOINT_ID)
    name = types.StringType(required=True, serialized_name=api_constants.NAME)
    ip_addresses = types.ListType(types.IPv4Type, required=True,
                                  serialized_name=api_constants.IP_ADDRESSES)

    @classmethod
    def create(cls, name=None, ip_addresses=None, snapmirror_endpoint_id=None):
        """Define a snapmirror endpoint to pass to CreateSnapMirrorEndpoint.

        :param str name: name for the SnapMirrorEndpoint cluster.
        :param typing.Iterable[str] ip_addresses: list of ip addresses for a cluster of
            snapmirror compatible storage systems.
        :param int snapmirror_endpoint_id: Id of snapmirror endpoint
        :return: Partially defined Snapmirror endpoint data model.
        :rtype: sf_api.models.SnapMirrorEndpoint
        """
        return super(SnapMirrorEndpoint, cls).create(
            name=name,
            ip_addresses=ip_addresses,
            snapmirror_endpoint_id=snapmirror_endpoint_id
        )

    def __repr__(self):
        return '{}<{}={}, {}={}>'.format(
            self.__class__.__name__,
            'snapmirror_endpoint_id', self.snapmirror_endpoint_id,
            'name', self.name)
