import unittest

from sf_models import _node_type

from sf_api import (
    enums,
    models,
)


class TestSfNodeBase(unittest.TestCase):
    def test_static_id(self):
        node_dm = models.SfNodeBase.create(mip='1.2.3.4')
        assert node_dm.static_id == '1.2.3.4'

        node_dm.static_id = '5.6.7.8'
        assert node_dm.static_id == '5.6.7.8'

    def test_role(self):
        node_dm = models.SfNodeBase.create()
        node_dm.platform_info = models.PlatformInfo.create(
            node_type=_node_type.NodeType(_node_type.NodeTypeEnum.SF3010.value)
        )
        assert node_dm.role is enums.NodeRole.storage

        node_dm.role = enums.NodeRole.witness
        assert node_dm.role is enums.NodeRole.witness
        assert node_dm.is_witness()

        fc_models = _node_type.FC_NODE_MODELS
        for model in fc_models:
            node_dm.platform_info = models.PlatformInfo.create(
                node_type=_node_type.NodeType(model)
            )
            assert node_dm.role is enums.NodeRole.fibre_channel
