import inspect
import unittest

from sf_api import models


class TestModelRepr(unittest.TestCase):
    """Test repr of all data models."""

    def test_model_repr(self):
        for name, dm in inspect.getmembers(models):
            if inspect.isclass(dm) and issubclass(dm, models.Base):
                assert repr(dm()) is not None


class TestCreate(unittest.TestCase):
    """Test the create method."""
    def test_base_create(self):
        base_dm = models.Base.create()
        assert isinstance(base_dm, models.Base)

    def test_account_create(self):
        account_dm = models.Account.create()
        assert isinstance(account_dm, models.Account)

    def test_address_blocks_create(self):
        address_blocks_dm = models.AddressBlocks.create()
        assert isinstance(address_blocks_dm, models.AddressBlocks)

    def test_snapmirror_endpoint_create(self):
        snapmirror_endpoint_dm = models.SnapMirrorEndpoint.create()
        assert isinstance(snapmirror_endpoint_dm, models.SnapMirrorEndpoint)

    def test_snapshot_create(self):
        snapshot_dm = models.Snapshot.create()
        assert isinstance(snapshot_dm, models.Snapshot)

    def test_weekday_create(self):
        weekday_dm = models.Weekday.create()
        assert isinstance(weekday_dm, models.Weekday)

    def test_schedule_info_create(self):
        schedule_info_dm = models.ScheduleInfo.create()
        assert isinstance(schedule_info_dm, models.ScheduleInfo)

    def test_schedule_create(self):
        schedule_dm = models.Schedule.create()
        assert isinstance(schedule_dm, models.Schedule)

    def test_initiator_create(self):
        initiator_dm = models.Initiator.create(alias='test', attributes={}, initiator_id=1,
                                               initiator_name='test', volume_access_groups=[1, 2, 3])
        assert isinstance(initiator_dm, models.Initiator)

    def test_qos_create(self):
        qos_dm = models.Qos.create(min_iops=1, max_iops=1, burst_iops=1, burst_time=1)
        assert isinstance(qos_dm, models.Qos)
