"""General request data models."""
from sf_api import (
    api_constants,
    enums,
    result_models,
    sf_version,
)
from sf_api.models import types
from sf_api.request_models import (
    _base,
    _private,
)


class StartGC(_base.BaseClusterApi):
    """StartGC request."""
    _version_added = sf_version.SfVersion('5.0')
    async_handle = types.BooleanType(
        required=True,
        choices=[True],
        serialized_name=api_constants.ASYNC_HANDLE,
        metadata={
            api_constants.NULL_INVALID: True,
            api_constants.EXCEPTION_OVERRIDES: {
                enums.ApiExceptionType.param_missing:
                    enums.SfException.gc_not_started_due_to_skip_gc_set,
                enums.ApiExceptionType.param_null:
                    enums.SfException.gc_not_started_due_to_skip_gc_set,
                enums.ApiExceptionType.param_value_invalid:
                    enums.SfException.gc_not_started_due_to_skip_gc_set}
        }
    )

    def __init__(self, **kwargs):
        super(StartGC, self).__init__(result_dm=result_models.StartGC, async_timeout=3600, **kwargs)
        self.add_setup_api(_private.SetConstants, skip_gc=True)
        self.add_setup_api(
            request_dm=GetGCStatus,
            loop_for_response={result_models.GetGCStatus.status: enums.GcStatus.complete}
        )
        self.add_teardown_api(
            request_dm=GetGCStatus,
            loop_for_response={result_models.GetGCStatus.status: enums.GcStatus.complete}
        )
        self.is_async = True


class GetGCStatus(_base.BaseClusterApi):
    """GetGCStatus request."""
    _version_added = sf_version.SfVersion('5.0')

    def __init__(self, **kwargs):
        super(GetGCStatus, self).__init__(result_dm=result_models.GetGCStatus, **kwargs)
