"""Data models that correspond to the result key in a response for SolidFire.

The names of these models should match the name of the SolidFire API that returns them.
NOTE: In some cases this may break Valence naming conventions.
"""
from sf_api.result_models._async import (
    GetAsyncResult,
    ListAsyncResults,
)
from sf_api.result_models.account import (
    AddAccount,
    GetAccountById,
    GetAccountByName,
    GetAccountEfficiency,
    ListAccounts,
    ModifyAccount,
)
from sf_api.result_models.auth_session import (
    CreateAuthSessionResult,
    DeleteAuthSessionResult,
    DeleteAuthSessionsResult,
    ListAuthSessionsResult,
    ListCurrentClusterAdminsResult,
    UpdateAuthSessionResult,
)
from sf_api.result_models.base_result import BaseApiResult
from sf_api.result_models.cluster import (
    AddBlockToBS,
    AddClusterAdmin,
    AddLdapClusterAdmin,
    AddNodes,
    AddVirtualNetwork,
    CheckBlock,
    CheckProposedNodeAdditions,
    ClusterGetNetworkInterfaceStatus,
    CompleteClusterPairing,
    CreateEvent,
    DeleteAllSupportBundles,
    DemoteClusterMaster,
    DisableStandbySliceAssignments,
    EnableStandbySliceAssignments,
    GetActiveTlsCiphers,
    GetAuthConfiguration,
    GetClusterCapacity,
    GetClusterFullThreshold,
    GetClusterHardwareInfo,
    GetClusterInfo,
    GetClusterMasterNodeID,
    GetClusterStats,
    GetClusterStructure,
    GetClusterVersionInfo,
    GetConstants,
    GetCurrentClusterAdmin,
    GetDatabaseEntry,
    GetFeatureStatus,
    GetFipsReport,
    GetGCStatus,
    GetHardwareInfoOverrides,
    GetImmutableValues,
    GetLimits,
    GetLoginSessionInfo,
    GetNetworkInterfaceStats,
    GetNodeHardwareInfo,
    GetNodeStats,
    GetNtpInfo,
    GetProtectionDomainLayout,
    GetRawStats,
    GetReport,
    GetServiceStatus,
    GetSnmpInfo,
    GetSslCertificate,
    GetSupportedTlsCiphers,
    ListActiveNodes,
    ListAllNodes,
    ListAvailableNodes,
    ListClusterAdmins,
    ListClusterFaults,
    ListClusterPairs,
    ListDatabaseChildrenData,
    ListEvents,
    ListISCSISessions,
    ListNodeStats,
    ListPendingActiveNodes,
    ListPendingNodes,
    ListProtectionDomainLevels,
    ListProtectionDomains,
    ListServices,
    ListVirtualNetworks,
    ModifyClusterFullThreshold,
    PromoteClusterMaster,
    ProtocolVersionUpgradeAvailable,
    RecoverDeadVolumes,
    RemoveBlockFromBS,
    RemoveSslCertificate,
    ResetConstants,
    ResetSupplementalTlsCiphers,
    SetAuthConfiguration,
    SetClusterStructure,
    SetConstants,
    SetDatabaseEntry,
    SetHardwareInfoOverrides,
    SetLoginBanner,
    SetProtectionDomainLayout,
    SetSslCertificate,
    SetSupplementalTlsCiphers,
    Shutdown,
    StartClusterBSCheck,
    StartClusterPairing,
    StartGC,
    StopClusterBSCheck,
)
from sf_api.result_models.drive import (
    AddDrives,
    GetDriveHardwareInfo,
    GetDriveStats,
    ListDrives,
    ListDriveStats,
    RemoveDrives,
    ResizeDrives,
    SecureEraseDrives,
)
from sf_api.result_models.fibre_channel import (
    GetVolumeAccessGroupLunAssignments,
    ListFibreChannelPortInfo,
    ListFibreChannelSessions,
    ListNodeFibreChannelPortInfo,
    ModifyVolumeAccessGroupLunAssignments,
)
from sf_api.result_models.ldap import (
    GetLdapConfiguration,
    TestLdapAuthentication,
)
from sf_api.result_models.mfa import (
    CreateIdpConfigurationResult,
    DeleteIdpConfigurationResult,
    DisableIdpAuthenticationResult,
    EnableIdpAuthenticationResult,
    GetIdpAuthenticationStateResult,
    ListIdpConfigurationsResult,
    UpdateIdpConfigurationResult,
)
from sf_api.result_models.node import (
    CheckProposedCluster,
    CreateClusterSupportBundle,
    CreateSupportBundle,
    DeleteDownloads,
    GetBootstrapConfig,
    GetClusterConfig,
    GetClusterState,
    GetConfig,
    GetConfigExtended,
    GetDriveConfig,
    GetEnsembleConfig,
    GetHardwareConfig,
    GetHardwareInfo,
    GetIpmiConfig,
    GetIpmiInfo,
    GetNetworkConfig,
    GetNetworkInterface,
    GetNodeFipsDrivesReport,
    GetPendingOperation,
    GetStartupFlags,
    GetVersionInfo,
    InstallPackage,
    ListDownloads,
    ListDriveHardware,
    ListMountedFileSystems,
    ListNetworkInterfaces,
    ListPackageInfo,
    ListTests,
    ListUtilities,
    NodeGetNetworkInterfaceStatus,
    ReadLink,
    ResetDrives,
    ResetNode,
    RestartServices,
    SetActivePackageVersion,
    SetClusterConfig,
    SetConfig,
    SetConfigExtended,
    SetNetworkConfig,
    SetNetworkInterfaceStatus,
    SetStartupFlags,
    TestConnectEnsemble,
    TestConnectMvip,
    TestConnectSvip,
    TestHardwareConfig,
    TestLocalConnectivity,
    TestLocateCluster,
    TestNetworkConfig,
    TestPing,
    TestRemoteConnectivity,
    UninstallPackage,
)
from sf_api.result_models.schedule import (
    CreateSchedule,
    ListSchedules,
    ModifySchedule,
)
from sf_api.result_models.security import (
    CreateKeyProviderKmip,
    CreateKeyServerKmip,
    EnableEncryptionAtRest,
    GetClientCertificateSignRequest,
    GetEncryptionAtRestInfo,
    GetKeyProviderKmip,
    GetKeyServerKmip,
    ListKeyProvidersKmip,
    ListKeyServersKmip,
    ModifyKeyServerKmip,
)
from sf_api.result_models.snapmirror import (
    CreateSnapMirrorEndpoint,
    ListSnapMirrorEndpoints,
    ModifySnapMirrorEndpoint,
)
from sf_api.result_models.snapshot import (
    CreateGroupSnapshot,
    CreateSnapshot,
    ListGroupSnapshots,
    ListSnapshots,
    ModifyGroupSnapshot,
    ModifySnapshot,
    RollbackToGroupSnapshot,
    RollbackToSnapshot,
)
from sf_api.result_models.volume import (
    AddInitiatorsToVolumeAccessGroup,
    AddVolumesToVolumeAccessGroup,
    CheckVolumeStandbysAssigned,
    CloneMultipleVolumes,
    CloneVolume,
    ComputeVolumeChecksum,
    CopyVolume,
    CreateBackupTarget,
    CreateInitiators,
    CreateMultipleVolumes,
    CreateQoSPolicy,
    CreateVolume,
    CreateVolumeAccessGroup,
    DeleteVolume,
    DeleteVolumes,
    FillSlice,
    GetBackupTarget,
    GetDefaultQoS,
    GetQoSPolicy,
    GetSliceInfo,
    GetVolumeAccessGroupEfficiency,
    GetVolumeCount,
    GetVolumeEfficiency,
    GetVolumeStats,
    ListActivePairedVolumes,
    ListActiveVolumes,
    ListBackupTargets,
    ListBulkVolumeJobs,
    ListCloneJobs,
    ListDeletedVolumes,
    ListInitiators,
    ListQoSPolicies,
    ListSyncJobs,
    ListVolumeAccessGroups,
    ListVolumeQoSHistograms,
    ListVolumes,
    ListVolumesForAccount,
    ListVolumeStats,
    ListVolumeStatsByAccount,
    ListVolumeStatsByVirtualVolume,
    ListVolumeStatsByVolume,
    ListVolumeStatsByVolumeAccessGroup,
    ModifyInitiator,
    ModifyInitiators,
    ModifyQoSPolicy,
    ModifyVolume,
    ModifyVolumeAccessGroup,
    ModifyVolumes,
    MovePrimariesAwayFromNode,
    PurgeAllDeletedVolumes,
    ReadSliceLBA,
    ReadSliceLBAChecksum,
    ReassignSlice,
    RemoveInitiatorsFromVolumeAccessGroup,
    RemoveVolumesFromVolumeAccessGroup,
    SetDefaultQoS,
    StartBulkVolumeRead,
    StartBulkVolumeWrite,
    StartVolumePairing,
    UpdateBulkVolumeStatus,
)
