"""Base class for result data models."""

import mock
import sf_models

import sf_api
from sf_api import api_constants


class BaseApiResult(sf_models.Base):
    """Base API result model class."""

    @classmethod
    def get_mock_response(cls, overrides=None, version=None, include_optional=None,
                          force_mock=None, exclude_fields=None):
        """Mock SfResponse.

        :param overrides: Overrides for the model. This should be a nested dictionary of field names in
            which the particular fields to override are mapped to the values with which they are to be
            overridden. Keys must be equal to the field names, not the serialized names
            (e.g. use drive_id instead of driveID).
            Example: ``{'parent_field': {'child_field': {... {'field_to_override': override_value}}``
        :type overrides: dict[str, Any]
        :param sf_api.SfVersion version: SolidFire API version to mock.
            Fields that do not exist in given version will be set to None.
        :param bool include_optional: Whether to mock non-required fields. None randomly mocks optional
            fields.
        :param force_mock: Force the mocking of a particular field. This should be a nested dictionary
            of field names in which the particular fields to be mocked are mapped to True. Keys must be
            equal to the field names, not the serialized names (e.g. use drive_id instead of driveID).
            Example: ``{'parent_field': {'child_field': {... {'field_to_mock': True}}}``
        :type force_mock: dict[str, dict|bool]
        :param exclude_fields: Exclude the specified fields. This should be a nested dictionary of
            field names in which the particular fields to be excluded are mapped to True. Keys must be
            equal to the field names, not the serialized names (e.g. use drive_id instead of driveID).
            Example: ``{'parent_field': {'child_field': {... {'field_to_exclude': True}}}``
        :type exclude_fields: dict[str, dict|bool]
        :rtype: sf_api.SfResponse
        """
        mock_method = mock.Mock()
        mock_data = cls.get_mock_data(overrides=overrides, version=version,
                                      include_optional=include_optional,
                                      force_mock=force_mock, exclude_fields=exclude_fields)
        mock_raw_data = {api_constants.ID: sf_api.get_json_rpc_id(), api_constants.RESULT: mock_data}
        mock_method.value = cls.__name__
        return sf_api.SfResponse(mock_raw_data, mock_method, result_dm_class=cls)
