"""Result models related to clusters."""
from sf_api import (
    api_constants,
    enums,
    models,
    sf_version,
)
from sf_api.models import types
from sf_api.result_models import base_result


class AddClusterAdmin(base_result.BaseApiResult):
    cluster_admin_id = types.IntIdType(required=True, serialized_name=api_constants.CLUSTER_ADMIN_ID)


class AddLdapClusterAdmin(base_result.BaseApiResult):
    cluster_admin_id = types.IntIdType(required=True, serialized_name=api_constants.CLUSTER_ADMIN_ID)


class AddNodes(base_result.BaseApiResult):
    auto_install = types.BooleanType(required=True, serialized_name=api_constants.AUTO_INSTALL)
    nodes = types.ListType(types.ModelType(models.PendingNode), required=True)


class CheckProposedNodeAdditions(base_result.BaseApiResult):
    proposed_cluster_valid = types.BooleanType(
        serialized_name='proposedClusterValid',
        required=True
    )
    proposed_cluster_errors = types.ListType(
        field=types.ModelType(models.ProposedClusterError),
        serialized_name='proposedClusterErrors',
        required=True
    )


class CreateEvent(models.ClusterEvent, base_result.BaseApiResult):
    pass


class DeleteAllSupportBundlesDetails(base_result.BaseApiResult):
    """Model for nested details field in DeleteAllSupportBundles."""
    output = types.StringType(required=True)


class DeleteAllSupportBundles(base_result.BaseApiResult):
    result = types.EnumType(enums.ApiTestResult, required=True)
    details = types.ModelType(DeleteAllSupportBundlesDetails, required=True)
    duration = types.TimedeltaStringType(required=True)


class DemoteClusterMaster(base_result.BaseApiResult):
    current_candidates = types.ListType(
        field=types.StringType(regex=r'^\d+-\d+$'),
        serialized_name='currentCandidates',
        version_added=sf_version.SfVersion('10.3'),
        required=True
    )
    node_id = types.IntIdType(
        serialized_name=api_constants.NODE_ID,
        version_added=sf_version.SfVersion('10.3'),
        required=True
    )
    sequence_number = types.IntType(
        serialized_name='sequenceNumber',
        version_added=sf_version.SfVersion('10.3'),
        required=True
    )
    service_id = types.IntIdType(
        serialized_name=api_constants.SERVICE_ID,
        version_added=sf_version.SfVersion('10.3'),
        required=True
    )


class GetConstants(models.ClusterConstants, base_result.BaseApiResult):
    pass


class SetConstants(models.ClusterConstants, base_result.BaseApiResult):
    request_api_version = types.SfApiVersionType(serialized_name='requestAPIVersion')


class ResetConstants(models.ClusterConstants, base_result.BaseApiResult):
    pass


class GetActiveTlsCiphers(models.CiphersInfo, base_result.BaseApiResult):
    pass


class GetSupportedTlsCiphers(base_result.BaseApiResult):
    mandatory_ciphers = types.ListType(
        field=types.StringType(),
        serialized_name='mandatoryCiphers',
        version_added=sf_version.SfVersion('11.3'),
        required=True)
    default_supplemental_ciphers = types.ListType(
        field=types.StringType(),
        serialized_name='defaultSupplementalCiphers',
        version_added=sf_version.SfVersion('11.3'),
        required=True)
    supported_supplemental_ciphers = types.ListType(
        field=types.StringType(),
        serialized_name='supportedSupplementalCiphers',
        version_added=sf_version.SfVersion('11.3'),
        required=True)


class SetSupplementalTlsCiphers(models.CiphersInfo, base_result.BaseApiResult):
    pass


class ResetSupplementalTlsCiphers(models.CiphersInfo, base_result.BaseApiResult):
    pass


class GetClusterCapacity(base_result.BaseApiResult):
    cluster_capacity = types.ModelType(
        model_spec=models.ClusterCapacity,
        required=True,
        serialized_name='clusterCapacity'
    )


class GetClusterFullThreshold(models.ClusterFullThreshold, base_result.BaseApiResult):
    pass


class GetClusterHardwareInfo(base_result.BaseApiResult):
    cluster_hardware_info = types.ModelType(
        model_spec=models.ClusterHardwareInfo,
        required=True,
        serialized_name='clusterHardwareInfo'
    )


class GetClusterInfo(base_result.BaseApiResult):
    cluster_info = types.ModelType(models.Cluster, required=True, serialized_name='clusterInfo')


class GetClusterMasterNodeID(base_result.BaseApiResult):
    node_id = types.IntIdType(required=True, serialized_name=api_constants.NODE_ID)


class GetClusterStats(base_result.BaseApiResult):
    cluster_stats = types.ModelType(models.ClusterStats, required=True, serialized_name='clusterStats')


class GetClusterVersionInfo(models.ClusterVersionInfo, base_result.BaseApiResult):
    pass


class GetCurrentClusterAdmin(base_result.BaseApiResult):
    cluster_admin = types.ModelType(models.ClusterAdmin, required=True, serialized_name='clusterAdmin')


class GetDatabaseEntry(base_result.BaseApiResult):
    data = types.DictType(types.BaseType())
    stat = types.ModelType(models.ZkEntryStat, required=True)


class SetDatabaseEntry(base_result.BaseApiResult):
    data = types.DictType(types.BaseType())
    stat = types.ModelType(models.ZkEntryStat, required=True)


class GetFeatureStatus(base_result.BaseApiResult):
    features = types.ListType(types.ModelType(models.FeatureStatus), required=True)


class GetFipsReport(base_result.BaseApiResult):
    nodes = types.ListType(types.ModelType(models.FipsNodeReport), required=True)
    error_nodes = types.ListType(
        types.ModelType(models.FipsErrorNodeReport),
        required=True,
        serialized_name='errorNodes'
    )


class GetGCStatus(base_result.BaseApiResult, models.GarbageCollectionDetails):
    """As of 12.0, GetGCStatus matches the internal details of a GC AsyncResult."""


class GetImmutableValues(base_result.BaseApiResult):
    per_block_overhead_bytes = types.IntType(
        required=True,
        default=42,
        serialized_name='perBlockOverheadBytes'
    )
    per_block_metadata_bytes = types.IntType(
        required=True,
        default=16,
        serialized_name='perBlockMetadataBytes',
        version_added=sf_version.SfVersion('12.0')
    )
    per_page_metadata_blocks = types.IntType(
        required=True,
        default=341,
        serialized_name='perPageMetadataBlocks',
        version_added=sf_version.SfVersion('12.0')
    )
    per_page_metadata_bytes = types.IntType(
        required=True,
        default=4096,
        serialized_name='perPageMetadataBytes',
        version_added=sf_version.SfVersion('12.0')
    )


class GetLimits(models.ClusterLimits, base_result.BaseApiResult):
    pass


class GetLoginSessionInfo(base_result.BaseApiResult):
    login_session_info = types.ModelType(
        model_spec=models.LoginSessionInfo,
        required=True,
        serialized_name='loginSessionInfo'
    )


class GetNetworkInterfaceStats(base_result.BaseApiResult):
    nodes = types.ListType(types.ModelType(models.NodeNetworkInterfaceStats), required=True)


class ClusterGetNetworkInterfaceStatus(base_result.BaseApiResult):
    """Result data model for the cluster api GetNetworkInterfaceStatus method."""
    nodes = types.ListType(types.ModelType(models.NodeNetworkInterfaceStatus), required=True)


class GetNodeHardwareInfo(base_result.BaseApiResult):
    node_hardware_info = types.ModelType(
        model_spec=models.NodeHardwareInfo,
        required=True,
        serialized_name='nodeHardwareInfo'
    )


class GetNodeStats(base_result.BaseApiResult):
    node_stats = types.ModelType(models.NodeStats, required=True, serialized_name='nodeStats')


class GetNtpInfo(models.NtpInfo, base_result.BaseApiResult):
    pass


class GetRawStats(models.RawStats, base_result.BaseApiResult):
    pass


class GetReport(base_result.BaseApiResult):
    reports = types.ListType(types.ModelType(models.ClusterReport), required=True)


class GetServiceStatus(base_result.BaseApiResult):
    success = types.DictType(types.BooleanType())
    failed = types.DictType(types.StringType())


class GetSnmpInfo(models.SnmpInfo, base_result.BaseApiResult):
    pass


class ListActiveNodes(base_result.BaseApiResult):
    nodes = types.ListType(types.ModelType(models.ActiveNode), required=True, default=list)


class ListAvailableNodes(base_result.BaseApiResult):
    nodes = types.ListType(types.ModelType(models.AvailableNode), required=True, default=list)


class ListAllNodes(base_result.BaseApiResult):
    nodes = types.ListType(types.ModelType(models.ActiveNode), required=True, default=list)
    pending_nodes = types.ListType(
        field=types.ModelType(models.PendingNode),
        required=True,
        default=list,
        serialized_name=api_constants.PENDING_NODES
    )
    pending_active_nodes = types.ListType(
        field=types.ModelType(models.PendingActiveNode),
        required=True,
        default=list,
        serialized_name=api_constants.PENDING_ACTIVE_NODES,
        version_added=sf_version.SfVersion('9.0')
    )


class ListClusterAdmins(base_result.BaseApiResult):
    cluster_admins = types.ListType(
        field=types.ModelType(models.ClusterAdmin),
        required=True,
        serialized_name='clusterAdmins'
    )


class ListClusterFaults(base_result.BaseApiResult):
    faults = types.ListType(types.ModelType(models.ClusterFault), required=True, default=list)


class ListClusterPairs(base_result.BaseApiResult):
    cluster_pairs = types.ListType(
        field=types.ModelType(models.ClusterPair),
        required=True,
        serialized_name=api_constants.CLUSTER_PAIRS
    )


class ListDatabaseChildrenData(base_result.BaseApiResult):
    children = types.ListType(
        field=types.ModelType(models.ZkDatabaseEntry),
        required=True,
        serialized_name=api_constants.CHILDREN
    )


class ListEvents(base_result.BaseApiResult):
    event_queue_type = types.EnumType(
        enum_class=enums.EventQueueType,
        required=True,
        serialized_name='eventQueueType'
    )
    events = types.ListType(types.ModelType(models.ClusterEvent), required=True)


class ListISCSISessions(base_result.BaseApiResult):
    sessions = types.ListType(types.ModelType(models.IscsiSession), required=True)


class ListNetworkInterfaces(base_result.BaseApiResult):
    interfaces = types.ListType(types.ModelType(models.NetworkInterface), required=True)


class AddVirtualNetwork(base_result.BaseApiResult):
    virtual_network_id = types.IntType(required=True, serialized_name='virtualNetworkID')


class ListVirtualNetworks(base_result.BaseApiResult):
    virtual_networks = types.ListType(types.ModelType(models.VirtualNetwork),
                                      serialized_name='virtualNetworks', required=True)


class ListNodeStats(base_result.BaseApiResult):
    node_stats = types.ModelType(models.ClusterNodeStats, required=True, serialized_name='nodeStats')


class ListPendingActiveNodes(base_result.BaseApiResult):
    pending_active_nodes = types.ListType(
        field=types.ModelType(models.PendingActiveNode),
        required=True,
        default=list,
        serialized_name=api_constants.PENDING_ACTIVE_NODES
    )


class ListPendingNodes(base_result.BaseApiResult):
    pending_nodes = types.ListType(
        field=types.ModelType(models.PendingNode),
        required=True,
        default=list,
        serialized_name=api_constants.PENDING_NODES
    )


class ListProtectionDomains(base_result.BaseApiResult):
    nodes = types.ListType(types.ModelType(models.PrivateProtectionDomain), required=True)


class GetProtectionDomainLayout(base_result.BaseApiResult):
    protection_domain_layout = types.ListType(
        types.ModelType(models.NodeProtectionDomainLayout), required=True,
        serialized_name=api_constants.PROTECTION_DOMAIN_LAYOUT)


class SetProtectionDomainLayout(base_result.BaseApiResult):
    protection_domain_layout = types.ListType(
        types.ModelType(models.NodeProtectionDomainLayout), required=True,
        serialized_name=api_constants.PROTECTION_DOMAIN_LAYOUT)


class ListProtectionDomainLevels(base_result.BaseApiResult):
    protection_domain_levels = types.ListType(
        types.ModelType(models.ProtectionDomainLevel), required=True,
        serialized_name='protectionDomainLevels')


class ListServices(base_result.BaseApiResult):
    services = types.ListType(types.ModelType(models.ClusterServiceInfo), required=True)


class ModifyClusterFullThreshold(models.ClusterFullThreshold, base_result.BaseApiResult):
    pass


class PromoteClusterMaster(base_result.BaseApiResult):
    current_candidates = types.ListType(
        field=types.StringType(regex=r'^\d+-\d+$'),
        serialized_name='currentCandidates',
        version_added=sf_version.SfVersion('10.3'),
        required=True
    )
    old_cluster_master_node_id = types.IntIdType(
        serialized_name='oldClusterMasterNodeID',
        version_added=sf_version.SfVersion('10.3'),
        required=True
    )
    old_cluster_master_service_id = types.IntIdType(
        serialized_name='oldClusterMasterServiceID',
        version_added=sf_version.SfVersion('10.3'),
        required=True
    )
    requested_cluster_master_node_id = types.IntIdType(
        serialized_name='requestedClusterMasterNodeID',
        version_added=sf_version.SfVersion('10.3'),
        required=True
    )
    requested_cluster_master_service_id = types.IntIdType(
        serialized_name='requestedClusterMasterServiceID',
        version_added=sf_version.SfVersion('10.3'),
        required=True
    )
    target_sequence_number = types.IntType(
        serialized_name='targetSequenceNumber',
        version_added=sf_version.SfVersion('10.3'),
        required=True
    )


class ProtocolVersionUpgradeAvailable(base_result.BaseApiResult):
    available = types.BooleanType(required=True)
    version = types.IntType(required=True)


class SetLoginBanner(base_result.BaseApiResult):
    banner = types.StringType()
    enabled = types.BooleanType()


class Shutdown(base_result.BaseApiResult):
    failed = types.ListType(types.IntIdType(), required=True)
    successful = types.ListType(types.IntIdType(), required=True)


class StartGC(base_result.BaseApiResult):
    async_handle = types.IntType(required=False, serialized_name='asyncHandle')


class GetHardwareInfoOverrides(base_result.BaseApiResult):
    hardware_info_overrides = types.ModelType(
        model_spec=models.HardwareInfoOverrides,
        required=True,
        serialized_name='hardwareInfoOverrides'
    )


class SetHardwareInfoOverrides(base_result.BaseApiResult):
    hardware_info_overrides = types.ModelType(
        model_spec=models.HardwareInfoOverrides,
        required=True,
        serialized_name='hardwareInfoOverrides'
    )


class GetClusterStructure(models.ClusterStructure, base_result.BaseApiResult):
    pass


class SetClusterStructure(base_result.BaseApiResult):
    async_handle = types.IntType(required=True, serialized_name='asyncHandle')


class StartClusterBSCheck(models.PerServiceStatus, base_result.BaseApiResult):
    async_handle = types.IntType(required=True, serialized_name='asyncHandle')


class StopClusterBSCheck(models.PerServiceStatus, base_result.BaseApiResult):
    pass


class StartClusterPairing(base_result.BaseApiResult):
    cluster_pairing_key = types.StringType(required=True,
                                           serialized_name=api_constants.CLUSTER_PAIRING_KEY)
    cluster_pair_id = types.IntIdType(required=True, serialized_name=api_constants.CLUSTER_PAIR_ID)


class CompleteClusterPairing(base_result.BaseApiResult):
    cluster_pair_id = types.IntIdType(required=True, serialized_name=api_constants.CLUSTER_PAIR_ID)


class SetAuthConfiguration(base_result.BaseApiResult):
    pass


class GetAuthConfiguration(base_result.BaseApiResult):
    auth_configuration = types.ModelType(models.AuthConfiguration, required=True,
                                         serialized_name=api_constants.AUTH_CONFIGURATION)


class RemoveBlockFromBS(models.PerServiceStatus, base_result.BaseApiResult):
    pass


class AddBlockToBS(models.PerServiceStatus, base_result.BaseApiResult):
    pass


class CheckBlock(base_result.BaseApiResult):
    block = types.ModelType(models.ClusterBlock, required=True)


class SetSslCertificate(base_result.BaseApiResult):
    pass


class GetSslCertificate(base_result.BaseApiResult):
    certificate = types.StringType(required=True)
    details = types.ModelType(models.CertificateDetails, required=True,
                              serialized_name=api_constants.DETAILS)


class RemoveSslCertificate(base_result.BaseApiResult):
    pass


class EnableStandbySliceAssignments(base_result.BaseApiResult):
    pass


class DisableStandbySliceAssignments(base_result.BaseApiResult):
    pass


class RecoverDeadVolumes(base_result.BaseApiResult):
    pass
