"""Result data models related to drives."""
from sf_api import (
    api_constants,
    models,
    sf_version,
)
from sf_api.models import types
from sf_api.result_models import base_result


class AddDrives(base_result.BaseApiResult):
    async_handle = types.IntType(
        required=True,
        version_added=sf_version.SfVersion('9.0'),
        serialized_name=api_constants.ASYNC_HANDLE
    )


class GetDriveHardwareInfo(base_result.BaseApiResult):
    drive_hardware_info = types.ModelType(
        model_spec=models.DriveHardwareInfo,
        required=True,
        serialized_name='driveHardwareInfo'
    )


class GetDriveStats(base_result.BaseApiResult):
    drive_stats = types.ModelType(models.DriveStats, required=True, serialized_name='driveStats')


class ListDrives(base_result.BaseApiResult):
    drives = types.ListType(types.ModelType(models.Drive), required=True, default=list)


class ListDriveStats(base_result.BaseApiResult):
    drive_stats = types.ListType(
        field=types.ModelType(models.DriveStats),
        required=True,
        serialized_name='driveStats'
    )
    errors = types.ListType(types.ModelType(models.DriveError), required=True)


class RemoveDrives(base_result.BaseApiResult):
    async_handle = types.IntType(
        required=True,
        serialized_name=api_constants.ASYNC_HANDLE
    )


class ResizeDrives(base_result.BaseApiResult):
    drives = types.ListType(types.ModelType(models.Drive), required=True, default=list)


class SecureEraseDrives(base_result.BaseApiResult):
    async_handle = types.IntType(
        required=True,
        serialized_name=api_constants.ASYNC_HANDLE
    )
