"""Result data models related to nodes."""
from sf_api import (
    enums,
    models,
)
from sf_api.models import types
from sf_api.result_models import base_result


class CheckProposedCluster(base_result.BaseApiResult):
    proposed_cluster_valid = types.BooleanType(
        serialized_name='proposedClusterValid',
        required=True
    )
    proposed_cluster_errors = types.ListType(
        field=types.ModelType(models.ProposedClusterError),
        serialized_name='proposedClusterErrors',
        required=True
    )


class CreateClusterSupportBundle(base_result.BaseApiResult):
    details = types.DictType(types.BaseType(), required=True)


class CreateSupportBundle(base_result.BaseApiResult):
    details = types.ModelType(models.SupportBundle, required=True)
    duration = types.TimedeltaStringType(required=True)
    result = types.EnumType(enums.ApiTestResult, required=True)


class GetClusterConfig(base_result.BaseApiResult):
    cluster = types.ModelType(models.ClusterConfig, required=True)


class GetClusterState(models.ClusterState, base_result.BaseApiResult):
    pass


class GetConfig(base_result.BaseApiResult):
    config = types.ModelType(models.ClusterNodeConfig, required=True)
    network = types.ModelType(models.NetworkConfig, required=True)


class GetConfigExtended(base_result.BaseApiResult):
    config = types.ModelType(models.ClusterNodeConfigExtended, required=True)


class GetBootstrapConfig(models.BootstrapConfig, base_result.BaseApiResult):
    pass


class GetDriveConfig(base_result.BaseApiResult):
    drive_config = types.ModelType(models.DriveConfig, required=True,
                                   serialized_name='driveConfig')


class GetNodeFipsDrivesReport(base_result.BaseApiResult):
    fips_drives = types.EnumType(enums.FipsDrivesStatus, required=True, serialized_name='fipsDrives')


class GetHardwareConfig(base_result.BaseApiResult):
    hardware_config = types.ModelType(
        model_spec=models.NodeHardwareConfig,
        required=True,
        serialized_name='hardwareConfig'
    )


class GetHardwareInfo(base_result.BaseApiResult):
    hardware_info = types.ModelType(
        model_spec=models.NodeHardwareInfo,
        required=True,
        serialized_name='hardwareInfo'
    )


class GetIpmiConfig(base_result.BaseApiResult):
    ipmi_config = types.ListType(
        field=types.ModelType(models.IpmiConfig),
        required=True,
        serialized_name='ipmiConfig'
    )


class GetIpmiInfo(base_result.BaseApiResult):
    ipmi_info = types.ModelType(
        model_spec=models.IpmiInfo,
        required=True,
        serialized_name='ipmiInfo'
    )


class GetNetworkConfig(base_result.BaseApiResult):
    network = types.ModelType(models.NetworkConfig, required=True)


class GetVersionInfo(base_result.BaseApiResult):
    version_info = types.ModelType(models.NodeBinaryVersionInfo, required=True,
                                   serialized_name='versionInfo')


class NodeGetNetworkInterfaceStatus(base_result.BaseApiResult):
    """Result data model for the node api GetNetworkInterfaceStatus method."""
    interfaces = types.DictType(types.EnumType(enums.NetworkStatus), required=True)


class GetPendingOperation(base_result.BaseApiResult):
    pending_operation = types.ModelType(
        model_spec=models.PendingOperation,
        required=True,
        serialized_name='pendingOperation'
    )


class GetEnsembleConfig(base_result.BaseApiResult):
    # Not included on nodes not in the quorum
    local_view_equals_cluster_view = types.BooleanType(serialized_name='localViewEqualsClusterView')
    # Not included on nodes not in the quorum
    local_view = types.ModelType(models.ZookeeperView, serialized_name='localView')
    node_info = types.ModelType(models.EnsembleNodeInfo, required=True, serialized_name='nodeInfo')
    cluster_view = types.ModelType(models.ZookeeperView, required=True, serialized_name='clusterView')
    # Not included on nodes not in the quorum
    server_stats = types.ModelType(models.ServerStats, serialized_name='serverStats')
    server_stats_detailed = types.DictType(
        types.StringType, serialized_name='serverStatsDetailed')


class GetNetworkInterface(base_result.BaseApiResult):
    interface = types.ModelType(models.NetworkInterface, required=True)


class ListNetworkInterfaces(base_result.BaseApiResult):
    interfaces = types.ListType(types.ModelType(models.NetworkInterface), required=True)


class ListDriveHardware(base_result.BaseApiResult):
    nodes = types.ListType(types.ModelType(models.NodeDriveHardware), required=True)


class ListDownloads(base_result.BaseApiResult):
    downloads = types.ListType(types.StringType, required=True)


class DeleteDownloads(ListDownloads):
    pass


class ListTests(base_result.BaseApiResult):
    tests = types.ListType(types.StringType(), required=True)


class ListUtilities(base_result.BaseApiResult):
    utilities = types.ListType(types.StringType(), required=True)


class ReadLink(base_result.BaseApiResult):
    absolute_path = types.StringType(required=True, serialized_name='absolutePath')


class ResetNode(base_result.BaseApiResult):
    details = types.ModelType(models.NodeRtfiDetails, required=True)
    duration = types.TimedeltaStringType(required=True)
    result = types.EnumType(enums.ApiTestResult, required=True)


class ResetDrives(base_result.BaseApiResult):
    details = types.ModelType(models.ResetDriveList, required=True)
    duration = types.TimedeltaStringType(required=True)
    result = types.EnumType(enums.ApiTestResult, required=True)


class RestartServices(base_result.BaseApiResult):
    details = types.StringType(required=True)
    duration = types.TimedeltaStringType(required=True)
    result = types.EnumType(enums.ApiTestResult, required=True)


class SetClusterConfig(base_result.BaseApiResult):
    cluster = types.ModelType(models.ClusterConfig, required=True)


class SetConfig(base_result.BaseApiResult):
    config = types.ModelType(models.ClusterNodeConfig, required=True)
    network = types.ModelType(models.NetworkConfig)


class SetConfigExtended(base_result.BaseApiResult):
    config = types.ModelType(models.ClusterNodeConfigExtended, required=True)


class SetNetworkConfig(base_result.BaseApiResult):
    network = types.DictType(types.ModelType(models.NetworkInterfaceDetails), required=True)


class SetNetworkInterfaceStatus(base_result.BaseApiResult):
    interfaces = types.DictType(types.EnumType(enums.NetworkStatus), required=True)


class TestConnectEnsemble(base_result.BaseApiResult):
    details = types.ModelType(models.TestConnectEnsembleDetails, required=True)
    duration = types.TimedeltaStringType(required=True)
    result = types.EnumType(enums.ApiTestResult, required=True)


class TestNodeBase(base_result.BaseApiResult):
    """Base class for node test API."""
    duration = types.TimedeltaStringType(required=True)
    result = types.EnumType(enums.ApiTestResult, required=True)


class TestConnectMvip(TestNodeBase):
    details = types.ModelType(models.NodeMvipDetails, required=True)


class TestConnectSvip(TestNodeBase):
    details = types.ModelType(models.NodeSvipDetails, required=True)


class TestHardwareConfig(TestNodeBase):
    details = types.DictType(types.ModelType(models.NodeHardwareTest, required=True))
    duration = types.TimedeltaStringType(required=True)
    result = types.EnumType(enums.ApiTestResult, required=True)


class TestLocateCluster(TestNodeBase):
    details = types.ModelType(models.ClusterLocationDetails, required=True)
    duration = types.TimedeltaStringType(required=True)
    result = types.EnumType(enums.ApiTestResult, required=True)


class TestLocalConnectivity(TestNodeBase):
    details = types.DictType(types.ModelType(models.PingResponse), required=True)
    duration = types.TimedeltaStringType(required=True)
    result = types.EnumType(enums.ApiTestResult, required=True)


class TestNetworkConfig(TestNodeBase):
    details = types.DictType(types.BaseType(), required=True)


class TestPing(TestNodeBase):
    details = types.DictType(types.ModelType(models.PingResponse), required=True)
    duration = types.TimedeltaStringType(required=True)
    result = types.EnumType(enums.ApiTestResult, required=True)


class TestRemoteConnectivity(TestNodeBase):
    details = types.DictType(types.ModelType(models.PingResponse), required=True)
    duration = types.TimedeltaStringType(required=True)
    result = types.EnumType(enums.ApiTestResult, required=True)


class GetStartupFlags(base_result.BaseApiResult):
    startup_flags = types.DictType(types.BaseType(),
                                   required=True,
                                   serialized_name='startupFlags')


class SetStartupFlags(base_result.BaseApiResult):
    startup_flags = types.DictType(types.BaseType(),
                                   required=True,
                                   serialized_name='startupFlags')


class ListPackageInfo(base_result.BaseApiResult):
    packages = types.ListType(types.ModelType(models.Package), required=True)


class InstallPackage(base_result.BaseApiResult):
    package = types.ModelType(models.Package, required=True)


class SetActivePackageVersion(base_result.BaseApiResult):
    package = types.ModelType(models.Package, required=True)


class UninstallPackage(base_result.BaseApiResult):
    package = types.ModelType(models.Package, required=True)


class ListMountedFileSystems(base_result.BaseApiResult):
    mount_entries = types.ListType(types.ModelType(models.MountedFileSystem), required=True,
                                   serialized_name='mountEntries')
