"""Result data models related to snapshots."""
from sf_api import (
    api_constants,
    models,
    sf_version,
)
from sf_api.models import types
from sf_api.result_models import base_result


class CreateGroupSnapshot(base_result.BaseApiResult):
    group_snapshot = types.ModelType(models.GroupSnapshot, required=True,
                                     version_added=sf_version.SfVersion('9.0'),
                                     serialized_name=api_constants.GROUP_SNAPSHOT)
    group_snapshot_id = types.IntIdType(required=True, serialized_name=api_constants.GROUP_SNAPSHOT_ID)
    members = types.ListType(types.ModelType(models.MemberSnapshot), required=True)


class ListGroupSnapshots(base_result.BaseApiResult):
    group_snapshots = types.ListType(types.ModelType(models.GroupSnapshot), required=True, default=list,
                                     serialized_name=api_constants.GROUP_SNAPSHOTS)


class ModifyGroupSnapshot(base_result.BaseApiResult):
    group_snapshot = types.ModelType(models.GroupSnapshot, required=True,
                                     version_added=sf_version.SfVersion('9.0'),
                                     serialized_name=api_constants.GROUP_SNAPSHOT)


class CreateSnapshot(base_result.BaseApiResult):
    snapshot = types.ModelType(models.Snapshot, required=True,
                               version_added=sf_version.SfVersion('9.0'))
    snapshot_id = types.IntIdType(required=True, serialized_name=api_constants.SNAPSHOT_ID)
    checksum = types.StringType(required=True)


class ListSnapshots(base_result.BaseApiResult):
    snapshots = types.ListType(types.ModelType(models.Snapshot), required=True, default=list)


class ModifySnapshot(base_result.BaseApiResult):
    snapshot = types.ModelType(models.Snapshot, required=True,
                               version_added=sf_version.SfVersion('9.0'))


class RollbackToGroupSnapshot(base_result.BaseApiResult):
    members = types.ListType(types.ModelType(models.MemberSnapshot), required=True, default=list)

    # Default the group_snapshot_id and group_snapshot to None, as the values will return null unless
    # you specify to take a snapshot prior to rollback.
    group_snapshot_id = types.IntIdType(required=True, default=None,
                                        serialized_name=api_constants.GROUP_SNAPSHOT_ID)
    group_snapshot = types.ModelType(models.GroupSnapshot, required=True, default=None,
                                     serialized_name=api_constants.GROUP_SNAPSHOT)


class RollbackToSnapshot(base_result.BaseApiResult):
    checksum = types.StringType(required=True)

    # Default the snapshot_id and snapshot to None, as the values will return null unless you specify
    # to take a snapshot prior to rollback.
    snapshot_id = types.IntIdType(required=True, default=None, serialized_name=api_constants.SNAPSHOT_ID)
    snapshot = types.ModelType(models.Snapshot, required=True, default=None)
