"""Test loading of REST data from healthcheck POST/GET calls."""

import json
import os
import unittest

from sf_api import (
    enums,
    models,
)

CWD, _ = os.path.split(os.path.abspath(os.path.expanduser(__file__)))
GET_RESULTS = os.path.join(CWD, 'data', 'healthcheck_get_result.json')
POST_RESULTS = os.path.join(CWD, 'data', 'healthcheck_post_result.json')
STATUS = os.path.join(CWD, 'data', 'healthcheck_status.json')


class TestHealthcheckResultModels(unittest.TestCase):
    def test_healthcheck_status(self):
        with open(STATUS) as f:
            text = f.read()
        text_json = json.loads(text)
        r = models.HealthcheckStatusModel(text_json)
        assert r.status == enums.HealthcheckStatus.running

    def test_healthcheck_post_result_initializing(self):
        with open(POST_RESULTS) as f:
            text = f.read()
        text_json = json.loads(text)
        r = models.HealthcheckResultModel(text_json)
        assert r.state == enums.HealthcheckState.initializing

    def test_healthcheck_get(self):
        with open(GET_RESULTS) as f:
            text = f.read()
        text_json = json.loads(text)
        r = models.HealthcheckResultModel(text_json)
        assert r.state == enums.HealthcheckState.error
        assert r.status.status == enums.HealthcheckStatus.running
        faults = [a for a in r.status.checks if a.name == 'check_cluster_faults']
        assert faults
        assert isinstance(faults[0], models.Healthcheck)
        assert isinstance(faults[0].data, list)

        async_res = [a for a in r.status.checks if a.name == 'check_async_results']
        assert async_res
        assert isinstance(async_res[0], models.Healthcheck)
        assert isinstance(async_res[0].data, dict)
        assert async_res[0].data['number_async_results'] == 0
