"""Test sf_api.result_models."""

import unittest

from sf_api import (
    models,
    result_models,
)

drive_removal_get = {
    'createTime': '2017-06-27T08:55:32Z',
    'lastUpdateTime': '2017-06-27T08:56:46Z',
    'result': {
        'bsDisks': [],
        'ssDisks': []
    },
    'resultType': 'DriveRemoval',
    'status': 'complete'
}

drive_add_get = {
    'createTime': '2017-06-27T08:56:53Z',
    'lastUpdateTime': '2017-06-27T08:56:55Z',
    'result': {
        'abortedDrives': [],
        'addedDrives': [
            11
        ],
        'pendingDrives': []
    },
    'resultType': 'DriveAdd',
    'status': 'complete'
}

list_async_results_data = {
    'asyncHandles': [
        {
            'asyncResultID': 2,
            'completed': True,
            'createTime': '2017-06-29T08:37:22Z',
            'data': {
                'bsDisks': [],
                'ssDisks': []
            },
            'lastUpdateTime': '2017-06-29T08:38:49Z',
            'resultType': 'DriveRemoval',
            'success': True
        },
        {
            'asyncResultID': 3,
            'completed': True,
            'createTime': '2017-06-29T08:38:53Z',
            'data': {
                'abortedDrives': [],
                'addedDrives': [
                    11
                ],
                'pendingDrives': []
            },
            'lastUpdateTime': '2017-06-29T08:38:55Z',
            'resultType': 'DriveAdd',
            'success': True
        }
    ]
}

drive_secure_erase_get = {
    'createTime': '2019-03-18T12:26:04Z',
    'lastUpdateTime': '2019-03-18T12:26:04Z',
    'result': {
        'drives': [],
        'successfulCompletion': True
    },
    'resultType': 'DriveSecureErase',
    'status': 'complete'
}


class TestResultModels(unittest.TestCase):

    def test_get_poly_type(self):
        remove_drive_async_result = result_models.GetAsyncResult(drive_removal_get)
        assert isinstance(remove_drive_async_result.details, models.RemoveDrivesDetails)

        add_drive_async_result = result_models.GetAsyncResult(drive_add_get)
        assert isinstance(add_drive_async_result.details, models.AddDrivesDetails)

        secure_erase_result = result_models.GetAsyncResult(drive_secure_erase_get)
        assert isinstance(secure_erase_result.details, models.SecureEraseDrivesDetails)

    def test_list_poly_type(self):
        list_async_results = result_models.ListAsyncResults(list_async_results_data)
        type_one = list_async_results.async_handles[0].details
        type_two = list_async_results.async_handles[1].details
        assert type_one != type_two
