"""Result data models related to volumes."""
from sf_api import (
    api_constants,
    enums,
    models,
    sf_version,
)
from sf_api.models import types
from sf_api.result_models import base_result


class AddInitiatorsToVolumeAccessGroup(base_result.BaseApiResult):
    volume_access_group = types.ModelType(
        model_spec=models.VolumeAccessGroup,
        required=True,
        serialized_name=api_constants.VOLUME_ACCESS_GROUP,
        version_added=sf_version.SfVersion('9.0')
    )


class AddVolumesToVolumeAccessGroup(base_result.BaseApiResult):
    volume_access_group = types.ModelType(
        model_spec=models.VolumeAccessGroup,
        required=True,
        serialized_name=api_constants.VOLUME_ACCESS_GROUP,
        version_added=sf_version.SfVersion('9.0')
    )


class CloneMultipleVolumes(base_result.BaseApiResult):
    async_handle = types.IntType(required=True, serialized_name=api_constants.ASYNC_HANDLE, min_value=1)
    group_clone_id = types.IntIdType(required=True, serialized_name='groupCloneID')
    members = types.ListType(types.ModelType(models.ClonedVolumeInfo), required=True)


class CloneVolume(base_result.BaseApiResult):
    async_handle = types.IntType(required=True, serialized_name=api_constants.ASYNC_HANDLE, min_value=1)
    clone_id = types.IntIdType(required=True, serialized_name=api_constants.CLONE_ID)
    curve = types.ModelType(models.QosCurve, required=True)
    volume = types.ModelType(models.Volume, required=True, version_added=sf_version.SfVersion('9.0'))
    volume_id = types.IntIdType(required=True, serialized_name=api_constants.VOLUME_ID)


class ComputeVolumeChecksum(base_result.BaseApiResult):
    checksums = types.DictType(types.StringType, required=True, serialized_name=api_constants.CHECKSUMS)


class CopyVolume(base_result.BaseApiResult):
    async_handle = types.IntType(required=True, serialized_name=api_constants.ASYNC_HANDLE, min_value=1)
    clone_id = types.IntIdType(required=True, serialized_name=api_constants.CLONE_ID)


class CreateBackupTarget(base_result.BaseApiResult):
    backup_target_id = types.IntIdType(required=True, serialized_name='backupTargetID')


class CreateInitiators(base_result.BaseApiResult):
    initiators = types.ListType(types.ModelType(models.Initiator), required=True, default=list)


class CreateMultipleVolumes(base_result.BaseApiResult):
    volume_ids = types.ListType(types.IntType, required=True, serialized_name=api_constants.VOLUME_IDS)
    volumes = types.ListType(
        field=types.ModelType(models.Volume),
        required=True,
        version_added=sf_version.SfVersion('9.0')
    )


class CreateQoSPolicy(base_result.BaseApiResult):
    qos_policy = types.ModelType(models.QosPolicy, required=True, serialized_name='qosPolicy')


class CreateVolume(base_result.BaseApiResult):
    volume_id = types.IntType(required=True, serialized_name=api_constants.VOLUME_ID)
    volume = types.ModelType(
        model_spec=models.Volume,
        required=True,
        version_added=sf_version.SfVersion('9.0')
    )
    curve = types.ModelType(models.QosCurve, required=True)


class CreateVolumeAccessGroup(base_result.BaseApiResult):
    volume_access_group = types.ModelType(
        model_spec=models.VolumeAccessGroup,
        required=True,
        serialized_name=api_constants.VOLUME_ACCESS_GROUP,
        version_added=sf_version.SfVersion('9.0')
    )
    volume_access_group_id = types.IntIdType(
        required=True,
        serialized_name=api_constants.VOLUME_ACCESS_GROUP_ID
    )


class DeleteVolume(base_result.BaseApiResult):
    volume = types.ModelType(
        model_spec=models.Volume,
        required=True,
        version_added=sf_version.SfVersion('9.0')
    )
    volume_id = types.IntType(required=True, serialized_name=api_constants.VOLUME_ID,
                              version_removed=sf_version.SfVersion('9.0'))


class DeleteVolumeAccessGroup(base_result.BaseApiResult):
    pass


class DeleteVolumes(base_result.BaseApiResult):
    volumes = types.ListType(types.ModelType(models.Volume), required=True, min_size=0, default=list)


class FillSlice(base_result.BaseApiResult):
    async_handle = types.IntType(
        required=True,
        serialized_name=api_constants.ASYNC_HANDLE
    )


class GetBackupTarget(base_result.BaseApiResult):
    backup_target = types.ModelType(models.BackupTarget, required=True, serialized_name='backupTarget')


class GetDefaultQoS(base_result.BaseApiResult):
    burst_iops = types.IntType(required=True, serialized_name=api_constants.BURST_IOPS)
    burstTime = types.IntType(required=True, serialized_name=api_constants.BURST_TIME)
    curve = types.ModelType(models.QosCurve, required=True)
    max_iops = types.IntType(required=True, serialized_name=api_constants.MAX_IOPS)
    min_iops = types.IntType(required=True, serialized_name=api_constants.MIN_IOPS)


class GetSliceInfo(base_result.BaseApiResult):
    db_version = types.IntIdType(required=True, serialized_name=api_constants.DB_VERSION)
    slice = types.ModelType(models.Slice, required=True)


class ReassignSlice(base_result.BaseApiResult):
    db_version = types.IntIdType(required=True, serialized_name=api_constants.DB_VERSION)
    slice = types.ModelType(models.Slice, required=True)


class GetQoSPolicy(base_result.BaseApiResult):
    qos_policy = types.ModelType(models.QosPolicy, required=True, serialized_name='qosPolicy')


class GetVolumeAccessGroupEfficiency(base_result.BaseApiResult):
    compression = types.DecimalType(required=True)
    deduplication = types.IntType(required=True)
    missing_volumes = types.ListType(types.IntIdType(), required=True, serialized_name='missingVolumes')
    thin_provisioning = types.DecimalType(required=True, serialized_name='thinProvisioning')
    timestamp = types.DateTimeType(required=True)


class GetVolumeCount(base_result.BaseApiResult):
    count = types.IntType(required=True)


class GetVolumeEfficiency(base_result.BaseApiResult):
    compression = types.DecimalType(required=True)
    deduplication = types.DecimalType(required=True)
    missing_volumes = types.ListType(types.IntIdType(), required=True, serialized_name='missingVolumes')
    thin_provisioning = types.DecimalType(required=True, serialized_name='thinProvisioning')
    timestamp = types.DateTimeType(required=True)


class GetVolumeStats(base_result.BaseApiResult):
    volume_stats = types.ModelType(models.VolumeStats, required=True, serialized_name='volumeStats')


class ListActiveVolumes(base_result.BaseApiResult):
    volumes = types.ListType(types.ModelType(models.Volume), required=True)


class ListBackupTargets(base_result.BaseApiResult):
    backup_targets = types.ListType(
        field=types.ModelType(models.BackupTarget),
        required=True,
        serialized_name='backupTargets'
    )


class ListBulkVolumeJobs(base_result.BaseApiResult):
    bulk_volume_jobs = types.ListType(
        field=types.ModelType(models.BulkVolumeJob),
        required=True,
        serialized_name='bulkVolumeJobs'
    )


class ListCloneJobs(base_result.BaseApiResult):
    clone_jobs = types.ListType(
        field=types.ModelType(models.CloneJob),
        required=True,
        serialized_name='cloneJobs'
    )


class ListDeletedVolumes(base_result.BaseApiResult):
    volumes = types.ListType(types.ModelType(models.Volume), required=True)


class ListInitiators(base_result.BaseApiResult):
    initiators = types.ListType(types.ModelType(models.Initiator), required=True, default=list)


class ListQoSPolicies(base_result.BaseApiResult):
    qos_policies = types.ListType(
        field=types.ModelType(models.QosPolicy),
        required=True,
        serialized_name='qosPolicies'
    )


class ListSyncJobs(base_result.BaseApiResult):
    sync_jobs = types.ListType(
        field=types.ModelType(models.SyncJob),
        required=True,
        default=list,
        serialized_name='syncJobs'
    )


class ListVolumeAccessGroups(base_result.BaseApiResult):
    volume_access_groups = types.ListType(
        field=types.ModelType(models.VolumeAccessGroup),
        required=True,
        serialized_name=api_constants.VOLUME_ACCESS_GROUPS
    )


class ListVolumeQoSHistograms(base_result.BaseApiResult):
    qos_histograms = types.ListType(
        field=types.ModelType(models.VolumeQosHistograms),
        required=True,
        serialized_name=api_constants.QOS_HISTOGRAMS
    )


class ListVolumes(base_result.BaseApiResult):
    volumes = types.ListType(types.ModelType(models.Volume), required=True, default=list)


class ListVolumesForAccount(base_result.BaseApiResult):
    volumes = types.ListType(types.ModelType(models.Volume), required=True)


class ListVolumeStatsBase(base_result.BaseApiResult):
    """Base class for list-volume-stats API."""
    volume_stats = types.ListType(
        field=types.ModelType(models.VolumeStats),
        required=True,
        default=list,
        serialized_name='volumeStats'
    )


class ListVolumeStats(ListVolumeStatsBase):
    pass


class ListVolumeStatsByAccount(ListVolumeStatsBase):
    pass


class ListVolumeStatsByVirtualVolume(ListVolumeStatsBase):
    pass


class ListVolumeStatsByVolume(ListVolumeStatsBase):
    pass


class ListVolumeStatsByVolumeAccessGroup(ListVolumeStatsBase):
    pass


class ModifyInitiator(base_result.BaseApiResult):
    initiator = types.ModelType(models.Initiator, required=True, default=list)


class ModifyInitiators(base_result.BaseApiResult):
    initiators = types.ListType(types.ModelType(models.Initiator), required=True, default=list)


class ModifyQoSPolicy(base_result.BaseApiResult):
    qos_policy = types.ModelType(models.QosPolicy, required=True, serialized_name='qosPolicy')


class ModifyVolume(base_result.BaseApiResult):
    curve = types.ModelType(models.QosCurve, required=True, version_removed=sf_version.SfVersion('9.0'))
    volume = types.ModelType(models.Volume, required=True, version_added=sf_version.SfVersion('9.0'))


class ModifyVolumeAccessGroup(base_result.BaseApiResult):
    volume_access_group = types.ModelType(
        model_spec=models.VolumeAccessGroup,
        required=True,
        serialized_name=api_constants.VOLUME_ACCESS_GROUP
    )


class ModifyVolumes(base_result.BaseApiResult):
    volumes = types.ListType(types.ModelType(models.Volume), required=True, default=list)


class PurgeAllDeletedVolumes(base_result.BaseApiResult):
    volume_ids = types.ListType(types.IntType, required=True, serialized_name=api_constants.VOLUME_IDS)
    failures = types.DictType(types.StringType, required=True, serialized_name=api_constants.FAILURES)


class ReadSliceLBA(base_result.BaseApiResult):
    block_ids = types.DictType(types.ListType(types.StringType), required=True,
                               serialized_name=api_constants.BLOCK_IDs)


class ReadSliceLBAChecksum(base_result.BaseApiResult):
    block_checksums = types.DictType(types.IntType(), default=dict,
                                     serialized_name='blockChecksums', required=True)


class RemoveInitiatorsFromVolumeAccessGroup(base_result.BaseApiResult):
    volume_access_group = types.ModelType(
        model_spec=models.VolumeAccessGroup,
        required=True,
        serialized_name=api_constants.VOLUME_ACCESS_GROUP,
        version_added=sf_version.SfVersion('9.0')
    )


class RemoveVolumesFromVolumeAccessGroup(base_result.BaseApiResult):
    volume_access_group = types.ModelType(
        model_spec=models.VolumeAccessGroup,
        required=True,
        serialized_name=api_constants.VOLUME_ACCESS_GROUP,
        version_added=sf_version.SfVersion('9.0')
    )


class SetDefaultQoS(base_result.BaseApiResult):
    burst_iops = types.IntType(required=True, serialized_name=api_constants.BURST_IOPS)
    max_iops = types.IntType(required=True, serialized_name=api_constants.MAX_IOPS)
    min_iops = types.IntType(required=True, serialized_name=api_constants.MIN_IOPS)


class StartBulkVolumeRead(base_result.BaseApiResult):
    async_handle = types.IntType(required=True, serialized_name=api_constants.ASYNC_HANDLE, min_value=1)
    key = types.StringType(required=True, regex=r'^[0-9a-z]+$')
    url = types.StringType(required=True)


class StartBulkVolumeWrite(base_result.BaseApiResult):
    async_handle = types.IntType(required=True, serialized_name=api_constants.ASYNC_HANDLE, min_value=1)
    key = types.StringType(required=True, regex=r'^[0-9a-z]+$')
    url = types.StringType(required=True)


class UpdateBulkVolumeStatus(base_result.BaseApiResult):
    status = types.EnumType(enums.AsyncResultStatus, required=True)
    url = types.StringType(required=True)


class StartVolumePairing(base_result.BaseApiResult):
    volume_pairing_key = types.StringType(required=True,
                                          serialized_name=api_constants.VOLUME_PAIRING_KEY)


class ListActivePairedVolumes(base_result.BaseApiResult):
    volumes = types.ListType(types.ModelType(models.Volume), required=True)


class CheckVolumeStandbysAssigned(base_result.BaseApiResult):
    all_standby_assigned = types.BooleanType(required=True, serialized_name='allStandbysAssigned')
    volumes_ids_with_unassigned_standbys = types.ListType(
        field=types.IntType, required=False,
        serialized_name='volumeIDsWithUnassignedStandbys')


class MovePrimariesAwayFromNode(base_result.BaseApiResult):
    pass
