import unittest

from sf_api import (
    enums,
    sf_version,
)


class TestClusterConstants(unittest.TestCase):
    def test_fields_in_version(self):
        fields_v11_3 = enums.ClusterConstants.fields_in_version(sf_version.SfVersion('11.3'))
        assert enums.ClusterConstants.auth_session_inactivity_duration not in fields_v11_3
        fields_v11 = enums.ClusterConstants.fields_in_version(sf_version.SfVersion('11.0'))
        assert enums.ClusterConstants.include_custom_pd_in_api not in fields_v11
        fields_v10 = enums.ClusterConstants.fields_in_version(sf_version.SfVersion('10.0'))
        assert enums.ClusterConstants.force_paging_sublist_count not in fields_v10
        fields_v9 = enums.ClusterConstants.fields_in_version(sf_version.SfVersion('9.0'))
        assert enums.ClusterConstants.gc_interval not in fields_v9
        fields_v8 = enums.ClusterConstants.fields_in_version(sf_version.SfVersion('8.0'))
        assert enums.ClusterConstants.disable_pcache_clean_files not in fields_v8
        fields_v7 = enums.ClusterConstants.fields_in_version(sf_version.SfVersion('7.0'))
        assert enums.ClusterConstants.disable_bin_assignment_updates not in fields_v7


class TestClusterFaultCode(unittest.TestCase):
    def test_fields_in_version(self):
        fields_v11 = enums.ClusterFaultCode.fields_in_version(sf_version.SfVersion('11.0'))
        assert enums.ClusterFaultCode.unbalanced_mixed_nodes in fields_v11
        fields_v12 = enums.ClusterFaultCode.fields_in_version(sf_version.SfVersion('12.0'))
        assert enums.ClusterFaultCode.unbalanced_mixed_nodes not in fields_v12


class TestNodeService(unittest.TestCase):
    def test_is_drive_service(self):
        assert enums.NodeService.slice.is_drive_service
        assert enums.NodeService.master.is_drive_service is False


class TestVolumeAccess(unittest.TestCase):
    def test_fields_in_version(self):
        fields_v10 = enums.VolumeAccess.fields_in_version(sf_version.SfVersion('10.0'))
        assert enums.VolumeAccess.snap_mirror_target in fields_v10
        fields_v9 = enums.VolumeAccess.fields_in_version(sf_version.SfVersion('9.0'))
        assert enums.VolumeAccess.snap_mirror_target not in fields_v9


class TestServiceType(unittest.TestCase):
    def test_is_drive_service(self):
        assert enums.ServiceType.volume.is_drive_service
        assert enums.ServiceType.transport.is_drive_service is False


class TestFeatureType(unittest.TestCase):
    def test_is_fields_in_version(self):
        fields_v10 = enums.SfFeature.fields_in_version(sf_version.SfVersion('10.0'))
        assert enums.SfFeature.vvols in fields_v10
        assert enums.SfFeature.fips_drives not in fields_v10
        fields_v11_4 = enums.SfFeature.fields_in_version(sf_version.SfVersion('11.4'))
        assert enums.SfFeature.fips_drives not in fields_v11_4
        fields_v11_5 = enums.SfFeature.fields_in_version(sf_version.SfVersion('11.5'))
        assert enums.SfFeature.fips_drives in fields_v11_5


class TestApiTestResult(unittest.TestCase):
    def test_is_passed(self):
        assert enums.ApiTestResult.passed.is_passed()
        assert enums.ApiTestResult.success.is_passed()
        assert enums.ApiTestResult.failed.is_passed() is False


class TestNvramVendor(unittest.TestCase):
    def test_is_vendor(self):
        assert enums.NvramVendor.marvell.is_marvell()
        assert enums.NvramVendor.micron_gen2.is_micron()
        assert enums.NvramVendor.radian_rms300.is_radian()
        assert enums.NvramVendor.smart_gen2.is_smart()
