import unittest

import mock

from sf_api import (
    ClusterApi,
    SfResponse,
    result_models,
)


class TestClusterApi(unittest.TestCase):
    def setUp(self):
        self.mvip = '0.0.0.0'

    def test_cluster_api_init(self):
        cluster_api = ClusterApi(self.mvip, 'user', 'pass', '12.0', False)
        assert cluster_api.mvip == self.mvip

    @mock.patch('sf_api._base.BaseJsonRpcClient._process_response')
    @mock.patch('sf_api._base.requests.Session.post')
    def test_get_api(self, mock_post, mock_process_response):
        mock_response = result_models.GetClusterVersionInfo.get_mock_response()
        mock_process_response.side_effect = [mock_response]

        cluster_api = ClusterApi(self.mvip, 'user', 'pass', '12.0', False)
        response = cluster_api.get_api()
        assert isinstance(response, SfResponse)

    @mock.patch('sf_api._base.BaseJsonRpcClient._process_response')
    @mock.patch('sf_api._base.requests.Session.post')
    def test_get_cluster_stats(self, mock_post, mock_process_response):
        mock_response = result_models.GetClusterStats.get_mock_response()
        mock_process_response.side_effect = [mock_response]

        cluster_api = ClusterApi(self.mvip, 'user', 'pass', '12.0', False)
        response = cluster_api.get_cluster_stats()
        assert isinstance(response, SfResponse)
