"""Models for build resources."""

from sf_models import (
    _base,
    at2_constants,
    sf_version,
    types,
)


class Build(_base.Base):
    """Contains info about a SolidFire build including the specific version and repository."""
    build_id = types.IntType(
        required=True,
        serialized_name=at2_constants.BUILD_ID)
    release = types.StringType(
        required=True,
        serialized_name=at2_constants.RELEASE)
    version = types.StringType(
        required=True,
        serialized_name=at2_constants.VERSION)
    element_id = types.IntType(
        required=True,
        serialized_name=at2_constants.ELEMENT_ID)
    repository_name = types.StringType(
        required=True,
        serialized_name=at2_constants.REPOSITORY_NAME)

    def __repr__(self):
        return '{}<build_id={},release={},version={},element_id={},repo_name={}>'.format(
            self.__class__.__name__, self.build_id, self.release, self.version, self.element_id,
            self.repository_name)

    @property
    def release_name(self):
        """The full release name <release>-<version>. Example: nitrogen-4.4.1.3.

        :rtype: str
        """
        return '{}-{}'.format(self.release, self.version)

    @property
    def package_name(self):
        """The full package name as refered to in apt.

        :rtype: str
        """
        return 'solidfire-san-{!s}'.format(self.release_name)

    @property
    def rtfi_name(self):
        """The full rtfi name as used in our pxe build servers.

        :rtype: str
        """
        return 'solidfire-rtfi-{!s}'.format(self.release_name)

    @property
    def sf_version(self):
        """Return the SFVersion object.

        :rtype: sf_models.SfVersion
        """
        return sf_version.SfVersion(self.version)
