"""Model for the fdva resource and related operations."""

from sf_models import (
    _base,
    _build,
    _resource_network,
    at2_constants,
    types,
)


class ResourceManagementNode(_base.Base, _base.ResourceInterface):
    """ManagementNode model.

    The Management Node (or mnode, or fdva) is a virtual node that exists on a customer's
    network and can communicate with both the cluster and the internet. This allows the fdva to:

    * send data on the customer's clusters to AIQ
    * download packages and manage upgrades on the customer's clusters
    """
    username = types.StringType(required=True, serialized_name=at2_constants.USERNAME)
    password = types.StringType(required=True, serialized_name=at2_constants.PASSWORD)
    resource_id = types.IntIdType(required=True, serialized_name=at2_constants.RESOURCE_ID)
    resource_type_id = types.IntType(required=True, serialized_name=at2_constants.RESOURCE_TYPE_ID)
    resource_type_name = types.StringType(serialized_name=at2_constants.RESOURCE_TYPE_NAME)
    mip = types.FqdnType(required=True, serialized_name=at2_constants.ONE_GIG_IP)
    mipV6 = types.IPAddressType()
    mipV4 = types.IPAddressType()
    location_id = types.IntType(required=True, serialized_name=at2_constants.LOCATION_ID)
    location_name = types.StringType(required=True, serialized_name=at2_constants.LOCATION_NAME)
    management_node_name = types.StringType(required=True, serialized_name=at2_constants.FDVA_NAME)
    network_profile = types.ModelType(_resource_network.NetworkProfile,
                                      serialized_name=at2_constants.NETWORK_PROFILE, required=True)
    build_version = types.ModelType(_build.Build, serialized_name=at2_constants.BUILD_VERSION)

    def __repr__(self):
        return '{}<management_node_name={}, mip={}>'.format(
            self.__class__.__name__, self.management_node_name, self.mip)

    @property
    def static_id(self):
        """Returns a unique identifier for the node.

        :rtype: str
        """
        return self.mip

    @static_id.setter
    def static_id(self, id_):
        """Set the static_id.

        :param str id_: new id
        """
        self.mip = id_
