"""Hypervisor related models."""

from sf_models import (
    _base,
    at2_constants,
    at2_enums,
    types,
)


class HypervisorHost(_base.Base):
    """HypervisorHost resource model."""
    name = types.StringType(required=True, serialized_name=at2_constants.NAME)
    management_ip = types.IPAddressType(required=True, serialized_name=at2_constants.MANAGEMENT_IP)
    type = types.EnumType(at2_enums.TypeEnum, required=True, serialized_name=at2_constants.TYPE)
    username = types.StringType(required=True, serialized_name=at2_constants.USERNAME)
    password = types.StringType(required=True, serialized_name=at2_constants.PASSWORD)

    def __repr__(self):
        return '{}<name={}, management_ip={}>'.format(
            self.__class__.__name__, self.name, self.management_ip)


class VirtualHardwareProfile(_base.Base):
    """VirtualHardwareProfile resource model."""
    name = types.StringType(required=True, serialized_name=at2_constants.NAME)
    image_location = types.URLType(serialized_name=at2_constants.IMAGE_LOCATION)
    cpu_count = types.IntType(required=True, serialized_name=at2_constants.CPU_COUNT)
    memory_bytes = types.IntType(required=True, serialized_name=at2_constants.MEMORY_BYTES)
    disk_bytes = types.LiteralStringListType(types.IntType(), required=True,
                                             serialized_name=at2_constants.DISK_BYTES)
    nics = types.LiteralStringListType(types.StringType(), required=True,
                                       serialized_name=at2_constants.NICS)

    def __repr__(self):
        return '{}<network_name={}, location_name={}>'.format(
            self.__class__.__name__, self.name, self.image_location)
