"""IP Data model."""
from sf_models import (
    _base,
    _resource_network,
    at2_constants,
    at2_enums,
    types,
)


class ResourceIp(_base.Base):
    """IP Resource Model class as per AT2."""
    address = types.IPAddressType(required=True)
    location_id = types.IntType(required=True, serialized_name=at2_constants.LOCATION_ID)
    network = types.EnumType(at2_enums.NetworkSpeed, required=True)
    network_profile = types.ModelType(_resource_network.NetworkProfile, required=True,
                                      serialized_name=at2_constants.NETWORK_PROFILE)
    resource_id = types.IntIdType(required=True, serialized_name=at2_constants.RESOURCE_ID)

    # AT2 Fields
    _location_name = types.StringType(required=True, serialized_name=at2_constants.LOCATION_NAME)
    _resource_type_id = types.IntType(serialized_name=at2_constants.RESOURCE_TYPE_ID)
    resource_type_name = types.StringType(required=True,
                                          serialized_name=at2_constants.RESOURCE_TYPE_NAME)

    def __repr__(self):
        return '{}<{}={}, {}={}>'.format(
            self.__class__.__name__,
            'resource_id', self.resource_id,
            'address', self.address)
