"""Upgrade data model."""

from sf_models import (
    _base,
    enums,
    types,
)


class MnodeConfig(_base.Base):
    """Contains configuration for an mnode 2.x."""
    deploy_bundle_update = types.BooleanType(required=False)
    service_bundle_url = types.URLType(required=False)
    jenkins_url = types.URLType(required=False)
    mnode_iso = types.StringType(required=False)
    mnode_iso_url = types.URLType(required=False)
    iso_upgrade_file_path = types.StringType(required=False)


class ServicesUpload(_base.Base):
    """Contains information about variables needed for the services."""
    checksum = types.StringType(serialized_name='checksum')


class About(_base.Base):
    """Response from mnode About/ endpoint."""
    mnode_bundle_version = types.StringType()
    mnode_host_ip = types.IPAddressType()
    token_url = types.URLType()
    version = types.StringType()


class MnodeBundleLogSettingsModel(_base.Base):
    """Contains information about variables needed for retrieving mnode bundle logs."""
    compute_ids = types.ListType(types.UUIDType, serialized_name='computeIds')
    compute_logs = types.BooleanType(serialized_name='computeLogs', default=True)
    controller_ids = types.ListType(types.UUIDType, serialized_name='controllerIds')
    mnode_logs = types.BooleanType(serialized_name='mnodeLogs', default=True)
    modified_since = types.DateTimeType(serialized_name='modifiedSince')
    resource_link = types.StringType(serialized_name='resourceLink')
    storage_crash_dump = types.BooleanType(serialized_name='storageCrashDumps', default=False)
    storage_ids = types.ListType(types.UUIDType, serialized_name='storageIds')
    storage_logs = types.BooleanType(serialized_name='storageLogs', default=True)
    storage_node_ids = types.ListType(types.UUIDType, serialized_name='storageNodeIds')


class MnodeBundleReportModel(_base.Base):
    """Contains information about mnode bundle log collection report."""
    host_name = types.StringType(serialized_name='hostName')
    ip_address = types.IPAddressType(serialized_name='ipAddress')
    state = types.EnumType(enums.BundleState, required=True, serialized_name='state')
    summary = types.StringType(required=True, serialized_name='summary')
    type = types.EnumType(enums.BundleReportType, required=True, serialized_name='type')


class MnodeTaskMonitorModel(_base.Base):
    """Contains information about Task monitor while support bundle generation is in progress."""
    percent_completed = types.IntType(serialized_name='percentComplete')
    resource_actions = types.ListType(types.StringType, serialized_name='resourceActions')
    resource_link = types.StringType(serialized_name='resourceLink')
    service_name = types.StringType(serialized_name='serviceName')
    state = types.EnumType(enums.BundleState, serialized_name='state')
    step = types.StringType(serialized_name='step')
    task_id = types.UUIDType(serialized_name='taskId')
    task_name = types.StringType(serialized_name='taskName')
    time_started = types.DateTimeType(serialized_name='timeStarted')
    time_updated = types.DateTimeType(serialized_name='timeUpdated')


class MnodeBundleLogsResponseModel(_base.Base):
    """Contains information about mnode bundle log API response."""
    links = types.DictType(types.StringType(), serialized_name='_links')
    download_link = types.StringType(serialized_name='downloadLink')
    end_date = types.DateTimeType(serialized_name='endDate')
    reports = types.ListType(types.ModelType(MnodeBundleReportModel), serialized_name='reports')
    settings = types.ModelType(MnodeBundleLogSettingsModel, serialized_name='settings')
    size = types.StringType(serialized_name='size')
    size_bytes = types.IntType(serialized_name='sizeBytes')
    start_date = types.DateTimeType(serialized_name='startDate')
    state = types.EnumType(enums.BundleState, required=True, serialized_name='state')
    summary = types.StringType(serialized_name='summary')
    task_monitor = types.ModelType(MnodeTaskMonitorModel, serialized_name='taskMonitor')
    time_period = types.StringType(serialized_name='timePeriod')


class MnodeBundleTaskResponseModel(_base.Base):
    """Contains information about mnode create bundle task API response."""
    links = types.DictType(types.StringType(), serialized_name='_links')
    task_id = types.UUIDType(serialized_name='taskId')
    task_link = types.StringType(required=True, serialized_name='taskLink')


class PackageInfo(_base.Base):
    """Response from the package service Package endpoint."""
    cifs_url = types.URLType(serialized_name='CIFSUrl', default=None)
    https_url = types.URLType(serialized_name='HTTPSUrl', default=None)
    nfs_url = types.URLType(serialized_name='NFSUrl', default=None)
    created = types.StringType()
    package_id = types.UUIDType(serialized_name='id')
    latest = types.BooleanType(default=False)
    manifest = types.BaseType(default={})
    name = types.StringType()
    version = types.StringType()


class PackageServiceStatus(_base.Base):
    """Status response from the package service."""
    cifs_url = types.URLType(serialized_name='CIFSUrl', default=None)
    https_url = types.URLType(serialized_name='HTTPSUrl', default=None)
    nfs_url = types.URLType(serialized_name='NFSUrl', default=None)
    created = types.StringType()
    downloaded = types.StringType()
    file_size = types.StringType(serialized_name='fileSize')
    package_id = types.UUIDType(serialized_name='id')
    name = types.StringType()
    percent_data_transferred = types.StringType(serialized_name='percentDataTransferred')
    remaining = types.StringType()
    state = types.StringType()
    version = types.StringType()
