"""Cluster data models."""
from sf_models import (
    _base,
    at2_constants,
    types,
)


class ResourceCluster(_base.Base):
    """AT2 Cluster Model.

    The mvip is an IPAddressType type to allow valence to work with docker clusters. Both 127.0.0.1 and
    127.0.0.1:8000 should validate. This may change as SDS progresses.
    """
    mvip = types.IPAddressType(required=True)
    rep_count = types.IntType(required=True, serialized_name=at2_constants.REP_COUNT)
    svip = types.IPAddressType(required=True)

    username = types.StringType(required=True)
    password = types.StringType(required=True)
    api_endpoint = types.SfApiVersionType(required=True, serialized_name=at2_constants.API_ENDPOINT)
    name = types.StringType(required=True, serialized_name=at2_constants.CLUSTER_NAME)
    unique_id = types.StringType(serialized_name=at2_constants.UNIQUE_ID)

    def __repr__(self):
        return '{}<name={}, mvip={}, svip={}>'.format(
            self.__class__.__name__, self.name, self.mvip, self.svip)

    @property
    def static_id(self):
        """Returns a unique identifier for the cluster.

        :rtype: str
        """
        return self.mvip
