"""Model for the network profile."""
from sf_models import (
    _base,
    at2_constants,
    types,
)


class ResourceSubnet(_base.Base):
    """Model to reflect the Subnet Resource."""
    name = types.StringType(required=True)
    start = types.IPAddressType(required=True)
    cidr = types.IntType(required=True, min_value=1, max_value=32)
    gateway = types.IPAddressType(required=True)
    virtual_network_tag = types.IntType(required=True, min_value=0, max_value=4095)
    namespace = types.BooleanType(required=True)

    def __repr__(self):
        return "{}<name={}, start={}, cidr={}, gateway={}, virtual_network_tag={}, namespace={}>".format(
            self.__class__.__name__,
            self.name,
            self.start,
            self.cidr,
            self.gateway,
            self.virtual_network_tag,
            self.namespace
        )


class NetworkProfile(_base.Base):
    """NetworkProfile resource model."""
    gateway_10g = types.IPAddressType(required=True, serialized_name=at2_constants.GATEWAY_10G)
    gateway_1g = types.IPAddressType(required=True, serialized_name=at2_constants.GATEWAY_1G)
    gateway_1gv6 = types.IPv6Type(serialized_name=at2_constants.GATEWAY_1GV6)
    image_listing_1g = types.URLType(required=True, serialized_name=at2_constants.IMAGE_LISTING_1G)
    image_server_1g = types.IPAddressType(required=True, serialized_name=at2_constants.IMAGE_SERVER_1G)
    jenkins_1g = types.StringType(required=True, serialized_name=at2_constants.JENKINS_1G)
    location_id = types.IntType(required=True, serialized_name=at2_constants.LOCATION_ID)
    location_name = types.StringType(required=True, serialized_name=at2_constants.LOCATION_NAME)
    name_search_1g = types.StringType(required=True, serialized_name=at2_constants.NAME_SEARCH_1G)
    name_server_1g = types.IPv4Type(required=True, serialized_name=at2_constants.NAME_SERVER_1G)
    name_search_10g = types.StringType(required=False, serialized_name=at2_constants.NAME_SEARCH_10G)
    name_server_10g = types.IPv4Type(required=False, serialized_name=at2_constants.NAME_SERVER_10G)
    netmask_10g = types.IPv4Type(required=True, serialized_name=at2_constants.NETMASK_10G)
    netmask_1g = types.IPv4Type(required=True, serialized_name=at2_constants.NETMASK_1G)
    network_id = types.IntType(required=True, serialized_name=at2_constants.NETWORK_ID)
    network_name = types.StringType(required=True, serialized_name=at2_constants.NETWORK_NAME)
    one_gig_vlan = types.StringType(required=True, serialized_name=at2_constants.ONE_GIG_VLAN)
    pxe_server_address = types.StringType(required=True,
                                          serialized_name=at2_constants.PXE_SERVER_ADDRESS)
    pxe_server_config_path = types.StringType(required=True,
                                              serialized_name=at2_constants.PXE_SERVER_CONFIG_PATH)
    pxe_server_images = types.StringType(required=True, serialized_name=at2_constants.PXE_SERVER_IMAGES)
    pxe_server_password = types.StringType(required=True,
                                           serialized_name=at2_constants.PXE_SERVER_PASSWORD)
    pxe_server_username = types.StringType(required=True,
                                           serialized_name=at2_constants.PXE_SERVER_USERNAME)
    resource_id = types.IntIdType(serialized_name=at2_constants.RESOURCE_ID)
    resource_type_id = types.IntType(required=True, serialized_name=at2_constants.RESOURCE_TYPE_ID)
    subnet_10g = types.IPAddressType(required=True, serialized_name=at2_constants.SUBNET_10G)
    subnet_1g = types.IPAddressType(required=True, serialized_name=at2_constants.SUBNET_1G)
    ten_gig_vlan = types.StringType(required=True, serialized_name=at2_constants.TEN_GIG_VLAN)

    def __repr__(self):
        return '{}<network_name={}, location_name={}>'.format(
            self.__class__.__name__, self.network_name, self.location_name)
