"""VirtualNetwork model that is needed by suite resources.

This is a SolidFire data model, but it is also used by the suite resources
which means that it needs to be part of the camelot package.
"""

import sf_models
from sf_models import types


class AddressBlocks(sf_models.Base):
    """AddressBlocks data model.

    Corresponds to an element of the VirtualNetwork data model.
    """
    available = types.StringType()
    size = types.DataSizeType(required=True)
    start = types.IPAddressType(required=True)

    def __repr__(self):
        return '{}<{}={}, {}={}>'.format(
            self.__class__.__name__,
            'size', self.size,
            'start', self.start
        )


class VirtualNetwork(sf_models.Base):
    """VirtualNetwork data model.

    Corresponds to the virtual network object in ListVirtualNetworks response.
    """
    address_blocks = types.ListType(types.ModelType(AddressBlocks), required=True,
                                    serialized_name='addressBlocks')
    attributes = types.DictType(types.BaseType(), required=True)
    name = types.StringType(required=True)
    namespace = types.BooleanType(required=True)
    netmask = types.IPAddressType(required=True)
    svip = types.IPAddressType(required=True)
    gateway = types.IPAddressType(required=True)
    virtual_network_id = types.IntType(required=True, serialized_name='virtualNetworkID')
    virtual_network_tag = types.IntType(required=True, serialized_name='virtualNetworkTag')

    def __repr__(self):
        return '{}<{}={}, {}={}, {}={}>'.format(
            self.__class__.__name__,
            'name', self.name,
            'svip', self.svip,
            'virtual_network_tag', self.virtual_network_tag)
