"""Test case that has attributes specific to testing element OS."""
import abc

import six

import camelot
from valence.functional_models import monitor_decorators


@six.add_metaclass(abc.ABCMeta)
class ElementTestCase(camelot.TestCase):

    @property
    def monitor_fail_on_faults(self):
        """Returns a list of faults that always cause a test to fail.

        Override this function to add always fail faults to your test case.  These events will be
        added to the default list of always fail faults.

        :return: list of faults to always fail on.
        :rtype: list[valence.enums.ClusterFaultCode]
        """
        return []

    @property
    def monitor_fail_on_events(self):
        """Returns a list of events that always cause a test to fail.

        Override this function to add always fail events to your test case. These events are
        added to the default list of always fail events.

        :return: list of events to always fail on.
        :rtype: list[valence.enums.ClusterEventType]
        """
        return []

    @property
    def monitor_ignore_faults(self):
        """Returns a list of faults that will not cause a test to fail.

        Override this function to add faults that will be ignored should they cause a failure
        to you test case.

        :rtype: list[valence.enums.ClusterFaultCode]
        """
        return []

    @property
    def monitor_ignore_events(self):
        """Returns a list of events that will not cause a test to fail.

        Override this function to add events that will be ignored if they cause a failure,
        to you test case.

        :rtype: list[valence.enums.ClusterEventType]
        """
        return []

    def _log_step(self, action, expected_result=''):
        """Log the step info with an optional expected result.

        :param str action: The step action. E.g. 'Power off node'. What action it takes to get there.
        :param str expected_result: What this test step intends to accomplish. E.g. Induce bin sync
        """
        super(ElementTestCase, self)._log_step(action=action, expected_result=expected_result)
        log_string = camelot.step_banner(
            camelot.FMT_TEST_STEP.format(index=self._current_step.index,
                                         name=self.name,
                                         action=self._current_step.action,
                                         expect=self._current_step.expected_result
                                         ))
        backend_monitor = monitor_decorators.get_backend_monitor()
        if backend_monitor and backend_monitor.using_custom_logger:
            backend_monitor.log.info(log_string)
