"""Unit tests for ElementTestCase."""

import mock

from valence import (
    functional_models,
    models,
)
from valence.monitors import BackendMonitor
from valence.unittest_utils import (
    cases,
    helpers,
)


class TestBaseCase(helpers.MockHistoryUnitTest):

    def setUp(self):
        super(TestBaseCase, self).setUp()

    @mock.patch.object(functional_models.monitor_decorators, 'get_backend_monitor')
    def test_log_step_method(self, mock_get_backend_monitor):
        case = cases.PassingElementCase()
        cluster_resource = models.ClusterSuiteResource.get_mock_object()
        resources = models.SuiteResources.create(clusters=[cluster_resource])
        monitor = BackendMonitor(resources)
        monitor.using_custom_logger = True
        mock_get_backend_monitor.return_value = monitor
        assert case._step_index == -1
        case._log_step('action')
        assert case._step_index == 0
        assert case._current_step is case._steps_ran[0]
        case._log_step('action')
        assert case._step_index == 1
        assert case._current_step is case._steps_ran[1]

    def test_element_test_case_attributes(self):
        case = cases.PassingElementCase()
        assert case.monitor_ignore_events == []
        assert case.monitor_ignore_faults == []
        assert case.monitor_fail_on_faults == []
        assert case.monitor_fail_on_events == []
