"""Test Component Base Classes.

All test components are one of 3 types:

permanent
    cannot be reset once applied

reversible
    can be reset once applied

transient
    by applying, the state will be back to normal

The method by which they are applied can be defined one of two ways:

blocking
    you define the blocking (synchronous) implementation, the
    nonblocking (asynchronous, concurrent) implementation will be
    created for you.

nonblocking
    you define the nonblocking (asynchronous, concurrent)
    implementation, the blocking (synchronous) implementation will be
    created for you.

The classes you create must inherit from exactly one component-type,
and exactly one apply-type and the abstract methods must be defined.
"""
from valence.components import _constants
from valence.components._continuous import (
    ContinuousReversible,
    ContinuousTransient,
)
from valence.components._enums import States
from valence.components._errors import (
    ComponentAsyncError,
    ComponentStateError,
    ComponentTimeoutError,
    PartialStateError,
    ResourceStateError,
    UnknownBlockIDError,
)
from valence.components._killable import (
    BlockingApplyKillable,
    NonBlockingApplyKillable,
)
from valence.components._mock import MockComponentMixin
from valence.components._processed import (
    ProcessedBlockingApply,
    ProcessedBlockingApplyKillable,
    ProcessedReversible,
)
from valence.components._threaded import (
    BlockingApply,
    Component,
    NonBlockingApply,
    Permanent,
    Reversible,
    Transient,
)
