"""Valence enums.

Some enums are imported from other packages so that users of the api do not have to import
another package in order to use Valence. The enums for Valence are in this package since multiple
subpackages (e.g. functional_models, vl_helpers) may use them.

isort:skip_file
"""

# noinspection PyUnresolvedReferences
import aenum

# noinspection PyUnresolvedReferences
from sf_api.data_size import DataUnit  # pylint: disable=unused-import
# pylint: disable=wildcard-import
# pylint: disable=unused-wildcard-import
from sf_api.enums import *  # pylint: disable=unused-import


class HtmlReport(aenum.Enum):
    """Enum for cluster HTML reports."""
    bins = 'bins'
    bs_disk_data = 'bsdiskdata'
    bs_maps = 'bsmaps'
    bs_queues = 'bsqueues'
    clones = 'clones'
    ensemble = 'ensemble'
    events = 'events'
    flushing = 'flushing'
    iscsi = 'iscsi'
    memory = 'memory'
    mutexes = 'mutexes'
    qos = 'qos'
    rpc = 'rpc'
    rpc_time = 'rpcTime'
    rpc_time_reset = 'rpcTimeReset'
    rrbdt = 'rrbdt'
    scsi_commands = 'scsiCommands'
    scsi_commands_and_sessions = 'scsiCommandsAndSessions'
    scsi_nexuses = 'scsiNexuses'
    scsi_reservations = 'scsiReservations'
    scsi_volumes = 'scsiVolumes'
    slice_balance = 'slicebalance'
    slice_filesystem = 'slicefilesystem'
    slice_info = 'sliceinfo'
    slices = 'slices'
    slice_syncing = 'slicesyncing'
    snapmirror = 'snapmirror'
    socket_service = 'socketservice'
    ss = 'ss'
    ss_director = 'ssdirector'
    timing = 'timing'
    unresponsive = 'unresponsive'
    vvol_bindings = 'vvolBindings'
    bin_syncing = 'binsyncing'
    iscsi_details = 'iscsidetails'
    scsi_details = 'scsidetails'


class IscsiMode(aenum.Enum):
    """Enum for valid iscsi modes."""
    batch = 'batch'
    all = 'all'


class NodeStatus(aenum.Enum):
    """Enum for different statuses of nodes.

    Used to filter nodes in code such as the node selector.
    """
    active = 'Active'
    pending_active = 'PendingActive'
    pending = 'Pending'


class UnixSignal(aenum.Enum):
    """Used to specify which signal to use when calling Unix commands."""
    SIGABRT = 6
    SIGALRM = 14
    SIGBUS = 7
    SIGCHLD = 17
    SIGCONT = 18
    SIGDFL = 0
    SIGFPE = 8
    SIGHUP = 1
    SIGILL = 4
    SIGINT = 2
    SIGIO = 29
    SIGKILL = 9
    SIGPIPE = 13
    SIGPOLL = 29
    SIGPROF = 27
    SIGPWR = 30
    SIGQUIT = 3
    SIGSEGV = 11
    SIGSTKFLT = 16
    SIGSTOP = 19
    SIGSYS = 31
    SIGTERM = 15
    SIGTRAP = 5
    SIGTSTP = 20
    SIGTTIN = 21
    SIGTTOU = 22
    SIGURG = 23
    SIGUSR1 = 10
    SIGUSR2 = 12
    SIGVTARLM = 26
    SIGWINCH = 28
    SIGXCPU = 24
    SIGXFSZ = 25


class SliceHealth(aenum.Enum):
    """Used to determine what state a slice assignment is in."""
    good = 'good'
    bad = 'bad'


class OobPowerState(aenum.Enum):
    """Node OOB/iDRAC power state."""
    on = 'ON'
    off = 'OFF'


class PortState(aenum.Enum):
    """Port states for bourbon sdk."""
    up = 'UP'
    down = 'DOWN'


class SwitchType(aenum.Enum):
    """Switch Types in bourbon sdk."""
    ethernet = 'ETHERNET'
    fiber = 'FIBER'


class BourbonBondMode(aenum.Enum):
    """Bond modes in bourbon sdk."""
    active = 'active'
    passive = 'passive'


class BourbonInterfaceType(aenum.Enum):
    """Types of interfaces in bourbon sdk."""
    switch_port = 'switch_port'
    port_channel = 'port_channel'
    vlan = 'vlan'


class HealthcheckName(aenum.enum):
    """Names of healthchecks in sfupgradecheck."""
    check_pending_nodes = 'check_pending_nodes'
    check_cluster_faults = 'check_cluster_faults'
    check_async_results = 'check_async_results'
    check_mnode_connectivity = 'check_mnode_connectivity'
    check_root_partition_space = 'check_root_disk_space'
    check_mnode_connection_speed = 'check_upload_speed'
    check_options_files = 'check_files'
    check_cores = 'check_cores'
    check_paging = 'check_paging_conversion'


class IdpType(aenum.enum):
    """Types of third party identity providers."""
    adfs = 'ADFS'
    shibboleth = 'Shibboleth'
