"""Example test suite for Camelot documentation purposes.

This suite inherits from camelot.TestSuite and converts the resources to the legacy ones.
"""
from valence import (
    camelot,
    functional_models,
)
from valence.examples import example_case
from valence.vl_components import vdbench


class ExampleSuite(camelot.TestSuite):
    """Example test suite that uses the vdbench component.

    This inherits from camelot.TestSuite and converts resources to the legacy data models.

    :param valence.models.SuiteResources resources: Suite resources data model
    :param valence.models.SuiteConfig config: Suite config data model
    """
    def __init__(self, resources, config):
        super(ExampleSuite, self).__init__(resources, config)

        # Grab needed data models from resources object
        self.cluster_dm = self.resources.cluster_dms[0]
        self.client_dms = self.resources.client_dms
        self.artifact_credentials = self.config.artifact_credentials

        self.cluster_fm = functional_models.Cluster(self.cluster_dm, authenticate=False)

        # Instantiate components
        self.vdb_profile = vdbench.StandardVdbProfile.from_clients(
            'suite-vdb',
            self.cluster_dm,
            self.client_dms[:3]
        )
        self.setup_vdb_env = self.vdb_profile.get_environment()
        self.manage_vdb_cmt = vdbench.ManageRunVdbs.from_environment_cmt(
            environments_cmt=self.setup_vdb_env
        )

        self.add_tests([example_case.ExampleCase(
            self.cluster_dm,
            self.manage_vdb_cmt,
            self.artifact_credentials
        )])

    def setup(self):
        """Auth cluster and apply manage vdb component."""
        self.cluster_fm.authenticate()
        self.manage_vdb_cmt.ensure_applied()

    def teardown(self):
        """Teardown manage vdb component."""
        self.manage_vdb_cmt.ensure_ready()
