"""CentOS client functional model."""
import autologger
from sf_models.at2_enums import ClientOS

from valence.functional_models._client import linux_client

_log = autologger.get_sflogging()

# start commands
ISCSI_D_START = '{};{};{}'.format('systemctl start iscsid.socket',
                                  'systemctl start iscsi.service',
                                  'systemctl start iscsid.service')
ISCSI_D_START_6 = '/etc/rc.d/init.d/iscsid start'

# restart commands
ISCSI_D_RESTART = 'systemctl restart iscsi.service'
ISCSI_D_RESTART_6 = '/etc/rc.d/init.d/iscsid restart'

# stop commands
ISCSI_D_STOP = '{};{};{}'.format('systemctl stop iscsi.service',
                                 'systemctl stop iscsid.socket',
                                 'systemctl stop iscsid.service')
ISCSI_D_STOP_6 = '/etc/rc.d/init.d/iscsid stop'


class CentOSClient(linux_client.LinuxClient):
    def iscsid_stop(self, iscsi_root_volume=False):
        """Stop iscsid.

        :param bool iscsi_root_volume: Is the volume the iscsi root
        """
        _log.debug('Stop iscsid on {}'.format(self))
        self._check_iscsi_initramfs(iscsi_root_volume=iscsi_root_volume)
        cmd = ISCSI_D_STOP
        if self.version is ClientOS.centos_6:
            cmd = ISCSI_D_STOP_6
        self._run_command_check_output(cmd)
        self.wait_until_iscsid('down')

    def iscsid_start(self, restart=False):
        """Start open-iscsi.

        :param bool restart: Restart iscsid
        """
        _log.debug('Start iscsid on {}'.format(self))

        if self.version is ClientOS.centos_6:
            cmd = ISCSI_D_RESTART_6 if restart else ISCSI_D_START_6
        else:
            cmd = ISCSI_D_RESTART if restart else ISCSI_D_START
        self._run_command_check_output(cmd)
