"""Helper function for constructing client functional models from a data model."""

from valence.functional_models._client import (
    centos_client,
    ubuntu_client,
    windows_client
)


def get_client_from_dm(client_dm):
    """Returns client functional model based on data model.

    :param valence.models.ResourceClient client_dm: client data model
    :rtype: valence.functional_models.LinuxClient
    """
    if client_dm.os.is_ubuntu:
        return ubuntu_client.UbuntuClient(client_dm)
    elif client_dm.os.is_centos:
        return centos_client.CentOSClient(client_dm)
    elif client_dm.os.is_windows:
        return windows_client.WindowsClient(client_dm)
    else:
        raise ValueError('Unsupported Client OS')
