"""Ubuntu client functional model."""
import autologger
from sf_models import at2_enums

from valence.functional_models._client import linux_client

_log = autologger.get_sflogging()

OS_NOT_SUPPORTING_SYSTEMCTL = [
    at2_enums.ClientOS.ubuntu_12_04,
    at2_enums.ClientOS.ubuntu_14_04,
]


class UbuntuClient(linux_client.LinuxClient):
    """Ubuntu Client functional model."""

    def iscsid_start(self, restart=False):
        """Start open-iscsi.

        :param bool restart: Restart iscsid
        """
        _log.debug('Start iscsid on {}'.format(self))
        os_version = self.version
        if os_version in OS_NOT_SUPPORTING_SYSTEMCTL:
            cmd = ('/etc/init.d/open-iscsi restart' if restart else
                   '/etc/init.d/open-iscsi start')
        else:
            cmd = ('systemctl restart iscsid open-iscsi' if restart else
                   'systemctl start iscsid open-iscsi')
        self._run_command_check_output(cmd)

        # Since it's Ubuntu, wait for iscsi to start
        _log.debug('We found a non RedHat variant, so we are waiting for iscsid to have pids...')
        self.wait_until_iscsid('up')

    def iscsid_stop(self, iscsi_root_volume=False):
        """Stop iscsid.

        :param bool iscsi_root_volume: Is the volume the iscsi root
        """
        _log.debug('Stop iscsid on {}'.format(self))
        os_version = self.version
        self._check_iscsi_initramfs(iscsi_root_volume=iscsi_root_volume)
        if os_version in OS_NOT_SUPPORTING_SYSTEMCTL:
            cmd = '/etc/init.d/open-iscsi stop'
        else:
            cmd = 'systemctl stop iscsid open-iscsi'
        self._run_command_check_output(cmd)
        self.wait_until_iscsid('down')
