"""Cluster FM methods related to Degraded Redundant Metadata (DRuM) APIs ."""
from sf_api import (
    api_constants,
    result_models,
)
from valence import enums
from valence.functional_models._cluster import base


class DrumMixin(base.ClusterBase):
    """Cluster FM Drum methods."""

    def check_volume_standbys_assigned(self, node_id):
        """Check if standbys have been assigned to all volumes managed by a node.

        :param int node_id: Node ID to check if standby has been assigned.
        :rtype: sf_api.result_models.CheckVolumeStandbysAssigned
        """

        response = self.api.call_method(enums.ClusterMethod.check_volume_standbys_assigned,
                                        params={api_constants.NODE_ID: node_id},
                                        result_dm_class=result_models.CheckVolumeStandbysAssigned)
        return response.result_dm

    def enable_standby_slice_assignments(self):
        """Instruct the slice balancer to assign a standby to each slice.

        :rtype: sf_api.result_models.EnableStandbySliceAssignments
        """

        response = self.api.call_method(enums.ClusterMethod.enable_standby_slice_assignments,
                                        params={},
                                        result_dm_class=result_models.EnableStandbySliceAssignments)
        return response.result_dm

    def disable_standby_slice_assignments(self):
        """Instruct the slice balancer to unassign all standby slices.

        :rtype: sf_api.result_models.DisableStandbySliceAssignments
        """

        response = self.api.call_method(enums.ClusterMethod.disable_standby_slice_assignments,
                                        params={},
                                        result_dm_class=result_models.DisableStandbySliceAssignments)
        return response.result_dm

    def recover_dead_volumes(self, volume_ids):
        """Recover dead volumes via DRuM.

        :param list[int] volume_ids: ID of volumes to recover
        :rtype: sf_api.result_models.RecoverDeadVolumes
        """

        response = self.api.call_method(enums.ClusterMethod.recover_dead_volumes,
                                        params={api_constants.VOLUME_IDS: volume_ids},
                                        result_dm_class=result_models.RecoverDeadVolumes)
        return response.result_dm
