from __future__ import division

import random
import unittest

import mock

import sf_api
from sf_api import (
    api_constants,
    result_models,
)
from valence import (
    enums,
    functional_models,
    models,
    unittest_utils,
)


@mock.patch('sf_api.ClusterApi.call_method',
            side_effect=unittest_utils.helpers.check_params_serializable)
class TestDrumMixin(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.cluster_fm = functional_models.Cluster.get_mock_fm()

    def test_check_volume_standbys_assigned(self, mock_call_method):
        node_id = 1
        self.cluster_fm.check_volume_standbys_assigned(node_id)
        mock_call_method.assert_called_with(
            enums.ClusterMethod.check_volume_standbys_assigned, params={api_constants.NODE_ID: node_id},
            result_dm_class=result_models.CheckVolumeStandbysAssigned)

    def test_check_volume_standbys_assigned_result_model(self, mock_call_method):
        node_id = 1
        raw_data = {sf_api.api_constants.RESULT: dict(allStandbysAssigned=False,
                                                      volumeIDsWithUnassignedStandbys=[306]
                                                      )}
        resp = sf_api.SfResponse(method=enums.ClusterMethod.check_volume_standbys_assigned,
                                 raw_data=raw_data,
                                 result_dm_class=sf_api.result_models.CheckVolumeStandbysAssigned)
        mock_call_method.return_value = resp
        result = self.cluster_fm.check_volume_standbys_assigned(node_id)
        assert result.serialize() == raw_data[api_constants.RESULT]
        assert isinstance(repr(result), str)

    def test_disable_standby_slice_assignments(self, mock_call_method):
        self.cluster_fm.disable_standby_slice_assignments()
        mock_call_method.assert_called_with(
            enums.ClusterMethod.disable_standby_slice_assignments, params={},
            result_dm_class=result_models.DisableStandbySliceAssignments)

    def test_enable_standby_slice_assignments(self, mock_call_method):
        self.cluster_fm.enable_standby_slice_assignments()
        mock_call_method.assert_called_with(
            enums.ClusterMethod.enable_standby_slice_assignments, params={},
            result_dm_class=result_models.EnableStandbySliceAssignments)

    def test_recover_dead_volumes(self, mock_call_method):
        self.cluster_fm.recover_dead_volumes([1, 2, 3])
        mock_call_method.assert_called_with(
            enums.ClusterMethod.recover_dead_volumes,
            params={api_constants.VOLUME_IDS: [1, 2, 3]},
            result_dm_class=result_models.RecoverDeadVolumes
        )
