"""
This file tests all the methods defined in
valence/functional_models/_cluster.protection_domain_mixin.py.
"""

import unittest

import mock

from sf_api import (
    api_constants,
    result_models,
)
from valence import (
    enums,
    functional_models,
    models,
)


@mock.patch('sf_api.ClusterApi.call_method')
class TestProtectionDomainMixin(unittest.TestCase):
    """Methods associated with protection domains."""

    @classmethod
    def setUpClass(cls):
        cls.cluster_fm = functional_models.Cluster.get_mock_fm()

    def test_get_protection_domain_layout(self, mock_call_method):
        mock_response = result_models.GetProtectionDomainLayout.get_mock_response()
        mock_call_method.return_value = mock_response
        result = self.cluster_fm.get_protection_domain_layout()
        assert isinstance(result[0], models.NodeProtectionDomainLayout)

    def test_set_chassis_protection_domains(self, mock_call_method):
        mock_call_method.return_value = result_models.GetProtectionDomainLayout.get_mock_response()
        protection_domain_dm = models.NodeProtectionDomainLayout.get_mock_object()
        self.cluster_fm.set_protection_domain_layout_chassis_override(
            node_protection_domain_layouts=[protection_domain_dm])
        expected_call_param = {api_constants.PROTECTION_DOMAIN_LAYOUT: [
                                   {api_constants.NODE_ID: nd.node_id,
                                    api_constants.PROTECTION_DOMAINS:
                                        [pd.serialize() for pd in nd.protection_domains if
                                         pd.protection_domain_type is
                                         enums.ProtectionDomainType.chassis]
                                    } for nd in [protection_domain_dm]]}
        mock_call_method.assert_called_with(
            enums.ClusterMethod.set_protection_domain_layout_chassis_override,
            params=expected_call_param,
            result_dm_class=result_models.SetProtectionDomainLayout,
            auth=mock.ANY, log_response=True,
        )

    def test_set_custom_protection_domains(self, mock_call_method):
        mock_call_method.return_value = result_models.GetProtectionDomainLayout.get_mock_response()
        protection_domain_dm = models.NodeProtectionDomainLayout.get_mock_object()
        self.cluster_fm.set_protection_domain_layout(
            node_protection_domain_layouts=[protection_domain_dm])
        expected_call_param = {api_constants.PROTECTION_DOMAIN_LAYOUT: [
            {api_constants.NODE_ID: nd.node_id,
             api_constants.PROTECTION_DOMAINS:
                 [pd.serialize() for pd in nd.protection_domains if
                  pd.protection_domain_type is
                  enums.ProtectionDomainType.custom]
             } for nd in [protection_domain_dm]]}
        mock_call_method.assert_called_with(enums.ClusterMethod.set_protection_domain_layout,
                                            params=expected_call_param,
                                            result_dm_class=result_models.SetProtectionDomainLayout,
                                            auth=mock.ANY, log_response=True)

    def test_list_protection_domain_levels(self, mock_call_method):
        mock_response = result_models.ListProtectionDomainLevels.get_mock_response()
        mock_call_method.return_value = mock_response
        result = self.cluster_fm.list_protection_domain_levels()
        assert isinstance(result[0], models.ProtectionDomainLevel)

    def test_list_protection_domains(self, mock_call_method):
        mock_response = result_models.ListProtectionDomains.get_mock_response()
        mock_call_method.return_value = mock_response
        result = self.cluster_fm.list_protection_domains()
        assert isinstance(result[0], models.PrivateProtectionDomain)

    def test_set_protection_domains(self, mock_call_method):
        mock_call_method.return_value = None
        protection_domain_dm = models.ProtectionDomain.get_mock_object()
        self.cluster_fm.set_protection_domains(protection_domains=[protection_domain_dm])
        expected_call_param = {'nodes': [protection_domain_dm.serialize()]}
        mock_call_method.assert_called_with(enums.ClusterMethod.set_protection_domains,
                                            params=expected_call_param,
                                            result_dm_class=None,
                                            auth=mock.ANY, log_response=True)
