import unittest

import mock

from sf_api import (
    api_constants,
    result_models,
)
from valence import (
    enums,
    functional_models,
    models,
    unittest_utils,
)


@mock.patch('sf_api.ClusterApi.call_method',
            side_effect=unittest_utils.helpers.check_params_serializable)
class TestClusterSchedules(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.cluster_fm = functional_models.Cluster.get_mock_fm()

    def test_create_schedule(self, mock_call_method):
        schedule_info = models.ScheduleInfo.get_mock_object()
        mock_response = result_models.CreateSchedule.get_mock_response()
        mock_call_method.return_value = mock_response
        result = self.cluster_fm.create_schedule('a name', schedule_info)

        params = {
            api_constants.SCHEDULE_INFO: schedule_info.serialize(),
            api_constants.SCHEDULE_NAME: 'a name',
            api_constants.SCHEDULE_TYPE: enums.ScheduleType.snapshot,
            api_constants.MONTHDAYS: None,
            api_constants.WEEKDAYS: None,
            api_constants.RECURRING: None,
            api_constants.PAUSED: None,
            api_constants.HOURS: None,
            api_constants.MINUTES: None,
            api_constants.ATTRIBUTES: None,
            api_constants.STARTING_DATE: None,
            api_constants.SNAPMIRROR_LABEL: None
        }
        mock_call_method.assert_called_with(enums.ClusterMethod.create_schedule, params=params,
                                            result_dm_class=result_models.CreateSchedule,
                                            auth=mock.ANY, log_response=True)
        assert isinstance(result, models.Schedule)

    def test_list_schedules(self, mock_call_method):
        mock_response = result_models.ListSchedules.get_mock_response()
        mock_call_method.return_value = mock_response
        result = self.cluster_fm.list_schedules()
        mock_call_method.assert_called_with(enums.ClusterMethod.list_schedules,
                                            params=None,
                                            result_dm_class=result_models.ListSchedules,
                                            auth=mock.ANY, log_response=True)
        assert isinstance(result[0], models.Schedule)

    def test_modify_schedules(self, mock_call_method):
        mock_response = result_models.ModifySchedule.get_mock_response()
        mock_call_method.return_value = mock_response
        mock_schedule = models.Schedule.get_mock_object()
        result = self.cluster_fm.modify_schedule(mock_schedule)

        params = {
            api_constants.SCHEDULE_ID: mock_schedule.schedule_id,
            api_constants.SCHEDULE_INFO: None,
            api_constants.SCHEDULE_NAME: None,
            api_constants.SCHEDULE_TYPE: enums.ScheduleType.snapshot,
            api_constants.MONTHDAYS: None,
            api_constants.WEEKDAYS: None,
            api_constants.RECURRING: None,
            api_constants.PAUSED: None,
            api_constants.HOURS: None,
            api_constants.MINUTES: None,
            api_constants.ATTRIBUTES: None,
            api_constants.STARTING_DATE: None,
            api_constants.SNAPMIRROR_LABEL: None,
            api_constants.TO_BE_DELETED: None,
        }

        mock_call_method.assert_called_with(enums.ClusterMethod.modify_schedule, params=params,
                                            result_dm_class=result_models.ModifySchedule,
                                            auth=mock.ANY, log_response=True)
        assert isinstance(result, models.Schedule)
