import unittest

import mock

from sf_api import (
    api_constants,
    result_models,
)
from valence import (
    enums,
    functional_models,
    unittest_utils,
)


@mock.patch('sf_api.ClusterApi.call_method',
            side_effect=unittest_utils.helpers.check_params_serializable)
class TestClusterAccountMixin(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.cluster_fm = functional_models.Cluster.get_mock_fm()

    def test_get_ntp_info(self, mock_call_method):
        """Basic test to verify get_ntp_info returns correctly."""
        mock_response = result_models.GetNtpInfo.get_mock_response()
        mock_call_method.return_value = mock_response
        get_ntp_info = self.cluster_fm.get_ntp_info()

    def test_set_ntp_info_default(self, mock_call_method):
        """Basic test to verify set_ntp_info returns correctly for defaults."""
        self.cluster_fm.set_ntp_info()
        mock_call_method.assert_called_with(
            enums.ClusterMethod.set_ntp_info,
            params={api_constants.SERVERS: [],
                    api_constants.BROADCAST_CLIENT: False},
            log_response=mock.ANY,
            auth=mock.ANY,
            result_dm_class=None)

    def test_set_ntp_info(self, mock_call_method):
        """Basic test to verify set_ntp_info returns correctly for non-defaults."""
        broadcast_client = False
        servers = ['1.1.1.1',
                   'bigben.netapp.com',
                   '1200:0000:AB00:1234:0000:2552:7777:1313',
                   '[2001:db8:0:1]']
        self.cluster_fm.set_ntp_info(servers=servers, broadcast_client=broadcast_client)
        mock_call_method.assert_called_with(
            enums.ClusterMethod.set_ntp_info,
            params={api_constants.SERVERS: servers,
                    api_constants.BROADCAST_CLIENT: broadcast_client},
            log_response=mock.ANY,
            auth=mock.ANY,
            result_dm_class=None)
