import unittest

import mock
import pytest

from sf_api import result_models
from valence import (
    functional_models,
    models,
    unittest_utils,
)


@mock.patch('sf_api.ClusterApi.call_method',
            side_effect=unittest_utils.helpers.check_params_serializable)
class TestCreateVirtualNetwork(unittest.TestCase):
    def test_create_virtual_network(self, mock_call_method):
        cluster_fm = functional_models.Cluster.get_mock_fm()
        virtual_network = models.VirtualNetwork.get_mock_object()
        mock_response = result_models.AddVirtualNetwork.get_mock_response()
        mock_call_method.return_value = mock_response
        result = cluster_fm.create_virtual_network(
            address_blocks=virtual_network.address_blocks, name=virtual_network.name,
            netmask=virtual_network.netmask, svip=virtual_network.svip,
            virtual_network_tag=virtual_network.virtual_network_tag,
            attributes=virtual_network.attributes, gateway=virtual_network.gateway,
            namespace=virtual_network.namespace)
        assert result == mock_response.result_dm.virtual_network_id


@mock.patch('sf_api.ClusterApi.call_method',
            side_effect=unittest_utils.helpers.check_params_serializable)
class TestModifyVirtualNetwork(unittest.TestCase):
    def test_modify_virtual_network(self, mock_call_method):
        cluster_fm = functional_models.Cluster.get_mock_fm()
        virtual_network = models.VirtualNetwork.get_mock_object()
        cluster_fm.modify_virtual_network(
            address_blocks=virtual_network.address_blocks, name=virtual_network.name,
            netmask=virtual_network.netmask, svip=virtual_network.svip,
            virtual_network_tag=virtual_network.virtual_network_tag,
            attributes=virtual_network.attributes, gateway=virtual_network.gateway,
            namespace=virtual_network.namespace)
        assert mock_call_method.called


@mock.patch('sf_api.ClusterApi.call_method',
            side_effect=unittest_utils.helpers.check_params_serializable)
class TestListVirtualNetworks(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.cluster_fm = functional_models.Cluster.get_mock_fm()
        cls.virtual_network = models.VirtualNetwork.get_mock_object()

    def test_list_virtual_networks(self, mock_call_method):
        mock_call_method.return_value = result_models.ListVirtualNetworks.get_mock_response()
        results = self.cluster_fm.list_virtual_networks()
        assert isinstance(results[0], models.VirtualNetwork)

    def test_list_virtual_networks_by_tag(self, mock_call_method):
        mock_call_method.return_value = result_models.ListVirtualNetworks.get_mock_response()
        results = self.cluster_fm.list_virtual_networks(
            virtual_network_tags=[self.virtual_network.virtual_network_tag])
        assert isinstance(results[0], models.VirtualNetwork)

    def test_list_virtual_networks_by_id(self, mock_call_method):
        mock_call_method.return_value = result_models.ListVirtualNetworks.get_mock_response()
        results = self.cluster_fm.list_virtual_networks(
            virtual_network_ids=[self.virtual_network.virtual_network_id])
        assert isinstance(results[0], models.VirtualNetwork)


@mock.patch('sf_api.ClusterApi.call_method',
            side_effect=unittest_utils.helpers.check_params_serializable)
class TestDeleteVirtualNetwork(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.cluster_fm = functional_models.Cluster.get_mock_fm()
        cls.virtual_network = models.VirtualNetwork.get_mock_object()

    def test_delete_filter_tag(self, mock_call_method):
        self.cluster_fm.delete_virtual_network(
            virtual_network_tag=self.virtual_network.virtual_network_tag)
        assert mock_call_method.called

    def test_delete_filter_id(self, mock_call_method):
        self.cluster_fm.delete_virtual_network(
            virtual_network_tag=None, virtual_network_id=self.virtual_network.virtual_network_id)
        assert mock_call_method.called
