import unittest

import mock

from sf_api import (
    models,
    result_models,
)
from valence import (
    functional_models,
    models,
    unittest_utils,
)


@mock.patch('sf_api.ClusterApi.call_method',
            side_effect=unittest_utils.helpers.check_params_serializable)
class TestZkMethods(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.cluster_fm = functional_models.Cluster.get_mock_fm()
        cls.database_entry = models.ZkDatabaseEntry.get_mock_object(force_mock='data')

    def test_list_database_children_data(self, mock_call_method):
        mock_call_method.return_value = result_models.ListDatabaseChildrenData.get_mock_response()

        result = self.cluster_fm.list_database_children_data('/')

        assert isinstance(result[0], models.ZkDatabaseEntry)

    def test_get_database_entry(self, mock_call_method):
        mock_call_method.return_value = result_models.GetDatabaseEntry.get_mock_response()

        result = self.cluster_fm.get_database_entry('/')

        assert isinstance(result, result_models.GetDatabaseEntry)

    def test_set_database_entry(self, mock_call_method):
        mock_call_method.return_value = result_models.SetDatabaseEntry.get_mock_response()

        result = self.cluster_fm.set_database_entry('/', self.database_entry.data,
                                                    self.database_entry.stat.data_version)
        assert isinstance(result, result_models.SetDatabaseEntry)
