"""Cluster FM methods related to time."""

from sf_api import (
    api_constants,
    result_models,
)
from valence import enums
from valence.functional_models._cluster import base


class TimeMixin(base.ClusterBase):
    """Cluster FM time methods."""

    def get_ntp_info(self):
        """Returns the current network time protocol configuration information.

        :returns: the ntp info class containing the ntp servers and if the
            nodes are listing for broadcasts
        :rtype: sf_api.result_models.GetNtpInfo
        """
        response = self.call_method(enums.ClusterMethod.get_ntp_info,
                                    result_dm_class=result_models.GetNtpInfo)
        return response.result_dm

    def set_ntp_info(self, servers=None, broadcast_client=False):
        """Sets the network time protocol configuration information.

        SetNtpInfo allows you to configure NTP on cluster nodes. The values you
        set with this interface apply to all nodes in the cluster. If an NTP
        broadcast server periodically broadcasts time information on your
        network, you can optionally configure nodes as broadcast clients

        :param list[str] servers: List of NTP servers to add to each node's
            NTP configuration.
        :param bool broadcast_client: Indicates whether or not the nodes in
            the cluster are listening for broadcast NTP messages.
        """
        if servers is None:
            servers = []

        params = {api_constants.SERVERS: servers,
                  api_constants.BROADCAST_CLIENT: broadcast_client}
        self.call_method(enums.ClusterMethod.set_ntp_info,
                         params=params)
