"""Cluster FM methods related to zookeeper database APIs ."""
from sf_api import (
    api_constants,
    result_models,
)
from valence import enums
from valence.functional_models._cluster import base


class ZkMixin(base.ClusterBase):
    """Cluster FM Drum methods."""

    def list_database_children_data(self, path):
        """Get list of zookeeper database children for given path.

        :param str path: zookeeper database path (root is /).
        :rtype: list[sf_api.models.ZkDatabaseEntry]
        """

        response = self.api.call_method(enums.ClusterMethod.list_database_children_data,
                                        params={api_constants.PATH: path},
                                        result_dm_class=result_models.ListDatabaseChildrenData)
        return response.result_dm.children

    def get_database_entry(self, path):
        """Get zookeeper data for given path.

        :param str path: zookeeper database path.
        :rtype: sf_api.result_models.GetDatabaseEntry
        """

        response = self.api.call_method(enums.ClusterMethod.get_database_entry,
                                        params={api_constants.PATH: path},
                                        result_dm_class=result_models.GetDatabaseEntry)
        return response.result_dm

    def set_database_entry(self, path, data, data_version):
        """Set zookeeper data for given path.

        :param str path: zookeeper database path.
        :param dict data: Database data to be written.
        :param int data_version: Version number.
        :rtype: sf_api.result_models.GetDatabaseEntry
        """

        response = self.api.call_method(enums.ClusterMethod.set_database_entry,
                                        params={api_constants.PATH: path,
                                                api_constants.DATA: data,
                                                api_constants.DATA_VERSION: data_version},
                                        result_dm_class=result_models.SetDatabaseEntry)
        return response.result_dm
