"""Constants for functional models."""
from valence import models

OPER_EQ = '=='
OPER_GTE = '>='
OPER_GT = '>'
OPER_LTE = '<='
OPER_LT = '<'
OPER_NEQ = '!='

OPER_MAP = {OPER_EQ:  'eq',
            OPER_GT:  'gt',
            OPER_GTE: 'ge',
            OPER_LT:  'lt',
            OPER_LTE: 'le',
            OPER_NEQ: 'ne'
            }

DEFAULT_BATCH_SIZE = 10
DEFAULT_THREADED_BATCH_SIZE = 5

open_iscsi_errors = {
    0: 'ISCSI_SUCCESS - command executed successfully',
    1: 'ISCSI_ERR - generic error code',
    2: 'ISCSI_ERR_SESS_NOT_FOUND - session could not be found',
    3: 'ISCSI_ERR_NOMEM - could not allocate resource for operation',
    4: 'ISCSI_ERR_TRANS - connect problem caused operation to fail',
    5: 'ISCSI_ERR_LOGIN - generic iSCSI login failure',
    6: 'ISCSI_ERR_IDBM - error accessing/managing iSCSI DB',
    7: 'ISCSI_ERR_INVAL - invalid argument',
    8: 'ISCSI_ERR_TRANS_TIMEOUT - connection timer expired while trying to connect',
    9: 'ISCSI_ERR_INTERNAL - generic internal iscsid/kernel failure',
    10: 'ISCSI_ERR_LOGOUT - iSCSI logout failed',
    11: 'ISCSI_ERR_PDU_TIMEOUT - iSCSI PDU timedout',
    12: 'ISCSI_ERR_TRANS_NOT_FOUND - iSCSI transport module not loaded in' +
        'kernel or iscsid',
    13: 'ISCSI_ERR_ACCESS - did not have proper OS permissions to access' +
        'iscsid or execute iscsiadm command',
    14: 'ISCSI_ERR_TRANS_CAPS - transport module did not support operation',
    15: 'ISCSI_ERR_SESS_EXISTS - session is logged in',
    16: 'ISCSI_ERR_INVALID_MGMT_REQ - invalid IPC MGMT request',
    17: 'ISCSI_ERR_ISNS_UNAVAILABLE - iSNS service is not supported',
    18: 'ISCSI_ERR_ISCSID_COMM_ERR - a read/write to iscsid failed',
    19: 'ISCSI_ERR_FATAL_LOGIN - fatal iSCSI login error',
    20: 'ISCSI_ERR_ISCSID_NOTCONN - could not connect to iscsid',
    21: 'ISCSI_ERR_NO_OBJS_FOUND - no records/targets/sessions/portals found' +
        'to execute operation on',
    22: 'ISCSI_ERR_SYSFS_LOOKUP - could not lookup object in sysfs',
    23: 'ISCSI_ERR_HOST_NOT_FOUND - could not lookup host',
    24: 'ISCSI_ERR_LOGIN_AUTH_FAILED - login failed due to authorization failure',
    25: 'ISCSI_ERR_ISNS_QUERY - iSNS query failure',
    26: 'ISCSI_ERR_ISNS_REG_FAILED - iSNS registration/deregistration failed',
    255: 'Unknown iSCSI error code',
    -99: 'iSCSI command timed out!'
}
BOURBON_USR_NAME = 'valence_dev'
BOURBON_PASSWORD = 'valence_dev_password'

# Upgrade constants
SFUPGRADECHECK = 'sfupgradecheck'
SFHEALTHTOOLS_URL_ENV = 'SFHEALTHTOOLS_URL'
URA_PACKAGES_DIR = '/tmp/ura-packages'
CMD_SFUPDATE_HEALTHTOOLS = '/sf/bin/sfupdate-healthtools'
SOLIDFIRE_HEALTHTOOLS = 'solidfire-healthtools'
CMD_SFUPGRADECHECK = '/sf/bin/sfupgradecheck'
SF_STAGING = '/sf/healthtools-staging/'
SF_DOWNLOADS = '/sf/downloads/'
COMPONENTS = 'components-'
SFINSTALLD = 'solidfire-sfinstalld-'
VAR_WWW = '/var/www/'
SETUP_MNODE_CMD = '/sf/packages/mnode/setup-mnode'

OXYGEN_PATCH6 = models.SfVersion('8.6.2.3')
OXYGEN_PATCH7 = models.SfVersion('8.7.0.15')
OXYGEN_PATCH7_2_1 = models.SfVersion('8.7.2.1')
FLUORINE_PATCH1 = models.SfVersion('9.1.0.38')
FLUORINE_PATCH2 = models.SfVersion('9.2.0.43')
FLUORINE_PATCH3 = models.SfVersion('9.3.0.40')
FLUORINE_PATCH4 = models.SfVersion('9.4.0.13')
FLUORINE_PATCH5 = models.SfVersion('9.5.0.6')
FLUORINE_PATCH5_1 = models.SfVersion('9.5.1.14')
NEON = models.SfVersion('10.0.0.282')
NEON_PATCH1 = models.SfVersion('10.1.0.83')
NEON_PATCH2 = models.SfVersion('10.2.1.7')
NEON_PATCH3 = models.SfVersion('10.3.0.157')
NEON_PATCH3_1 = models.SfVersion('10.3.1.5')
NEON_PATCH4 = models.SfVersion('10.4.0.35')
NEON_PATCH4_1 = models.SfVersion('10.4.1.8')
SODIUM = models.SfVersion('11.0.0.781')
SODIUM_PATCH1 = models.SfVersion('11.1.0.46')
SODIUM_PATCH2 = models.SfVersion('11.2.0.0')
SODIUM_PATCH3 = models.SfVersion('11.3.0.14235')
SODIUM_PATCH5 = models.SfVersion('11.5.0.36')
SODIUM_PATCH7 = models.SfVersion('11.7.0.1')
MAGNESIUM = models.SfVersion('12.0.0.3')

UPGRADE_START_EVENT = 'Storage Cluster Upgrade Start'
UPGRADE_INTERRUPT_EVENT = 'Cluster Upgrade interrupted by user'
NODE_UPGRADE_START_EVENT = 'Storage Node Upgrade Start'
NODE_UPGRADE_COMPLETE_EVENT = 'Storage Node Upgrade Complete'
UPGRADE_FREE_SPACE_EVENT = 'Insufficient free space in /var/log on the following nodes'
UPGRADE_AVAILABLE_DRIVES_EVENT = 'Too many drives in available state'
UPGRADE_DOWN_DELAY_EVENT = 'bond-downdelay on the network interfaces is set to custom values on nodes'
NODE_UPGRADE_FAILED_EVENT = 'Storage Node Upgrade Failed'
UPGRADE_RESUMED_EVENT = 'Resume Cluster Upgrade'
UPGRADE_RESUMED_EVENT_19_1_1 = 'Storage Cluster Upgrade Resumed'
UPGRADE_FINISHED_EVENT = 'Storage Cluster Upgrade Complete'
UPGRADE_STAGE_EVENT = 'Storage Node Upgrade Staging Successful'
EXCESSIVE_ASYNC_RESULTS_EVENT = 'Storage Cluster Upgrade error: The cluster configuration database'
UPGRADE_HALTED_ERROR = 'Storage Cluster Upgrade halted with an error'
MGMT_PLATFORM_LEGACY = 'Management Node Platform: 10.x'
MGMT_PLATFORM_11x = 'Management Node Platform: {}'
MGMT_PLATFORM_EMBER = 'Management Node Platform: Ember'

VUPC_SFINSTALL_ERROR = 'Invalid upgrade path.  Must choose supported upgrade path.'
WRONG_MNODE_ERROR = "This mnode can't be used to upgrade to sodium"
WRONG_UPGRADE_METHOD_ERROR = "You're attempting to perform an upgrade using unsupported method"
HEALTHTOOLS_MISSING_CRYPTO_KEY_ERROR = 'This management node was initially installed with a pre-9.0 ' \
                                       'version and does not have the required key file. Recommend ' \
                                       'installing the latest 10.x management node or contact Support ' \
                                       'to download and install D-patch CSD-2214.'
KB_INSTRUCTIONS = 'in the KB article: https://kb.netapp.com/app/answers/answer_view/a_id/1086510'

HEALTHTOOLS_UNABLE_TO_VERIFY = 'Unable to verify latest available version of healthtools'
HEALTHTOOLS_MANUALLY_VERIFY = 'Manually verify latest version by checking'
HEALTHTOOLS_DOWNLOAD_INSTRUCTIONS = 'To download the latest version of the HealthTools, go to:'
HEALTHTOOLS_DOWNLOAD_INSTRUCTIONS_19_1 = 'The latest version of HealthTools can be downloaded from'
HEALTHTOOLS_DOWNLOAD_URL = 'https://mysupport.netapp.com/NOW/cgi-bin/HealthTools/'
HEALTHTOOLS_DOWNLOAD_URL_19_1 = 'https://mysupport.netapp.com/products/p/healthtools.html'
CURRENTLY_INSTALLED_HEALTHTOOLS_VERSION = 'Currently installed version of HealthTools:'
PREVIOUSLY_INSTALLED_HEALTHTOOL_VERSIONS = 'Previously installed versions of the HealthTools:'
NO_PREVIOUSLY_INSTALLED_VERSIONS_FOUND = 'No previously installed versions of the HealthTools found'
LATEST_INSTALLED_HEALTHTOOLS_VERSION = 'Latest available version of HealthTools:'

FIRST_VERSION_WITH_SFUPDATE_HEALTHTOOLS = NEON_PATCH2
FIRST_VERSION_WITH_SFUPGRADECHECK = FLUORINE_PATCH3
FIXING_CORRUPT_SFINSTALLD = 'Fixing corrupt sfinstalld'
SFINSTALLD_CORRUPTION_DETECTED = 'sfinstalld corruption detected'
SFUPGRADECHECK_V_OUT_OF_DATE = 'healthtools is out of date:'
SFUPGRADECHECK_LATEST_VERSION_AVAILABLE = 'Latest version of upgrade tools available:'
SFUPGRADECHECK_GENERIC_ERROR = 'sfupgradecheck failed, an unexpected error has occurred'
SFUPGRADECHECK_NODES_NOT_REACHABLE = 'all nodes in the cluster must be reachable'
SFUPGRADECHECK_INTERNAL_ERROR_500 = 'sfupgradecheck failed: method GetAPIVersion name xInternalError ' \
                                    'code 500'
SFUPGRADECHECK_DOWNLOAD_ERROR = 'sfupgradecheck failed: method Download name xSystemCommandFailure ' \
                                'code 500'
SFUPGRADECHECK_UNABLE_TO_VERIFY_LATEST = 'sfupgradecheck failed: Unable to verify latest available ' \
                                         'version of healthtools.'
SFUPGRADECHECK_MANUALLY_VERIFY = 'Manually verify latest version by checking'
SFUPGRADECHECK_DOWNLOAD_INSTRUCTIONS = 'The latest version of the HealthTools can be downloaded from ' \
                                       'NetApp'


HEALTHTOOLS_10_2_VERSION = models.GenericVersion('2018.01.01.100')
HEALTHTOOLS_10_3_VERSION = models.GenericVersion('2018.05.01.0')
HEALTHTOOLS_10_3_SHIPPING_VERSION = models.GenericVersion('2018.05.01.500')
HEALTHTOOLS_10_3_POST_RELEASE_VERSION = models.GenericVersion('2018.05.01.600')
HEALTHTOOLS_10_3_BUGFIX_VERSION = models.GenericVersion('2018.05.01.627')
HEALTHTOOLS_18_3_VERSION = models.GenericVersion('2018.09.01.131')
HEALTHTOOLS_18_3_POST_RELEASE_VERSION = models.GenericVersion('2018.09.01.135')
HEALTHTOOLS_18_3_BUGFIX_VERSION = models.GenericVersion('2018.09.01.136')
HEALTHTOOLS_18_3_1_VERSION = models.GenericVersion('2018.12.01.102')
HEALTHTOOLS_18_3_1_OOB_VERSION = models.GenericVersion('2018.12.01.107')

HEALTHTOOLS_19_1_VERSION = models.GenericVersion('2019.06.01.111')
HEALTHTOOLS_19_1_OOB_VERSION = models.GenericVersion('2019.06.01.112')
HEALTHTOOLS_19_2_VERSION = models.GenericVersion('2019.12.01.0')
HEALTHTOOLS_20_1_VERSION = models.GenericVersion('2020.01.01.0')

HEALTHCHECK_EXECUTION_TIMEOUT_S = 300

MNODE_MAIN_URL = \
    'https://hijenkinsv1.ten.den.solidfire.net:8081/job/mnode-main/job/dev-2.x/lastCompletedBuild/' \
    'api/json'
