"""Functional model exceptions."""

import valence.errors


class Error(valence.errors.Error):
    """Base exception for functional models package.

    This error should not be raised directly.
    """


class AsyncHandlerError(Error):
    """Base exception for the async handler.

    :param str msg: Exception message
    :param valence.models.AsyncResult async_result: Optional async_result reference
    """

    def __init__(self, msg, async_result=None):
        super(AsyncHandlerError, self).__init__(msg)
        self.async_result = async_result


class FunctionalClientError(Error):
    """Base exception for the client functional model."""


class FunctionalClusterError(Error):
    """Base exception for the cluster functional model."""


class FunctionalNodeError(Error):
    """Base exception for the node functional model."""


class FunctionalManagementNodeError(Error):
    """Base exception for the management node functional model."""


class MissingManagementNodeConnectionAdapterError(Error):
    """When mnode fm doesn't have a connection adapter."""


class HealthcheckError(Error):
    """When healthchecks fail to finish in expected time."""


class IscsiError(Error):
    """Base exception for iscsi related errors."""


class PollMaxError(Error):
    """Exception for when polling reaches max timeout."""


class BundleError(Error):
    """Generic error when getting support bundles."""


class NodeBundleError(FunctionalNodeError):
    """Exception for when collection of support bundle on a node failed."""


class NvdimmError(FunctionalNodeError):
    """Exception for NVDIMM function."""


class ClientBundleError(FunctionalClientError):
    """Exception for when collection of support bundle on a client failed."""


class VerifyNvdimmFailedError(NvdimmError):
    """Exception for when verification of NVDIMM fails."""


class FlashNvdimmFirmwareError(NvdimmError):
    """Exception for when flashing nvdimm firmware fails."""


class ExternalKeyServerError(Error):
    """Base exception for the external key server functional model."""


class HyTrustExternalKeyServerError(ExternalKeyServerError):
    """Exception for the HyTrust EKS functional model."""


class CryptsoftExternalKeyServerError(ExternalKeyServerError):
    """Exception for the Cryptsoft EKS functional model."""


class SafenetExternalKeyServerError(ExternalKeyServerError):
    """Exception for the Safenet EKS functional model."""
