"""Helper function for constructing EKS functional models from a data model."""

from valence import enums
from valence.functional_models._external_key_server import (
    cryptsoft,
    hytrust,
    safenet,
    vormetric,
)


def get_eks_from_dm(eks_dm):
    """Returns eks functional model based on data model.

    :param sf_models.ExternalKeyServer eks_dm: eks data model
    :rtype: valence.functional_models._external_key_server.ExternalKeyServerBase
    """
    valid_safenet_enums = [
        enums.ExternalKeyServerVendor.gemalto_safenet,
        enums.ExternalKeyServerVendor.gemalto_safenet_150v,
        enums.ExternalKeyServerVendor.safenet_at_350v,
    ]

    if eks_dm.vendor == enums.ExternalKeyServerVendor.cryptsoft:
        return cryptsoft.CryptsoftExternalKeyServer(eks_dm)
    elif eks_dm.vendor in valid_safenet_enums:
        return safenet.SafenetExternalKeyServer(eks_dm)
    elif eks_dm.vendor == enums.ExternalKeyServerVendor.hytrust:
        return hytrust.HyTrustExternalKeyServer(eks_dm)
    elif eks_dm.vendor == enums.ExternalKeyServerVendor.vormetric:
        return vormetric.VormetricExternalKeyServer(eks_dm)
