"""Management node adapter."""

import autologger

import sf_api
from valence.functional_models import _node

_log = autologger.get_sflogging()


class ManagementNodeAdapter(_node.Node):
    """Management node adapter to use per node api.

    :param valence.models.ResourceManagementNode management_node_dm: node data model
    :param valence.models.ResourceCluster|None cluster_dm: Optional cluster data model. If given,
        the node will authenticate using the cluster's credentials. For an orphaned node, set this
        argument to None. If the node is orphaned, you must call node_fm.authenticate(cluster_dm) to
        authenticate the node once it is added to the cluster.
    :param bool authenticate: If True, check if node can be authenticated via HTTP in constructor.
    :param bool validate_responses: If True, validate responses from SF Api (default=True)
    """

    def __init__(self, management_node_dm, cluster_dm=None, authenticate=False, validate_responses=True):
        super(ManagementNodeAdapter, self).__init__(management_node_dm, cluster_dm, authenticate,
                                                    validate_responses)
        # Why the resource doesn't have the correct credentials?  I couldn't say
        # pylint: disable=protected-access
        self.api._auth = sf_api.SfAuth(ip=management_node_dm.mip, username='admin',
                                       password='admin', port=442)
