"""Unit tests for Node Functional Models."""

import time
import unittest

import mock
import pytest
import sf_ssh

from valence import (
    functional_models,
    models,
    unittest_utils,
)

module_mock = unittest_utils.ModuleLevelMock()


def setUpModule():
    # Mock Ssh.close and Ssh._init_client globally for the module so we don't need to mock the
    # Ssh context manager and can mock the Ssh methods directly.
    module_mock.mock(sf_ssh.Ssh, 'close')
    module_mock.mock(sf_ssh.Ssh, '_init_client')

    # Sleep no more
    module_mock.mock(time, 'sleep')


def tearDownModule():
    module_mock.restore()


@mock.patch.object(sf_ssh.Ssh, 'run_command')
class TestGetUptime(unittest.TestCase):
    def test_get_uptime(self, mock_run_command):
        mnode = functional_models.ManagementNodeBase.get_mock_fm()
        mock_run_command.return_value = sf_ssh.ssh.RunCommandOutput(stdout='237320.88 2683148.12\n',
                                                                    stderr='',
                                                                    return_code=0)
        uptime = mnode.get_uptime(timeout=1)
        assert uptime == 237320
        mock_run_command.return_value = sf_ssh.ssh.RunCommandOutput(stdout='237320.88 2683148.12\n',
                                                                    stderr='SOme ERrror',
                                                                    return_code=0)
        with pytest.raises(functional_models.FunctionalManagementNodeError):
            _ = mnode.get_uptime(timeout=1)


class TestMnodeAdapter(unittest.TestCase):
    def test_adapter(self):
        mnode_dm = models.ResourceManagementNode.get_mock_object()
        mnode_fm = functional_models.ManagementNodeAdapter(mnode_dm, None)
