import time
import unittest

import mock
import pytest
import sf_ssh

from valence import (
    functional_models,
    models,
    unittest_utils,
    vl_helpers,
)

module_mock = unittest_utils.ModuleLevelMock()


def setUpModule():
    # Mock Ssh.close and Ssh._init_client globally for the module so we don't need to mock the
    # Ssh context manager and can mock the Ssh methods directly.
    module_mock.mock(sf_ssh.Ssh, 'close')
    module_mock.mock(sf_ssh.Ssh, '_init_client')

    # Sleep no more
    module_mock.mock(time, 'sleep')


def tearDownModule():
    module_mock.restore()


STD_ERR = """
Uploading version 2019.06.01.102 to node nlabp2814
Uploading version 2019.06.01.102 to node nlabp2815
Uploading version 2019.06.01.102 to node nlabp2816
Uploading version 2019.06.01.102 to node nlabp2813
Uploading solidfire-sfinstalld-2017.10.10.102.tgz to nodeID 1
Uploading solidfire-sfinstalld-2017.10.10.102.tgz to nodeID 2
Uploading solidfire-sfinstalld-2017.10.10.102.tgz to nodeID 3
Uploading solidfire-sfinstalld-2017.10.10.103.tgz to nodeID 4
"""

STD_OUT = """
Uploading version 2019.01.01.131 to node nlabp2814
Uploading version 2019.01.01.131 to node nlabp2815
Uploading version 2019.01.01.131 to node nlabp2816
Uploading version 2019.01.01.131 to node nlabp2813
Uploading solidfire-sfinstalld-2017.10.10.102.tgz to nodeID 1
Uploading solidfire-sfinstalld-2017.10.10.102.tgz to nodeID 2
Uploading solidfire-sfinstalld-2017.10.10.102.tgz to nodeID 3
Uploading solidfire-sfinstalld-2017.10.10.102.tgz to nodeID 4
"""

STD_OUT_EXP = """
Uploading version 2019.12.03.101 to node nlabp2814
Uploading version 2019.12.03.101 to node nlabp2815
Uploading version 2019.12.03.101 to node nlabp2816
Uploading version 0 to node nlabp2813
Uploading solidfire-sfinstalld-2017.10.10.102.tgz to nodeID 1
Uploading solidfire-sfinstalld-2017.10.10.102.tgz to nodeID 2
Uploading solidfire-sfinstalld-2017.10.10.102.tgz to nodeID 3
Uploading solidfire-sfinstalld-2017.10.10.102.tgz to nodeID 0
"""

SFUP_V_STD_OUT = """
Latest version of upgrade tools available: 2018.12.01.107
Installed version of upgrade tools on management node: 2018.12.01.102
Management Node Platform: Ember
Node 1 is on version 2018.12.01.102
HealthTools is out of date: The latest version of HealthTools can be downloaded from NetApp Support Site
Go to https://mysupport.netapp.com/NOW/dgi-bin/software/,
and select HealthTools to download the latest version
"""


class TestOutputVersionsMnodeLegacy(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.mnode_fm = functional_models.ManagementNodeLegacy.get_mock_fm()
        cls.cluster_dm = models.ResourceCluster.get_mock_object()
        cls.node_dms = [models.ActiveNode.get_mock_object(overrides={'node_id': 0, 'name': 'nlabp2813'}),
                        models.ActiveNode.get_mock_object(overrides={'node_id': 1, 'name': 'nlabp2814'}),
                        models.ActiveNode.get_mock_object(overrides={'node_id': 2, 'name': 'nlabp2815'}),
                        models.ActiveNode.get_mock_object(overrides={'node_id': 3, 'name': 'nlabp2816'})]

    @mock.patch.object(sf_ssh.Ssh, 'run_command')
    def test_run_cmd(self, mock_run_command):
        mock_run_command.return_value = sf_ssh.ssh.RunCommandOutput(
            stdout='', stderr='', return_code=1)

        self.mnode_fm.execute_preupgrade_health_check(
            self.cluster_dm,
            show_help=True,
            sfhealthtools_url_to_use='http://jenkins.eng.solidfire.net/healthtools-2019.06.01...',
            show_version=True,
            metadata='http://jenkins.eng.solidfire.net/healthtools-metadata.json',
            raise_on_failure=True)

        assert mock_run_command.call_count == 1
