import json
import os
import unittest

from valence.functional_models._management_node import processed_healthchecks

CWD, _ = os.path.split(os.path.abspath(os.path.expanduser(__file__)))
DATA = os.path.join(CWD, 'data')
CHECKS = os.path.join(DATA, 'checks.json')
CHECKS_10_3_NO_CONN = os.path.join(DATA, 'checks_10_3_no_conn_speed.json')
CHECKS_10_3_ALL_CHECKS = os.path.join(DATA, 'checks_10_3_all_checks.json')
CHECKS_ALL_PASS = os.path.join(DATA, 'checks_all_pass.json')
CHECKS_CLUSTER_FAIL = os.path.join(DATA, 'checks_cluster_fail.json')
CHECKS_10_3_DATA_VALIDATION = os.path.join(DATA, 'checks_10_3_missing_data_validation.json')


class TestCheckModel(unittest.TestCase):
    def setUp(self):
        with open(CHECKS) as f:
            self.checks_text = f.read()

        with open(CHECKS_10_3_NO_CONN) as f:
            self.checks_10_3_text_no_conn = f.read()

        with open(CHECKS_ALL_PASS) as f:
            self.checks_all_pass_text = f.read()

        with open(CHECKS_CLUSTER_FAIL) as f:
            self.checks_cluster_fail_text = f.read()

        with open(CHECKS_10_3_ALL_CHECKS) as f:
            self.checks_10_3_all_pass_text = f.read()

        with open(CHECKS_10_3_DATA_VALIDATION) as f:
            self.checks_10_3_data_validation_text = f.read()

        self.all_checks = json.loads(self.checks_text)

    def test_parse(self):
        """Parse the version numbers."""
        processed = processed_healthchecks.ProcessedHealthchecks.from_json(self.checks_text)

        nodes_result = {
            i: [n.serialize() for n in checks] for (i, checks) in processed.node_checks.items()
        }

        assert [c.serialize() for c in processed.cluster_checks] == \
            self.all_checks['result']['clusterChecks']
        assert nodes_result[1] == self.all_checks['result']['nodes'][0]['result']

    def test_parse_err(self):
        """Parse the false json text for except block."""
        processed_healthchecks.ProcessedHealthchecks.from_json('blah')
