"""SSL Certificate mixin."""
from sf_api import (
    api_constants,
    result_models,
)
from valence import enums
from valence.functional_models._node import base


class CertMixin(base.NodeBase):
    """Node FM ssl certificate methods."""

    def set_node_ssl_certificate(self, certificate, private_key, force=False):
        """Set the node ssl certificate.

        :param str certificate: certificate.pem string to set
        :param str private_key: private key for the certificate
        :param bool force: hidden method to force setting the ssl certificate.
                           WARNING:  this will bypass checking to see if the certificate is valid.
                           In theory the cluster will check again when attempting to apply the
                           certificate
        """
        params = {api_constants.CERTIFICATE: certificate,
                  api_constants.PRIVATE_KEY: private_key,
                  api_constants.FORCE: force}
        self.api.call_method(enums.NodeMethod.set_node_ssl_certificate, params=params,
                             result_dm_class=result_models.SetSslCertificate)

    def get_node_ssl_certificate(self):
        """Get the node ssl certificate.

        :rtype: sf_api.result_models.GetSslCertificate
        """
        response = self.api.call_method(enums.NodeMethod.get_node_ssl_certificate,
                                        result_dm_class=result_models.GetSslCertificate)
        return response.result_dm

    def remove_node_ssl_certificate(self):
        """Remove any node ssl certificates that are present."""
        self.api.call_method(enums.NodeMethod.remove_node_ssl_certificate,
                             result_dm_class=result_models.RemoveSslCertificate)
