"""Node FM methods related to downloading packages."""

import autologger

from sf_api import (
    api_constants,
    result_models,
)
from valence import enums
from valence.functional_models._node import base

_log = autologger.get_sflogging()


class DownloadMixin(base.NodeBase):
    """Node FM download methods."""

    def download(self, url, destination=None, extract_path=None):
        """Downloads a file at the given URL to destination on the node.

        :param str url: URL for download
        :param str destination: destination path on the host OS
        :param str extract_path: path to extract tarball to
        """

        params = {
            api_constants.URL: url,
            api_constants.DESTINATION: destination,
            api_constants.EXTRACT_PATH: extract_path
        }

        self._call_method(enums.NodeMethod.download, params=params)

    def list_downloads(self):
        """Lists downloaded packages on the node.

        :rtype: list[str]
        """

        response = self._call_method(enums.NodeMethod.list_downloads,
                                     result_dm_class=result_models.ListDownloads)

        return response.result_dm.downloads

    def delete_downloads(self, downloads):
        """Deletes the provided downloads from the node.

        :param list[str] downloads: list of downloads to delete
        :rtype: list[str]
        """

        response = self._call_method(enums.NodeMethod.delete_downloads,
                                     params={api_constants.DOWNLOADS: downloads},
                                     result_dm_class=result_models.DeleteDownloads)

        return response.result_dm.downloads
