import unittest

import mock
import six

import sf_api
from sf_api import (
    api_constants,
    enums,
    result_models,
)
from sf_api.models import types
from valence import (
    functional_models,
    models,
    unittest_utils,
)

module_mock = unittest_utils.ModuleLevelMock()


@mock.patch.object(sf_api.NodeApi, 'call_method')
class TestPackageApi(unittest.TestCase):
    """Test methods that manipulate packages on a node."""

    @classmethod
    def setUpClass(cls):
        cls.node_fm = functional_models.Node.get_mock_fm(validate_responses=True)

    def test_download(self, mock_call_method):
        pkg = 'http://a.b.c.'
        dest = '/var'
        ext = '/'
        self.node_fm.download(pkg, destination=dest, extract_path=ext)

        params = {
            api_constants.URL: pkg,
            api_constants.DESTINATION: dest,
            api_constants.EXTRACT_PATH: ext
        }

        mock_call_method.assert_called_with(enums.NodeMethod.download,
                                            params=params,
                                            auth=mock.ANY, log_response=True,
                                            result_dm_class=None)

    def test_list_downloads(self, mock_call_method):

        mock_call_method.return_value = result_models.ListDownloads.get_mock_response()

        res = self.node_fm.list_downloads()

        assert isinstance(res[0], six.string_types)

    def test_delete_downloads(self, mock_call_method):

        mock_call_method.return_value = result_models.DeleteDownloads.get_mock_response()

        res = self.node_fm.delete_downloads(['/a'])

        assert isinstance(res[0], six.string_types)

        mock_call_method.assert_called_with(
            enums.NodeMethod.delete_downloads,
            params={api_constants.DOWNLOADS: ['/a']},
            auth=mock.ANY, log_response=True,
            result_dm_class=result_models.DeleteDownloads,
        )
