import unittest

import mock

import sf_api
from sf_api import result_models
from sf_api.models import types
from valence import (
    functional_models,
    models,
    unittest_utils,
)

module_mock = unittest_utils.ModuleLevelMock()


@mock.patch.object(sf_api.NodeApi, 'call_method')
class TestPackageApi(unittest.TestCase):
    """Test methods that manipulate packages on a node."""

    @classmethod
    def setUpClass(cls):
        cls.node_fm = functional_models.Node.get_mock_fm(validate_responses=True)

    def test_list_package_info(self, mock_call_method):
        dc = result_models.ListPackageInfo.get_mock_response()
        mock_call_method.return_value = dc
        result = self.node_fm.list_package_info()
        assert result == dc.result_dm.packages

    def test_install_package(self, mock_call_method):
        dc = result_models.InstallPackage.get_mock_response()
        mock_call_method.return_value = dc
        url = types.URLType()._mock()
        result = self.node_fm.install_package(url=url)
        assert result == dc.result_dm.package
        assert isinstance(result, models.Package)

    def test_install_package_persisted_inactive(self, mock_call_method):
        dc = result_models.InstallPackage.get_mock_response()
        mock_call_method.return_value = dc
        url = types.URLType()._mock()
        result = self.node_fm.install_package(url=url, set_active=False, persist=True)
        assert result == dc.result_dm.package
        assert isinstance(result, models.Package)

    def test_set_active_package_version(self, mock_call_method):
        dc = result_models.SetActivePackageVersion.get_mock_response()
        mock_call_method.return_value = dc
        result = self.node_fm.set_active_package_version(name='somename', version='1.0.0')
        assert result == dc.result_dm.package
        assert isinstance(result, models.Package)

    def test_uninstall_package(self, mock_call_method):
        dc = result_models.UninstallPackage.get_mock_response()
        mock_call_method.return_value = dc
        result = self.node_fm.uninstall_package(name='somename', version='1.0.0')
        assert result == dc.result_dm.package
        assert isinstance(result, models.Package)
