import unittest

import mock
import sf_rtfi

import sf_api
from sf_api import (
    api_errors,
    result_models,
)
from valence import (
    functional_models,
    models,
    unittest_utils,
)

module_mock = unittest_utils.ModuleLevelMock()


@mock.patch.object(sf_api.NodeApi, 'call_method')
class TestRestartMixin(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.node_fm = functional_models.Node.get_mock_fm(validate_responses=True)

    @mock.patch.object(sf_rtfi.node_rtfi.NodeRtfiProvider, 'start_reactor')
    @mock.patch.object(sf_rtfi.node_rtfi.NodeRtfiProvider, 'monitor_rtfi_status')
    def test_reset_node(self, mock_monitor, mock_reactor, mock_call_method):
        response = result_models.ResetNode.get_mock_response()
        version = models.SfVersion('10.0.0.284')
        self.node_fm.node_dm.network_profile = models.NetworkProfile()
        mock_call_method.return_value = response
        result_dm = self.node_fm.reset_node(version)
        assert isinstance(result_dm, functional_models.RtfiAsyncHandler)

    @mock.patch.object(sf_rtfi.node_rtfi.NodeRtfiProvider, 'start_reactor')
    @mock.patch.object(sf_rtfi.node_rtfi.NodeRtfiProvider, 'monitor_rtfi_status')
    @mock.patch('sf_rtfi.node_rtfi.NodeRtfiProvider')
    def test_reset_node_err(self, mock_watcher, mock_monitor, mock_start_reactor, mock_call_method):
        response = result_models.ResetNode.get_mock_response()
        version = models.SfVersion('10.0.0.284')
        self.node_fm.node_dm.network_profile = models.NetworkProfile()
        mock_watcher.reactor.join.return_value = True
        mock_call_method.return_value = response
        mock_call_method.side_effect = api_errors.SfApiException(
            'Something caught fire and blah blah blah', response)
        with self.assertRaises(api_errors.SfApiException):
            self.node_fm.reset_node(version)
