"""Unittests for functional model utils."""
import unittest

import mock
import pytest

from valence import functional_models
from valence.functional_models import (
    _constants,
    _fm_utils,
)


class TestHelperFunctions(unittest.TestCase):

    def test_simple_sf_comparator(self):
        oper = _constants.OPER_EQ
        first_list = [1, 2, 3]
        second_list = [1, 2, 3]
        result = _fm_utils.simple_sf_comparator(first_list, second_list, oper)
        assert result is True

    def test_simple_sf_comparator_neq(self):
        oper = _constants.OPER_NEQ
        first_list = [1, 2, 3]
        second_list = [1, 2, 3]
        result = _fm_utils.simple_sf_comparator(first_list, second_list, oper)
        assert result is False


class TestBasePoll(unittest.TestCase):

    def test_poll(self):
        mock_api_func = mock.Mock()
        mock_api_func.return_value = []
        oper = _constants.OPER_EQ
        base_poll = _fm_utils.BasePoll(10 * 60,
                                       mock_api_func,
                                       [], oper, poll_period=10)
        base_poll.poll()

    @mock.patch('time.time')
    @mock.patch('time.sleep')
    def test_poll_timeout(self, mock_sleep, mock_time):
        mock_api_func = mock.Mock()
        mock_api_func.return_value = [1, 2]
        oper = _constants.OPER_EQ
        base_poll = _fm_utils.BasePoll(10 * 60,
                                       mock_api_func,
                                       [], oper, poll_period=10)
        mock_time.side_effect = [0, 1000]
        with pytest.raises(functional_models._errors.PollMaxError):
            base_poll.poll()


class TestBlockPoll(unittest.TestCase):

    def test_start_polling(self):
        mock_api_func = mock.Mock()
        mock_api_func.return_value = []
        oper = _constants.OPER_EQ
        block_poll = _fm_utils.BlockPoll(10 * 60,
                                         mock_api_func,
                                         [], oper, poll_period=10)
        block_poll.start_polling()
