"""Models to represent a switchport in bourbon."""
from sf_api import models
from sf_api.models import types
from valence import enums


class PortChannel(models.Base):
    ip = types.IPAddressType(required=True, serialized_name='switch')
    port_channel = types.IntType(serialized_name='interface')
    switch_type = types.EnumType(enums.SwitchType)
    access_native_vlan = types.IntType()
    trunk_native_vlan = types.IntType()
    tagged_vlans = types.StringType()
    hybrid_mode = types.BooleanType(default=False)
    port_slots = types.ListType(types.IntType(), required=True, serialized_name='interfaces')


class SwitchPort(models.Base):
    ip = types.IPAddressType(required=True, serialized_name='switch')
    port_slot = types.StringType(required=True, serialized_name='interface', default='')
    state = types.EnumType(enums.PortState)
    switch_type = types.EnumType(enums.SwitchType)
    access_native_vlan = types.IntType()
    trunk_native_vlan = types.IntType()
    tagged_vlans = types.StringType()
    hybrid_mode = types.BooleanType(default=False)
    mac = types.StringType(required=True, serialized_name='address')
    wwpn = types.StringType()
    port_channel = types.IntType()

    def __repr__(self):
        return '{} <{}={}, {}={}>'.format(self.__class__.__name__,
                                          'ip', self.ip, 'mac_address', self.mac)
