"""Models related to external key servers."""
from sf_api import (
    api_constants,
    models,
)
from sf_api.models import types


class CertificateSubject(models.Base):
    common_name = types.StringType(required=True, serialized_name=api_constants.COMMON_NAME)
    organization = types.StringType(serialized_name=api_constants.ORGANIZATION)
    organizational_unit = types.StringType(serialized_name=api_constants.ORGANIZATIONAL_UNIT)
    locality = types.StringType(serialized_name=api_constants.LOCALITY)
    state = types.StringType(serialized_name=api_constants.STATE)
    country = types.StringType(serialized_name=api_constants.COUNTRY)
    email_address = types.StringType(serialized_name=api_constants.EMAIL_ADDRESS)

    def __repr__(self):
        return '{}<{}={}, {}={}, {}={}, {}={}, {}={}, {}={}>'.format(
            self.__class__.__name__,
            'CN', self.common_name,
            'O', self.organization,
            'OU', self.organizational_unit,
            'L', self.locality,
            'ST', self.state,
            'C', self.country)


class SafenetClientCertificate(models.Base):
    ca_name = types.StringType(required=True)
    purpose = types.StringType(required=True)
    duration = types.IntType(required=True)
    text = types.StringType(required=True)

    def __repr__(self):
        return '{}<{}={}, {}={}>'.format(
            self.__class__.__name__,
            'purpose', self.purpose,
            'duration', self.duration)


class SafenetLocalCertificateAuthority(models.Base):
    name = types.StringType(required=True)
    serial_number = types.StringType(required=True)
    key_size = types.IntType(required=True)
    start_date = types.StringType(required=True)
    expiration = types.StringType(required=True)
    subject = types.ModelType(CertificateSubject, required=True)
    issuer = types.ModelType(CertificateSubject, required=True)
    purpose = types.ListType(types.StringType(), required=True)
    text = types.StringType(required=True)

    def __repr__(self):
        return '{}<{}={}>'.format(
            self.__class__.__name__,
            'name', self.name)


class SafenetNaeServer(models.Base):
    protocol = types.StringType(required=True)
    ip = types.StringType(required=True)
    port = types.IntType(required=True)
    use_ssl = types.BooleanType(required=True)
    server_certificate = types.StringType(required=True)

    def __repr__(self):
        return '{}<{}={}, {}={}, {}={}>'.format(
            self.__class__.__name__,
            'protocol', self.protocol,
            'ip', self.ip,
            'server_certificate', self.server_certificate)


class SafenetService(models.Base):
    service_group = types.StringType(required=True)
    service = types.StringType(required=True)
    status = types.StringType(required=True)
    startup = types.StringType(required=True)

    def __repr__(self):
        return '{}<{}={}, {}={}>'.format(
            self.__class__.__name__,
            'service_group', self.service_group,
            'status', self.status)
