"""Tests valence.models.node."""

import unittest

from sf_models import _node_type

from sf_api import enums
from valence import models


class TestNodeModel(unittest.TestCase):
    def setUp(self):
        self.node_dm = models.NodeBase.get_mock_object()

    def test_get_attributes(self):
        """This provides code coverage on the properties."""
        resource_dm = self.node_dm.resource_dm
        assert self.node_dm.oob_ip == resource_dm.oob_ip
        assert self.node_dm.node_name == resource_dm.node_name
        assert self.node_dm.ssh_username == resource_dm.username
        assert self.node_dm.ssh_password == resource_dm.password
        assert self.node_dm.resource_id == resource_dm.resource_id
        assert self.node_dm.location_id == resource_dm.location_id
        assert self.node_dm.fibre_channel == resource_dm.fibre_channel
        assert self.node_dm.model == resource_dm.model
        assert self.node_dm.network_profile == resource_dm.network_profile

        assert isinstance(repr(resource_dm), str)

    def test_repr(self):
        assert isinstance(repr(models.ActiveNode()), str)
        assert isinstance(repr(models.PendingNode()), str)
        assert isinstance(repr(models.PendingActiveNode()), str)


class TestNodeModelProperties(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        default_node_type = _node_type.NodeType(_node_type.NodeTypeEnum.SF3010.value)
        cls.active_node_dm = models.ActiveNode.get_mock_object(
            overrides=dict(platform_info=dict(node_type=default_node_type))
        )
        cls.pending_node_dm = models.PendingNode.get_mock_object(
            overrides=dict(platform_info=dict(node_type=default_node_type))
        )
        cls.pending_active_node_dm = models.PendingActiveNode.get_mock_object(
            overrides=dict(platform_info=dict(node_type=default_node_type))
        )

    def test_static_id(self):
        assert self.active_node_dm.static_id == self.active_node_dm.mip
        assert self.pending_node_dm.static_id == self.pending_node_dm.mip
        assert self.pending_active_node_dm.static_id == self.pending_active_node_dm.mip

    def test_role(self):
        self.active_node_dm.internal_role = None
        assert self.active_node_dm.role is enums.NodeRole.storage

        # Witness Nodes
        self.pending_node_dm.internal_role = enums.NodeRole.witness
        assert self.pending_node_dm.role is enums.NodeRole.witness

        # FibreChannel Nodes
        self.pending_active_node_dm.internal_role = enums.NodeRole.storage
        fc_models = _node_type.FC_NODE_MODELS
        for model in fc_models:
            self.pending_active_node_dm.platform_info.node_type = \
                _node_type.NodeType(model)
            assert self.pending_active_node_dm.role is enums.NodeRole.fibre_channel


class TestRTFIAdapter(unittest.TestCase):
    def setUp(self):
        network_profile = models.NetworkProfile.get_mock_object()
        self.node = models.ResourceNode.get_mock_object(overrides=dict(network_profile=network_profile))
        self.adapter_node = models.RtfiNodeAdapter(self.node.serialize())

    def test_attributes(self):
        assert isinstance(self.adapter_node.to_dict(), dict)
        assert self.adapter_node.ip_address == self.node.mip
        assert self.adapter_node.local_attributes == {}
        assert self.adapter_node.name == self.node.node_name
        assert self.adapter_node.location == self.node.network_profile.location_name
        assert isinstance(self.adapter_node.network, models.RtfiNetworkProfileAdapter)
        assert self.adapter_node.is_virtual == self.node.model.is_virtual
        assert self.adapter_node.control_ip == self.node.oob_ip
        assert self.adapter_node.resource_type_name == self.adapter_node.type

        obj = {'test': 1}
        self.adapter_node.local_attributes = obj
        assert self.adapter_node.local_attributes == obj
