"""Constants for the backend monitor."""
import operator

from valence import (
    criteria,
    enums,
    models,
)

# Default list of faults that will always fail a test case
DEFAULT_ALWAYS_FAIL_FAULTS = (
    enums.ClusterFaultCode.duplicate_cluster_master_candidates,
    enums.ClusterFaultCode.fibre_channel_access_degraded,
    enums.ClusterFaultCode.fibre_channel_access_unavailable,
    enums.ClusterFaultCode.fibre_channel_config,
    enums.ClusterFaultCode.file_system_capacity_low,
    enums.ClusterFaultCode.fips_self_test_failure,
    enums.ClusterFaultCode.inconsistent_bond_modes,
    enums.ClusterFaultCode.inconsistent_interface_configuration,
    enums.ClusterFaultCode.inconsistent_mtus,
    enums.ClusterFaultCode.inconsistent_routing_rules,
    enums.ClusterFaultCode.inconsistent_subnet_masks,
    enums.ClusterFaultCode.incorrect_bond_port_count,
    enums.ClusterFaultCode.invalid_configured_fibre_channel_node_count,
    enums.ClusterFaultCode.irq_balance_failed,
    enums.ClusterFaultCode.memory_usage_threshold,
    enums.ClusterFaultCode.mtu_check_failure,
    enums.ClusterFaultCode.no_available_virtual_network_ip_addresses,
    enums.ClusterFaultCode.ssl_certificate_expiration,
    enums.ClusterFaultCode.time_since_last_full_gc_exceeded_limit,
    enums.ClusterFaultCode.unresponsive_service,
)

# Default list of events that will always fail a test case
DEFAULT_ALWAYS_FAIL_EVENTS = (
    enums.ClusterEventType.data_event,
    enums.ClusterEventType.ie_event,
    enums.ClusterEventType.fault_checker_event,
)

# Events that will raise a MonitorCriticalError and stop the test suite from running
CRITICAL_ERROR_EVENTS = (
    criteria.And(
        criteria.Equals(models.ClusterEvent.event_info_type, enums.ClusterEventType.ie_event),
        criteria.Equals(models.ClusterEvent.details, 'xPhysicalPageChecksumFailed',
                        getter=operator.itemgetter('exceptionCode'))
    ),
)
