import os
import unittest

import mock
import pytest
import repo_utils

from valence.scribe import (
    ScribeDocstringChecker,
    _errors,
)

CHECK_PATH = 'valence/unittest_utils/cases'


@mock.patch.object(repo_utils.Repo, 'get_py_changes')
@mock.patch('repo_utils.convert_to_python_path')
@mock.patch('camelot.driver.get_changesets')
@mock.patch('importlib.import_module')
@mock.patch('valence.scribe.ModuleDocumentation.test_case_docs', new_callable=mock.PropertyMock)
class TestScribeCheck(unittest.TestCase):
    def setUp(self):
        self.repo = repo_utils.GitRepo.from_path(os.path.dirname(__file__))
        self.repo.pycs_removed = True

    @mock.patch.object(ScribeDocstringChecker, '_get_tree_python_files')
    def test_scribe_step_errors(self, mock_get_tree, mock_tc_docs, mock_import, mock_changeset,
                                mock_python_path, mock_changes):
        mock_get_tree.return_value = ['/some/nonsense/path.py']
        mock_python_path.return_value = 'some.nonsense.path'
        mock_tc_doc = mock.MagicMock()
        mock_tc_doc.validate_qtest_fields.return_value = True
        mock_tc_doc.test_case_errors = [object()]
        mock_tc_docs.return_value = [mock_tc_doc]

        check = ScribeDocstringChecker(CHECK_PATH, repo=self.repo, modified_only=False)
        check_errors = check.run_verification()

        assert check_errors

    def test_qtest_docstring_errors(self, mock_tc_docs, mock_import, mock_changeset,
                                    mock_python_path, mock_changes):
        mock_changes.return_value = ['/some/nonsense/path.py']
        mock_python_path.return_value = 'some.nonsense.path'
        mock_tc_doc = mock.MagicMock()
        mock_tc_doc.validate_qtest_fields.return_value = False
        mock_tc_doc.get_qtest_errors.return_value = ['some error string']
        mock_tc_docs.return_value = [mock_tc_doc]

        check = ScribeDocstringChecker(CHECK_PATH, repo=self.repo, modified_only=False)
        check_errors = check.run_verification()

        assert check_errors

    def test_no_errors(self, mock_tc_docs, mock_import, mock_changeset,
                       mock_python_path, mock_changes):
        mock_changes.return_value = ['/some/nonsense/path.py']
        mock_python_path.return_value = 'some.nonsense.path'
        mock_tc_doc = mock.MagicMock()
        mock_tc_doc.validate_fields.return_value = True
        mock_tc_doc.test_case_errors = []
        mock_tc_docs.return_value = [mock_tc_doc]

        check = ScribeDocstringChecker(CHECK_PATH, repo=self.repo)
        check_errors = check.run_verification()

        assert not check_errors

    def test_no_errors_full_repo(self, mock_tc_docs, mock_import, mock_changeset,
                                 mock_python_path, mock_changes):
        mock_changes.return_value = []
        mock_tc_doc = mock.MagicMock()
        mock_tc_doc.validate_fields.return_value = True
        mock_tc_docs.return_value = [mock_tc_doc]

        check = ScribeDocstringChecker(CHECK_PATH, repo=self.repo, modified_only=False)
        check_errors = check.run_verification()

        assert not check_errors
