"""Test suite for Monitors to run Sanity."""
import os

import autologger
import logbook
import sf_artifacts

from valence import (
    camelot,
    cases,
    monitors,
)


class MonitorsSanitySuite(camelot.ElementTestSuite):
    """Test basic sanity steps.

    :param valence.models.SuiteResources resources: Resources data model for all test cases
    :param valence.models.SuiteConfig config: Suite configuration data model
    """
    _skip_remote_reporting = True

    def __init__(self, resources, config):
        super(MonitorsSanitySuite, self).__init__(resources, config)
        self.rs_cluster_dm = self.resources.cluster_dms[0]
        self.rs_node_dms = self.resources.node_dms
        self.rs_client_dms = self.resources.client_dms

        # Create the Monitor's FileHandler. Records should not bubble up to a higher handlers
        if self.local_artifacts_dir is not None:
            monitor_log_file = os.path.join(self.local_artifacts_dir, 'monitor.log')
            monitor_file_handler = logbook.FileHandler(
                filename=monitor_log_file,
                bubble=False,
                format_string=autologger.FORMAT_STRING
            )
        else:
            monitor_file_handler = None
        self.backend_monitor = monitors.BackendMonitor(resources, monitor_file_handler)

        manager = sf_artifacts.get_and_set_global_manager(config.artifact_credentials.serialize())
        self.add_tests([
            cases.MonitorsSanityCommitSmoke(
                rs_cluster_dm=self.rs_cluster_dm,
                rs_node_dms=self.rs_node_dms,
                rs_client_dm=self.rs_client_dms[0],
                artifact_manager=manager,
                backend_monitor=self.backend_monitor
            )
        ])

    def setup(self):
        """No setup needed yet."""
        self.backend_monitor.start_monitoring()

    def teardown(self):
        """No teardown needed yet."""
        self.backend_monitor.stop_monitoring()
