"""Test suite for Sanity."""
import sf_artifacts

from valence import (
    camelot,
    cases,
)


class SanitySuite(camelot.ElementTestSuite):
    """Test basic sanity steps.

    :param valence.models.SuiteResources resources: Resources data model for all test cases
    :param valence.models.SuiteConfig config: Suite configuration data model
    """
    _skip_remote_reporting = True

    def __init__(self, resources, config):
        super(SanitySuite, self).__init__(resources, config)
        self.rs_cluster_dm = self.resources.cluster_dms[0]
        self.rs_node_dms = self.resources.node_dms
        self.rs_client_dms = self.resources.client_dms

        manager = sf_artifacts.get_and_set_global_manager(config.artifact_credentials.serialize())
        self.add_tests([
            cases.SanityCommitSmoke(
                rs_cluster_dm=self.rs_cluster_dm,
                rs_node_dms=self.rs_node_dms,
                rs_client_dms=self.rs_client_dms,
                artifact_manager=manager
            ),
            cases.SanityClones(
                rs_cluster_dm=self.rs_cluster_dm,
                rs_node_dms=self.rs_node_dms,
                rs_client_dms=self.rs_client_dms,
                artifact_manager=manager
            )
        ])

    def setup(self):
        """No setup needed yet."""

    def teardown(self):
        """No teardown needed yet."""
