"""Contains a failing test case class."""
from valence import camelot


class FailingCase(camelot.TestCase):
    """A test case that fails an assert."""

    def _test(self):
        self._log_step('Assert True', 'Pass')
        assert True
        self._log_step('Assert False', 'Pass')  # wont actually pass, since it will fail
        assert False is True, 'False is not True'
        self._log_step('Assert True', 'Pass')
        assert True


class ExceptionInSetup(camelot.TestCase):
    """Test case that throws a given exception in setup.

    :param types.ClassType exception_type: Class of exception to throw (default=RuntimeError)
    """

    def __init__(self, exception_type=None):
        super(ExceptionInSetup, self).__init__()
        if exception_type is None:
            self.exception_type = RuntimeError
        else:
            self.exception_type = exception_type

    def _setup(self):
        raise self.exception_type('Setup failed')

    def _test(self):
        pass


class ExceptionInPreconditions(camelot.TestCase):
    """Test case that throws an exception in preconditions."""

    def _preconditions(self):
        assert False, 'Some bad stuff happened'

    def _test(self):
        pass


class ExceptionInTeardown(camelot.TestCase):
    """Test case that throws a given exception in teardown.

    :param types.ClassType exception_type: Class of exception to throw (default=RuntimeError)
    """

    def __init__(self, exception_type=None):
        super(ExceptionInTeardown, self).__init__()
        if exception_type is None:
            self.exception_type = RuntimeError
        else:
            self.exception_type = exception_type

    def _teardown(self):
        raise self.exception_type('Teardown failed')

    def _test(self):
        pass


class ExceptionInTest(camelot.TestCase):
    """Test case that throws an exception in the test method.

    :param types.ClassType exception_type: Class of exception to throw (default=RuntimeError)
    """

    def __init__(self, exception_type=None):
        super(ExceptionInTest, self).__init__()
        if exception_type is None:
            self.exception_type = RuntimeError
        else:
            self.exception_type = exception_type

    def _test(self):
        raise self.exception_type('Test failed')


class ExceptionInTestAndTeardown(camelot.TestCase):
    """Test case that throws an exception in the test and teardown methods.

    :param types.ClassType exception_type: Class of exception to throw (default=RuntimeError)
    """
    def __init__(self, exception_type=None):
        super(ExceptionInTestAndTeardown, self).__init__()
        if exception_type is None:
            self.exception_type = RuntimeError
        else:
            self.exception_type = exception_type

    def _test(self):
        raise self.exception_type('Test failed')

    def _teardown(self):
        raise self.exception_type('Teardown failed')


class ExceptionInPostTestAssertions(camelot.TestCase):
    """Test case that throws an exception in the test and teardown methods.

    :param types.ClassType exception_type: Class of exception to throw (default=RuntimeError)
    """
    def __init__(self, exception_type=None):
        super(ExceptionInPostTestAssertions, self).__init__()
        if exception_type is None:
            self.exception_type = RuntimeError
        else:
            self.exception_type = exception_type

    def _test(self):
        assert True

    def _post_test_assertions(self):
        raise self.exception_type('Post Test Assertions failed')


class ExceptionInTestAndPostTestAssertions(camelot.TestCase):
    """Test case that throws an exception in the test and teardown methods.

    :param types.ClassType exception_type: Class of exception to throw (default=RuntimeError)
    """
    def __init__(self, exception_type=None):
        super(ExceptionInTestAndPostTestAssertions, self).__init__()
        if exception_type is None:
            self.exception_type = RuntimeError
        else:
            self.exception_type = exception_type

    def _test(self):
        raise self.exception_type('Test failed')

    def _post_test_assertions(self):
        raise self.exception_type('Post Test Assertions failed')
