"""Some feature specific case examples."""
from valence import camelot


class ExampleFeatureTestCase(camelot.TestCase):
    """This is an example feature-specific test case.

    The assumption is that the api provided in tests/generic doesn't work
    for this test case.

    This test case takes in a cluster and asserts that it can create and
    delete volumes.

    :param sf_platform.models.Cluster cluster_dm: the cluster to run the test on.

    :tc_info ID: 1
    :tc_info Requirements: FR-EX-1
    :tc_info Feature: EXP
    :tc_info CreatedOn: 2/8/17
    :tc_info Author: garrett.berg@solidfire.com
    :tc_info LastUpdate: mike.watson@solidfire.com
    :tc_info LastModifiedOn: 3/1/17
    :tc_info Regression: Y
    """

    def __init__(self, cluster_dm):
        super(ExampleFeatureTestCase, self).__init__()
        self.cluster_dm = cluster_dm

    def _preconditions(self):
        """Preconditions.

        :tc_info MinNode: 1
        :tc_info MinFDVA: 1
        :tc_info MinClients: 1
        """
        self.volumes_cmt = 'Does Nothing'

    def _setup(self):
        """No setup for this test."""

    def _test(self):
        """This is a a docstring."""

        self._log_step(action='Create 10 Volumes', expected_result='List Volumes returns 10 volumes')
        self._log_step('Delete 10 Volumes', 'List Volumes returns 0 volumes')


class SecondExampleFeatureTestCase(camelot.TestCase):
    """This is another example feature-specific test case.

    The assumption is that the api provided in tests/generic doesn't work
    for this test case.

    This test case takes in a cluster and asserts that it can create and
    delete volumes.

    :param sf_platform.models.Cluster cluster_dm: the cluster to run the test on.

    :tc_info ID: 2
    :tc_info Requirements: FR-EX-2
    :tc_info Feature: OTHER
    :tc_info CreatedOn: 4/25/17
    :tc_info Author: sean.turner@solidfire.com
    :tc_info LastUpdate: new.user@solidfire.com
    :tc_info LastModifiedOn: 4/25/17
    :tc_info Regression: N
    """

    def __init__(self, cluster_dm):
        super(SecondExampleFeatureTestCase, self).__init__()
        self.cluster_dm = cluster_dm

    def _preconditions(self):
        """Preconditions.

        :tc_info MinNode: 4
        :tc_info MinFDVA: 1
        :tc_info MinClients: 1

        """
        self.volumes_cmt = 'Does Nothing'

    def _setup(self):
        """No setup for this test."""

    def _test(self):
        """This is a a docstring."""

        self._log_step('Create 10 Volumes', 'List Volumes returns 10 volumes')
        self._log_step('Delete 10 Volumes', 'List Volumes returns 0 volumes')
        self._log_step('Create 10 Volumes', 'List Volumes returns 10 volumes')
        self._log_step('Delete 10 Volumes', 'List Volumes returns 0 volumes')
        self._log_step('Create 10 Volumes', 'List Volumes returns 10 volumes')
        self._log_step('Delete 10 Volumes', 'List Volumes returns 0 volumes')

    def _teardown(self):
        del self.volumes_cmt


class ThirdExampleFeatureTestCase(camelot.TestCase):
    """Terrible description.

    :param sf_platform.models.Cluster cluster_dm: the cluster to run the test on.

    :tc_info id: 3
    :tc_info ReQuirements: FR-EX-1
    :tc_info feature: EXP
    :tc_info createdOn: 12/10/2020
    :tc_info Last_update: mike.watson@solidfire.com
    :tc_info LastModified: 3/1/17
    """

    def __init__(self, cluster_dm):
        super(ThirdExampleFeatureTestCase, self).__init__()
        self.cluster_dm = cluster_dm
        self.volumes_cmt = object()

    def _test(self):
        """This is a a docstring."""

        self._log_step('Create 10 Volumes', 'List Volumes returns 10 volumes')
        self._log_step('Delete 10 Volumes', 'List Volumes returns 0 volumes')
        self._log_step('Create 10 Volumes', 'List Volumes returns 10 volumes')
        self._log_step('Delete 10 Volumes', 'List Volumes returns 0 volumes')
        self._log_step('Create 10 Volumes', 'List Volumes returns 10 volumes')
        self._log_step('Delete 10 Volumes', 'List Volumes returns 0 volumes')

    def _teardown(self):
        """3rd testcase teardown."""
        del self.volumes_cmt

    def _post_test_assertions(self):
        """Verifies that everything is fine in post-test."""


class TryingToUseCVARTestCase(camelot.TestCase):
    """Terrible description.

    :tc_info id: 3
    :tc_info ReQuirements: FR-EX-1
    :tc_info feature: EXP
    :tc_info createdOn: 12/10/2020
    :tc_info Last_update: mike.watson@solidfire.com
    :tc_info LastModified: 3/1/17
    """

    ACTION = 'WTF'

    def _test(self):
        """This is a a docstring."""
        self._log_step(self.ACTION)
