"""Suites that contains a failing test cases."""
from valence import monitors
from valence.unittest_utils import cases
from valence.unittest_utils.suites.base import UnittestTestSuite
from valence.utils import time_utils


class FailingSuite(UnittestTestSuite):
    """Contains a single failing test case that fails an assert."""
    def __init__(self, resources_dm, config_dm):
        super(FailingSuite, self).__init__(resources_dm, config_dm)
        self.add_tests([cases.FailingCase()])

    def setup(self):
        pass


class FailingSuitePostTestAssertions(UnittestTestSuite):
    """Contains a single failing test case that fails an assert in post test assertions."""
    def __init__(self, resources_dm, config_dm):
        super(FailingSuitePostTestAssertions, self).__init__(resources_dm, config_dm)
        self.add_tests([cases.ExceptionInPostTestAssertions()])

    def setup(self):
        pass


class FailingSuitePostTestAssertionsAndTest(UnittestTestSuite):
    """Contains a single failing test case that fails an assert in post test assertions."""
    def __init__(self, resources_dm, config_dm):
        super(FailingSuitePostTestAssertionsAndTest, self).__init__(resources_dm, config_dm)
        self.add_tests([cases.ExceptionInTestAndPostTestAssertions()])

    def setup(self):
        pass


class FailingSuiteTCError(UnittestTestSuite):
    """Contains a single failing test case that fails an assert."""
    def __init__(self, resources_dm, config_dm):
        super(FailingSuiteTCError, self).__init__(resources_dm, config_dm)
        self.add_tests([cases.ExceptionInSetup()])

    def setup(self):
        pass


class FailInPreconditions(UnittestTestSuite):
    """One of the test cases fails in preconditions."""
    def __init__(self, resources_dm, config_dm):
        super(FailInPreconditions, self).__init__(resources_dm, config_dm)
        self.add_tests([
            cases.ExceptionInPreconditions(),
            cases.PassingCase(),
        ])


class MonitorFailuresSuite(UnittestTestSuite):
    """A failure occurs in monitoring."""
    _skip_monitoring = False

    def __init__(self, resources, config):
        super(MonitorFailuresSuite, self).__init__(resources, config)
        self.add_tests([
            cases.PassingCase(),
        ])

    def _check_for_monitor_failures(self, run):
        failures = [
            monitors.MonitorFailure('Unexpected Fault: ABC', time_utils.get_now_utc()),
            monitors.MonitorFailure('Unexpected Fault: DEFG', time_utils.get_now_utc()),
        ]
        self.result.monitor_failures.extend(failures)
