"""Suites for testing KeyboardInterrupt functionality."""
import types  # pylint: disable=unused-import

from valence.unittest_utils.cases import (
    failing_case,
    passing_case,
)
from valence.unittest_utils.suites import (
    base,
    exception_in_setup,
)


class KeyboardInterruptInSetupSuite(exception_in_setup.ExceptionSuite):
    """Suite throws a keyboard interrupt in setup."""
    EXCEPTION_TYPE = KeyboardInterrupt


class KeyboardInterruptInTeardownSuite(base.UnittestTestSuite):
    """Suite throws a keyboard interrupt in teardown."""
    def __init__(self, resources, config):
        super(KeyboardInterruptInTeardownSuite, self).__init__(resources, config)
        self.add_tests([passing_case.PassingCase()])

    def teardown(self):
        raise KeyboardInterrupt('Teardown failed')


class KeyboardInterruptInCaseSetupSuite(base.UnittestTestSuite):
    """Suite throws a keyboard interrupt in test case setup."""
    def __init__(self, resources, config):
        super(KeyboardInterruptInCaseSetupSuite, self).__init__(resources, config)
        keyboard_interrupt = KeyboardInterrupt  # type: types.ClassType
        self.add_tests([
            passing_case.PassingCase(),
            failing_case.ExceptionInSetup(exception_type=keyboard_interrupt)
        ])


class KeyboardInterruptInCaseTestSuite(base.UnittestTestSuite):
    """Suite throws a keyboard interrupt in the test case."""
    def __init__(self, resources, config):
        super(KeyboardInterruptInCaseTestSuite, self).__init__(resources, config)
        keyboard_interrupt = KeyboardInterrupt  # type: types.ClassType
        self.add_tests([
            failing_case.ExceptionInTest(exception_type=keyboard_interrupt),
            passing_case.PassingCase()
        ])


class KeyboardInterruptInCaseTeardownSuite(base.UnittestTestSuite):
    """Suite throws a keyboard interrupt in test case teardown."""
    def __init__(self, resources, config):
        super(KeyboardInterruptInCaseTeardownSuite, self).__init__(resources, config)
        keyboard_interrupt = KeyboardInterrupt  # type: types.ClassType
        self.add_tests([
            failing_case.ExceptionInTeardown(exception_type=keyboard_interrupt)
        ])
