"""Contains suite that passes."""
from valence import enums
from valence.monitors import _constants
from valence.unittest_utils.cases import (
    failing_case,
    passing_case,
)
from valence.unittest_utils.suites.base import UnittestTestSuite


class PassingSuite(UnittestTestSuite):
    """Contains a single test case that passes."""

    def __init__(self, resources_dm, config_dm):
        super(PassingSuite, self).__init__(resources_dm, config_dm)
        self.add_tests([passing_case.PassingCase()])

    def setup(self):
        pass


class PassingElementSuite(UnittestTestSuite):
    """Contains a single test case that passes."""

    def __init__(self, resources_dm, config_dm):
        super(PassingElementSuite, self).__init__(resources_dm, config_dm)
        case = passing_case.PassingElementCaseWithAttributes()
        self.add_tests([case])

    def _post_test_case_execute(self, run):
        """Override _post_test_case_execute to test that the attributes are properly set.

        Attributes that are set in the _pre_test_case_execute function are checked here before
        calling the base class function.
        """
        events_set = set(_constants.DEFAULT_ALWAYS_FAIL_EVENTS)
        events_set.remove(enums.ClusterEventType.ie_event)

        assert self._backend_monitor.get_always_fail_events() == events_set

        faults_set = set(_constants.DEFAULT_ALWAYS_FAIL_FAULTS)
        faults_set.remove(enums.ClusterFaultCode.mtu_check_failure)

        assert self._backend_monitor.get_always_fail_faults() == faults_set

        super(PassingElementSuite, self)._post_test_case_execute(run)

    def setup(self):
        pass


class PassingSuiteArtifacts(UnittestTestSuite):
    """Contains a single test case that passes.

    Allow artifacts to write. Ensure you mock properly.
    """
    _skip_artifact_dir = False

    def __init__(self, resources_dm, config_dm):
        super(PassingSuiteArtifacts, self).__init__(resources_dm, config_dm)
        self.add_tests([passing_case.PassingCase()])

    def setup(self):
        pass


class PassingTwoTestSuite(UnittestTestSuite):
    """Contains two test cases that passes."""

    def __init__(self, resources_dm, config_dm):
        super(PassingTwoTestSuite, self).__init__(resources_dm, config_dm)
        self.add_tests([passing_case.PassingCase(), passing_case.PassingCase()])

    def setup(self):
        pass


class PassingGrootSuite(UnittestTestSuite):
    """Runs two test cases and pushes to Groot and writes artifacts."""

    _skip_artifact_dir = False
    _skip_remote_reporting = False

    def __init__(self, resources_dm, config_dm):
        super(PassingGrootSuite, self).__init__(resources_dm, config_dm)
        self.add_tests([passing_case.PassingCase(), passing_case.PassingCaseWithAssertions()])

    def setup(self):
        pass


class MultiErrorFailingSuiteArtifacts(UnittestTestSuite):
    """Contains a passing and failing test case. Allow artifacts to write. Ensure you mock properly.

    Tests will result in the following statuses [pass, fail]
    """
    _skip_artifact_dir = False

    def __init__(self, resources_dm, config_dm):
        super(MultiErrorFailingSuiteArtifacts, self).__init__(resources_dm, config_dm)
        self.add_tests([passing_case.PassingCase(), failing_case.ExceptionInSetup()])

    def setup(self):
        pass
