"""Utility for filtering strings via regular expressions.

See criteria.py for utilities for filtering data models.
"""
import re


def regex_filter(input_list, regex_list, negate=False):
    """Return strings that match a list of regexes.

    :param list[str] input_list: The list to apply the filtering on.
    :param list[str] regex_list: What to filter with.
    :param bool negate: If True, return strings that do not match the regexes
    :rtype: list[str]
    """
    match_list = []
    for s in input_list:
        match = False
        for regex in regex_list:
            if re.match(regex, s):
                match = True
                break
        if negate and not match:
            match_list.append(s)
        elif not negate and match:
            match_list.append(s)
    return match_list
