"""Timeout related utilities."""
import time


def timeout_timer(timeout_length):
    """Generator to time something.

    Returns a generator object so you can say:

         >>> timer = timeout_timer(5)
         >>> while next(timer):
         >>>     pass  # do something
         >>> raise RuntimeError('Operation timed out!')

    :param int timeout_length: timeout as an integer
    :rtype: collection.Iterable
    """
    start_time = time.time()
    while True:
        yield time.time() - float(start_time) <= float(timeout_length)
