"""Valence Bin Assignment Components."""

from valence import (
    components,
    functional_models,
)


class ContinuousValidateBinAssignments(components.ContinuousTransient):
    """Continuously validate bin assignment distribution, status, and balance.

    :param valence.models.ResourceCluster cluster_dm: Resource cluster data model. This is needed for
        cluster credentials if the nodes are part of a cluster.
    :param int kill_after_seconds: Time to run the component
    :param int sleep_time: How long in seconds to sleep between loops
    :param int max_iterations: How many iterations to loop
    """

    def __init__(self, cluster_dm, kill_after_seconds=None, sleep_time=None, max_iterations=None):
        super(ContinuousValidateBinAssignments, self).__init__(kill_after_seconds, sleep_time,
                                                               max_iterations)
        self.cluster_fm = functional_models.Cluster(cluster_dm)

    def _apply_continuous(self):
        """Get and validate bins.json in a loop."""
        self.cluster_fm.get_bin_assignments(validate=True)

    def _reset(self, timeout=None):
        pass
