import unittest

import mock
import pytest
from sf_models import at2_enums

from valence import models
from valence.vl_components import mfa


class TestDockerizedShibbolethIdpDeployComponent(unittest.TestCase):
    """Test the Dockerized Shibboleth Idp Deployment component."""

    def test_init(self):
        client_dm = models.ResourceClient.get_mock_object(
            overrides=dict(os=at2_enums.ClientOS.ubuntu_18_04))

        test_dockerized_shib_deploy = mfa.DockerizedShibbolethIdpDeployComponent(client_dm)
        assert isinstance(test_dockerized_shib_deploy, mfa.DockerizedShibbolethIdpDeployComponent)


class TestShibbolethUserManagementComponent(unittest.TestCase):
    """Test the Dockerized Shibboleth Idp user management component."""

    def test_init(self):
        client_dm = models.ResourceClient.get_mock_object(
            overrides=dict(os=at2_enums.ClientOS.ubuntu_18_04))

        # TODO ELEM-7193: Refined the param definition for idp_user_dm when that data model exists.
        idp_user_dm = mock.Mock()

        test_shib_user_cmt = mfa.ShibbolethUserManagementComponent(client_dm, idp_user_dm)
        assert isinstance(test_shib_user_cmt, mfa.ShibbolethUserManagementComponent)


class TestAdfsIdpDeployComponent(unittest.TestCase):
    """Test the ADFS deployment management component."""

    def test_init(self):
        client_dm = models.ResourceClient.get_mock_object(
            overrides=dict(os=at2_enums.ClientOS.windows_2008))

        test_adfs_user_mgt_cmt = mfa.AdfsIdpDeployComponent(client_dm)
        assert isinstance(test_adfs_user_mgt_cmt, mfa.AdfsIdpDeployComponent)


class TestAdfsUserManagementComponent(unittest.TestCase):
    """Test the ADFS user management component."""

    def test_init(self):
        client_dm = models.ResourceClient.get_mock_object(
            overrides=dict(os=at2_enums.ClientOS.windows_2008))

        # TODO ELEM-7193: Refined the param definition for idp_user_dm when that data model exists.
        idp_user_dm = mock.Mock()

        test_adfs_user_mgt_cmt = mfa.AdfsUserManagementComponent(client_dm, idp_user_dm)
        assert isinstance(test_adfs_user_mgt_cmt, mfa.AdfsUserManagementComponent)
