import unittest

import mock
import pytest
import sf_ssh

from camelot.models import at2_enums
from valence import (
    functional_models,
    models,
    vl_components,
)
from valence.functional_models import _errors


@mock.patch.object(functional_models.LinuxClient, 'get_ssh_connection')
@mock.patch.object(functional_models.LinuxClient, 'create_file')
@mock.patch.object(functional_models.LinuxClient, 'get_file_contents')
@mock.patch.object(functional_models.LinuxClient, 'create_temp_cert_dir')
@mock.patch.object(functional_models.Cluster, 'set_ssl_certificate')
@mock.patch.object(functional_models.Cluster, 'remove_ssl_certificate')
@mock.patch.object(functional_models.ManagementNodeAdapter, 'set_node_ssl_certificate')
@mock.patch.object(functional_models.ManagementNodeAdapter, 'remove_node_ssl_certificate')
@mock.patch.object(functional_models.ManagementNodeAdapter, 'authenticate')
@mock.patch('time.sleep')
class TestCertComponents(unittest.TestCase):
    def setUp(self):
        self.client_dm = models.ResourceClient.get_mock_object(
            overrides=dict(os=at2_enums.ClientOS.ubuntu_14_04))
        self.cluster_dm = models.Cluster.get_mock_object()
        self.mnode_dm = models.ResourceManagementNode.get_mock_object()

    def test_certs(self, mock_sleep, mock_auth, mock_rmn, mock_sn, mock_r, mock_s, mock_tmp, mock_cat,
                   mock_create, mock_ssh):
        cert_comp = vl_components.CreateSelfSignedIPCert(self.client_dm, self.cluster_dm, self.mnode_dm)
        mock_tmp.return_value = '/tmp'
        mock_cat.return_value = 'unittest'
        run_command = mock_ssh.return_value.__enter__.return_value.run_command
        run_command.return_value = sf_ssh.RunCommandOutput(stdout='/tmp',
                                                           stderr='',
                                                           return_code=0)
        cert_comp.apply()
        cert_comp.reset()

    def test_revoked(self, mock_sleep, mock_auth, mock_rmn, mock_sn, mock_r, mock_s, mock_tmp, mock_cat,
                     mock_create, mock_ssh):
        cert_comp = vl_components.CreateSelfSignedIPCert(self.client_dm, self.cluster_dm, self.mnode_dm)
        mock_tmp.return_value = '/tmp'
        cert_comp.temp_dir = 'unittest'
        run_command = mock_ssh.return_value.__enter__.return_value.run_command
        ssh_list = [sf_ssh.RunCommandOutput(stdout='', stderr='', return_code=0)] * 24
        ssh_list.append(_errors.FunctionalClientError('message'))
        run_command.side_effect = iter(ssh_list)
        cert_comp.apply()
        with pytest.raises(_errors.FunctionalClientError):
            cert_comp.reset()
        cert_comp = vl_components.CreateSelfSignedIPCert(self.client_dm, self.cluster_dm, self.mnode_dm)
        cert_comp.temp_dir = 'unittest'
        ssh_list = [sf_ssh.RunCommandOutput(stdout='', stderr='', return_code=0)] * 24
        ssh_list.append(_errors.FunctionalClientError('Already revoked'))
        run_command.side_effect = iter(ssh_list)
        cert_comp.apply()
        cert_comp.reset()

    def test_CAs(self, mock_sleep, mock_auth, mock_rmn, mock_sn, mock_r, mock_s, mock_tmp, mock_cat,
                 mock_create, mock_ssh):
        cssdom = vl_components.CreateSelfSignedDOMCert(self.client_dm, self.cluster_dm, self.mnode_dm)
        caip = vl_components.CreateCertAuthorityIPCert(self.client_dm, self.cluster_dm, self.mnode_dm)
        cadom = vl_components.CreateCertAuthorityDOMCert(self.client_dm, self.cluster_dm, self.mnode_dm)
        caipsoon = vl_components.CreateCertAuthorityIPSoonToExpire(self.client_dm, self.cluster_dm,
                                                                   self.mnode_dm)
        cadomsoon = vl_components.CreateCertAuthorityDOMSoonToExpire(self.client_dm, self.cluster_dm,
                                                                     self.mnode_dm)
        caipe = vl_components.CreateCertAuthorityIPExpired(self.client_dm, self.cluster_dm,
                                                           self.mnode_dm)
        cadome = vl_components.CreateCertAuthorityDOMExpired(self.client_dm, self.cluster_dm,
                                                             self.mnode_dm)
        run_command = mock_ssh.return_value.__enter__.return_value.run_command
        run_command.return_value = sf_ssh.RunCommandOutput(stdout='/tmp',
                                                           stderr='',
                                                           return_code=0)
        mock_tmp.return_value = '/tmp'
        mock_cat.return_value = 'unittest'
        cssdom.apply()
        caip.apply()
        cadom.apply()
        caipsoon.apply()
        cadomsoon.apply()
        caipe.apply()
        cadome.apply()
