"""Test valence.vl_components._constants"""
import unittest

import mock
import pytest

from valence import models
from valence.vl_components import _constants


@mock.patch('valence.functional_models.Cluster.set_constants')
@mock.patch('valence.functional_models.Cluster.get_constants')
class TestSetClusterConstantsComponents(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.cluster_dm = models.ResourceCluster.get_mock_object()
        cls.cluster_constants = models.ClusterConstants.get_mock_object()

    def test_set_cluster_constants(self, mock_get_cluster_const, mock_set_cluster_const):
        mock_get_cluster_const.return_value = self.cluster_constants

        cluster_constants = models.ClusterConstants.create(max_volumes_per_multi_api=42)
        set_constants_cmt = _constants.SetClusterConstants(self.cluster_dm, cluster_constants)
        set_constants_cmt.apply()
        mock_set_cluster_const.assert_called_with(cluster_constants)
        set_constants_cmt.reset()
        mock_set_cluster_const.assert_called_with(self.cluster_constants)

    def test_disable_bin_sync(self, mock_get_cluster_const, mock_set_cluster_const):
        mock_get_cluster_const.return_value = self.cluster_constants

        set_constants_cmt = _constants.DisableBinSync(self.cluster_dm)
        set_constants_cmt.apply()
        mock_set_cluster_const.assert_called_with(set_constants_cmt.bin_sync_constants)
        set_constants_cmt.reset()
        mock_set_cluster_const.assert_called_with(self.cluster_constants)

    def test_disable_auto_slice_balancing(self, mock_get_cluster_const, mock_set_cluster_const):
        mock_get_cluster_const.return_value = self.cluster_constants

        set_constants_cmt = _constants.DisableAutomaticSliceBalancing(self.cluster_dm)
        set_constants_cmt.apply()
        mock_set_cluster_const.assert_called_with(set_constants_cmt.slice_balance_constants)
        set_constants_cmt.reset()
        mock_set_cluster_const.assert_called_with(self.cluster_constants)

    def test_disable_auto_gc(self, mock_get_cluster_const, mock_set_cluster_const):
        mock_get_cluster_const.return_value = self.cluster_constants

        set_constants_cmt = _constants.DisableAutomaticGarbageCollection(self.cluster_dm)
        set_constants_cmt.apply()
        mock_set_cluster_const.assert_called_with(set_constants_cmt.gc_constants)
        set_constants_cmt.reset()
        mock_set_cluster_const.assert_called_with(self.cluster_constants)

    def test_enable_merkle_checker(self, mock_get_cluster_const, mock_set_cluster_const):
        mock_get_cluster_const.return_value = self.cluster_constants

        set_constants_cmt = _constants.EnableMerkleChecker(self.cluster_dm)
        set_constants_cmt.apply()
        mock_set_cluster_const.assert_called_with(set_constants_cmt.merkle_constants)
        set_constants_cmt.reset()
        mock_set_cluster_const.assert_called_with(self.cluster_constants)

    def test_set_drive_service_timeout(self, mock_get_cluster_const, mock_set_cluster_const):
        mock_get_cluster_const.return_value = self.cluster_constants

        with pytest.raises(ValueError):
            set_constants_cmt = _constants.SetDriveServiceTimeout(self.cluster_dm, timeout=301)

        set_constants_cmt = _constants.SetDriveServiceTimeout(self.cluster_dm)
        set_constants_cmt.apply()
        mock_set_cluster_const.assert_called_with(set_constants_cmt.service_timeout_constants)
        set_constants_cmt.reset()
        mock_set_cluster_const.assert_called_with(self.cluster_constants)
