"""Test valence.vl_components._time."""

import unittest

import mock

from sf_api import result_models
from valence import (
    functional_models,
    vl_components,
)


@mock.patch('valence.functional_models.Cluster.get_ntp_info')
@mock.patch('valence.functional_models.Cluster.set_ntp_info')
class TestSetNTPServers(unittest.TestCase):
    """Unit tests for the Set NTP Info component."""

    @classmethod
    def setUpClass(cls):
        cls.cluster_fm = functional_models.Cluster.get_mock_fm()

    def test_different_ntp_servers(self, mock_set_ntp_info, mock_get_ntp_info):
        """Test doing a basic config of the component using both inputs."""
        orig_servers = ['ntp.solidfire.com']
        orig_broadcast_client = True

        servers = ['1.1.1.1',
                   'bigben.netapp.com',
                   '1200:0000:AB00:1234:0000:2552:7777:1313',
                   '[2001:db8:0:1]']
        broadcast_client = False

        mock_get_ntp_info.return_value = result_models.GetNtpInfo.get_mock_response(
            overrides={'servers': orig_servers,
                       'broadcast_client': orig_broadcast_client}).result_dm

        cmt = vl_components.SetNTPInfo(cluster_dm=self.cluster_fm.cluster_dm,
                                       ntp_servers=servers,
                                       broadcast_client=broadcast_client)
        cmt.apply()

        err_msg = ('Previous ntp server info does not match expected.\n'
                   'Expected = {}\nReturned = {}')
        self.assertListEqual(cmt.previous_ntp_info.servers, orig_servers,
                             err_msg.format(servers, cmt.previous_ntp_info))
        self.assertEqual(cmt.previous_ntp_info.broadcast_client, orig_broadcast_client,
                         err_msg.format(servers, cmt.previous_ntp_info))
        mock_set_ntp_info.assert_called_with(servers=servers,
                                             broadcast_client=broadcast_client)

        cmt.reset()
        self.assertTrue(mock_set_ntp_info.called, 'SetNTPInfo never called')
        mock_set_ntp_info.assert_called_with(servers=orig_servers,
                                             broadcast_client=orig_broadcast_client)
