"""Vdbench helper methods."""
from sf_ssh import ssh

from valence.vl_components.vdbench import _errors


def tail_vdb_log(client_dm, log_file_path, start_line, max_lines):
    """Tail the master log starting at the last line read.

    :param valence.models.ResourceClient client_dm: Client to get log from.
    :param str log_file_path: Path to the logfile on the client.
    :param int start_line: Line to start at.
    :param int max_lines: Maximum number of lines to return.
    :rtype: list[str]
    """
    # Use tail to get subsequent lines
    with ssh.Ssh(client_dm.mip) as conn:
        run_command_output = conn.run_command('tail -n +{} {} | head -n {}'.format(
            start_line + 1, log_file_path, max_lines))
    # No more new lines because the logfile is cleaned up
    if run_command_output.return_code == 1 and len(run_command_output.stderr) == 1:
        raise _errors.VdbenchLogFileMissingError(
            '{}:{}'.format(client_dm, log_file_path))
    # No more new lines for some other (unknown) reason
    if run_command_output.return_code != 0 or run_command_output.stderr:
        raise _errors.VdbenchScraperError(
            'Error reading logfile {} on {}'.format(log_file_path, client_dm))
    return run_command_output.stdout.split('\n')
