"""Vdbench log constants.

Regex for different types of lines in the Vdbench master log.
"""
import re

# 03:23:01.003 Starting RD=r0000; I/O rate: 2000; elapsed=20; For loops: None
RUN_START_LINE = re.compile(r'\d*:\d*:\d*\.\d*\s+Starting RD=([\w\d]+);')
RUN_DATE_LINE = re.compile(r'\w*\s*\d*,\s*\d*\s*interval\s*i/o\s*MB/sec')
IO_LINE = re.compile(
    r'\d\d:\d\d:\d\d\.\d{1,9}\s+\d{1,9}(\s+\d{1,9}\.\d{1,3}){2}\s+\d{1,9}(\s+(\d{1,9}\.\d{1,3}|NaN)){9}'
)
AVG_IO_LINE = re.compile(r'\d*:\d*:\d*\.\d*\s*avg_\d*-\d*(\s*\d*\.\d*){2}\s*\d*(\s*(\d*\.\d*|NaN)){9}')
CLIENT_HEARTBEAT_LINE = re.compile(
    r'\d*:\d*:\d*\.\d*\s*HeartBeat\.checkHeartBeat\(\):\sslave\s.*has\snot\sresponded\sfor\s125\sseconds'
)
MASTER_HEARTBEAT_LINE = re.compile(
    r'\d*:\d*:\d*\.\d*\s*Slave\shd\d*-\d*\saborting:\sHeartbeat\smonitor:'
)

# "18:32:15.022 * Slave h0000-7 aborting: No read validations done ...
NO_READ_VALIDATIONS_LINE = re.compile(
    r'\d*:\d*:\d*\.\d*\s\*\s+Slave\s([\w-]+)\saborting: '
    r'No read validations done during a Data Validation run. \*'
)

# ERROR Lines
EIO_LINE = re.compile(r'EIO I/O error')
EOF_LINE = re.compile(r'\d*:\d*:\d*\.\d*\s*Receiving\s*unexpected\s*EOFException\s*from\s*slave:')
SOCKET_EXCEPTION_LINE = re.compile(r'\d*:\d*:\d*\.\d*\s*SocketException\s*from\s*slave:')
DATA_VALIDATION_ERROR_LINE = re.compile(r'At least one Data Validation error detected.')

# JAVA Lines
JSTACK_LINE = re.compile(r'\d*:\d*:\d*\.\d*\s*execute\(\):\s*.*\sjstack')
JAVA_POINTER_ERROR_LINE = re.compile(r'java\.lang\.NullPointerException')
JAVA_RUNTIME_ERROR_LINE = re.compile(r'java\.lang\.RuntimeException')
PREMATURE_TERMINATION_LINE = re.compile(
    r'java\.lang\.RuntimeException: Slave ([\w-]+) prematurely terminated.'
)
