"""Vdbench host definition objects."""

from camelot.models import at2_enums
from valence.vl_components.vdbench._parm import (
    base,
    constants,
    utils,
)


class HostDef(base.ParmLine):
    """Vdbench host definition.

    :param str name: Name for this host definition.
    :param valence.models.ResourceClient client_dm: Client model for this host definition.
    :param int jvms: The number of java virtual machines to use (default: Inferred from client type).
    :param str vdbench: Shell type to use (i.e. rsh|ssh|vdbench) (default: vdbench).
    :param str shell: The shell command to use.
    :param str user: Username for this host definition (default: Inferred from client type).
    :param str mount: The mount location.
    :param str comment: A comment to put above the line in the _parm file. Broken up into multiple lines
        based on the presence of '/n' characters.
    """

    def __init__(self, name, client_dm, jvms=None, vdbench='/opt/vdbench', shell='vdbench',
                 user=None, mount=None, comment=None):
        super(HostDef, self).__init__(comment)
        self.name = name
        self.client_dm = client_dm
        if self.client_dm:
            if jvms is None:
                jvms = 1 if self.client_dm.type is at2_enums.ClientType.virtual else 8
            if user is None:
                user = 'administrator' if self.client_dm.os in constants.WIN_OS else 'root'
        self.system = str(self.client_dm.mip)
        self.jvms = jvms
        self.vdbench = vdbench
        self.shell = shell
        self.user = user
        self.mount = mount

    def _parm_str(self):
        """The specific representation of this item as it may appear in the Vdbench parm file.

        :rtype: str
        """
        config = ['hd={}'.format(self.name)]
        utils.extend_config(config, self.system, constants.SYSTEM)
        utils.extend_config(config, self.jvms, constants.JVMS)
        utils.extend_config(config, self.vdbench, constants.VDBENCH)
        utils.extend_config(config, self.shell, constants.SHELL)
        utils.extend_config(config, self.user, constants.USER)
        utils.extend_config(config, self.mount, constants.MOUNT)
        return ','.join(config)
