"""Vdbench parm file utility methods."""
import aenum

from valence.vl_components.vdbench._parm import base


def tuple_str(value):
    """Convert a tuple to a vdb tuple str.

    :param typing.Iterable[Any] value: The tuple to convert
    :rtype: str
    """
    return '({})'.format(','.join(map(str, value)))


def bytes_str(value):
    """Convert a byte value to an easy-to-read string recognized by Vdbench.

    :param int|None value: Value to represent in bytes.
    :rtype: str
    """
    if value is None:
        return None
    elif value % 1024 == 0:
        return '{}k'.format(value // 1024)
    elif value % 512 == 0:
        return '{}.5k'.format(value // 1024)
    else:
        return str(value)


def extend_config(config, item, name=None):
    """Extend the provided configuration with the specified item.

    :param list config: List of configuration items to extend.
    :param Any item: The configuration item to add.
    :param str name: The name of the configuration item.
    """
    if item is None:
        pass
    elif isinstance(item, base.ConfigItem):
        config.append(item.get_parm_str())
    else:
        assert name is not None
        if isinstance(item, aenum.Enum):
            config.append('{}={}'.format(name, item.value))
        elif isinstance(item, (list, tuple)):
            config.append('{}={}'.format(name, tuple_str(item)))
        else:
            config.append('{}={}'.format(name, item))
