"""Vdbench profile objects.

The Vdbench Profile objects are the primary interface in which you create vdbench components from
existing cluster, clients, connections-components and/or volumes components. They are _helper_ classes
that allow you to create a "default" profile and then modify that to suit your use case as needed,
including mutating it to add volumes/connections, change workloads, etc.

Once the settings are as you want them, you can call `.get_environment()` to get the
`ManageSetupVdbEnvironments` component.

Unlike components, these objects can be safely **copied** and **mutated**.

Rules:
    * Follow general best practices to not mutate the caller's objects (call ``.copy()`` first)
    * Never mutate the volume/connection component instance variables (never mutate component instances)

Example Usage:
    Setup::

        vdb_profile = StandardVdbProfile(connections_cmt)
        setup_vdb_cmt = vdb_profile.get_environment()
        run_vdb_cmt = RunVdb(setup_vdb_cmt)
        setup_vdb_cmt.apply()

    Run::

        run_vdb_cmt.apply_async()
        ...
        run_vdb_cmt.finish(kill=True)

    Teardown::

        run_vdb_cmt.ensure_ready()
        setup_vdb_cmt.ensure_ready()
"""
from valence.vl_components.vdbench.profiles._base import (
    BaseVdbExtendedProfile,
    BaseVdbProfile,
)
from valence.vl_components.vdbench.profiles._csv import (
    CsvVdbExtendedProfile,
    CsvVdbProfile,
)
from valence.vl_components.vdbench.profiles._helpers import (
    get_max_transfer_size,
    get_varied_workload_configs,
    get_workloads,
    isolate_access_ranges,
)
from valence.vl_components.vdbench.profiles._qos_testing import (
    QosHistIopsProfile,
    QosHistStandardBlockSizeProfile,
    QosHistThrottlingProfile,
)
from valence.vl_components.vdbench.profiles._simple import SimpleVdbProfile
from valence.vl_components.vdbench.profiles._standard import (
    StandardVdbExtendedProfile,
    StandardVdbProfile,
)
