import unittest

import mock
import pytest
from sf_ssh import ssh

from valence import models
from valence.vl_components.vdbench import (
    _errors,
    _helpers,
)


class TestTailVdbLog(unittest.TestCase):
    def setUp(self):
        self.client_dm = models.ResourceClient.get_mock_object()

    @mock.patch('sf_ssh.ssh.Ssh.close')
    @mock.patch('sf_ssh.ssh.Ssh.run_command')
    @mock.patch('sf_ssh.ssh.Ssh._init_client')
    def test_all(self, init_client, run_ssh_cmd, close_client):
        run_ssh_cmd.return_value = ssh.RunCommandOutput('line1...\nline2', '', 0)
        output = _helpers.tail_vdb_log(self.client_dm, '/log/path', 2, 2000)
        assert output == ['line1...', 'line2']
        run_ssh_cmd.assert_called_with('tail -n +{} {} | head -n {}'.format(2 + 1, '/log/path', 2000))

        run_ssh_cmd.return_value = ssh.RunCommandOutput('output', ['error'], 1)
        with pytest.raises(_errors.VdbenchLogFileMissingError):
            output = _helpers.tail_vdb_log(self.client_dm, '/log/path', 0, 2000)

        run_ssh_cmd.return_value = ssh.RunCommandOutput('output', ['error'], 0)
        with pytest.raises(_errors.VdbenchScraperError):
            output = _helpers.tail_vdb_log(self.client_dm, '/log/path', 0, 2000)

        run_ssh_cmd.return_value = ssh.RunCommandOutput('output', '', 1)
        with pytest.raises(_errors.VdbenchScraperError):
            output = _helpers.tail_vdb_log(self.client_dm, '/log/path', 0, 2000)
