import random
import unittest

import mock
import pytest

from valence import functional_models
from valence.vl_helpers import _bundle

# Mock the FMs once to avoid extra work
NODE_FMS = [functional_models.Node.get_mock_fm()]
CLIENT_FMS = [functional_models.UbuntuClient.get_mock_fm(),
              functional_models.UbuntuClient.get_mock_fm(),
              functional_models.UbuntuClient.get_mock_fm()]


@mock.patch.object(functional_models.Node, 'pull_support_bundle', autospec=True)
@mock.patch.object(functional_models.UbuntuClient, 'pull_support_bundle', autospec=True)
class TestSupportBundleHelpers(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.artifact_manager = mock.MagicMock()

    def test_nodes_provided(self, mock_client_pull, mock_node_pull):
        """Test pull_support_bundles with node_fms."""
        purge_cores = random.choice([True, False])
        purge_crash_dumps = random.choice([True, False])
        compress = random.choice([True, False])
        collect_backtraces = random.choice([True, False])
        zk_heap_dump = random.choice([True, False])
        verify_logs = random.choice([True, False])

        _bundle.pull_support_bundles(self.artifact_manager, node_fms=NODE_FMS,
                                     purge_cores=purge_cores,
                                     purge_crash_dumps=purge_crash_dumps, compress=compress,
                                     collect_backtraces=collect_backtraces,
                                     zk_heap_dump=zk_heap_dump,
                                     verify_logs=verify_logs)
        mock_node_pull.assert_called_with(NODE_FMS[0], self.artifact_manager, purge_cores=purge_cores,
                                          purge_crash_dumps=purge_crash_dumps, compress=compress,
                                          collect_backtraces=collect_backtraces,
                                          zk_heap_dump=zk_heap_dump, verify_logs=verify_logs)
        assert not mock_client_pull.called

    def test_clients_provided(self, mock_client_pull, mock_node_pull):
        """Test pull_support_bundles with client_fms."""
        _bundle.pull_support_bundles(self.artifact_manager, client_fms=CLIENT_FMS)

        calls = [mock.call(CLIENT_FMS[0], self.artifact_manager),
                 mock.call(CLIENT_FMS[1], self.artifact_manager),
                 mock.call(CLIENT_FMS[2], self.artifact_manager)]
        mock_client_pull.assert_has_calls(calls, any_order=True)
        assert not mock_node_pull.called

    def test_no_input_provided(self, mock_client_pull, mock_node_pull):
        with pytest.raises(TypeError):
            _bundle.pull_support_bundles(self.artifact_manager)
