import time
import unittest

import mock
import pytest
from sf_ssh import ssh

import sf_api.api_constants
from camelot.models import at2_enums
from sf_api import (
    enums,
    models,
    result_models,
    sf_version,
)
from valence import (
    errors,
    functional_models,
    models,
    unittest_utils,
    utils,
    vl_helpers,
)
from valence.vl_helpers import _cluster

# noinspection PyUnusedLocal

# Mock the cluster DM and FM once to avoid extra work
RS_CLUSTER_DM = models.ResourceCluster.get_mock_object()
CLUSTER_FM = functional_models.Cluster(RS_CLUSTER_DM, authenticate=False)
SS_REPORT = """
<html><head><title>ss :: Report</title></head><body><h2>ServiceID: 5 (Slice) NodeID: 1 ChassisName:
QTFCR28040024 DriveIDs:  Drive Serial: nvme-sn-S3T2NX0K401216</h2><p><table border=1
style="border-collapse: collapse;"> <tr>  <th colspan=2>PrimaryCache</th>
<th colspan=5>PrimaryCacheFileManager</th>  <th colspan=3>SecondaryCache</th>
<th colspan=4>ClusterStorage</th> </tr> <tr>  <th>ssLoad</th>  <th>LSBlockCount</th>
<th>ActiveSegments</th>  <th>DirtySegments</th>  <th>CleanSegments</th>  <th>FreeSegments</th>
<th>CacheFiles</th>  <th>FlushSetOutstanding</th>  <th>LSBlockCount</th>  <th>OutstandingWrites</th>
<th>default number of outstanding message</th>  <th>number of outstanding messages</th>
</tr><tr><td>0</td><td>0</td><td>{active_segments}</td><td>0</td><td>0</td><td>72</td><td>1</td>
<td>0</td><td>0</td><td>0</td><td></td><td> cDefaultNumberOfBlocksInWriteBlocksMessage=4000</td>
<td> GetNumberOfBlocksInMessageLimit()=4000</td></tr></table><h3>Cache Flusher</h3> running=true
mPrimarySetCount=1 mPersistPauseWork=0<h3>Persist Pump</h3><p><table border=1 style="border-collapse:
collapse;"><tr><th>Index</th><th>SuccessBlocks</th><th>FailBlocks</th></tr></table></p><h3>Flush
Sets and Blocks</h3><table border=1 style="border-collapse: collapse;"><tr><th></th><th colspan=3>
Sets</th><th colspan=2>Blocks</th></tr><tr><th></th><th>Sent</th><th>Completed</th><th>Outstanding</th>
<th>Sent</th><th>Failed</th> </tr><tr><td><b>Primary</b></td><td>14</td><td>14</td><td>0</td>
<td>326958</td><td>16205</td></tr><tr><td><b>Secondary</b></td><td>1</td><td>1</td><td>(n/a)</td>
<td>16205</td><td>0</td></tr></table><br/><b>Queue Depth:0</b><H3>Secondary Cache Flush Statistics</H3>
<table border=1 style="border-collapse: collapse;"><tr><th>SCacheBytesUsed</th>
<th>BytesAdded(Cumulative)</th><th>BytesFlushed(Cumulative)</th><th>AdditionRateMBPerSec</th>
<th>FlushRateMBPerSec</th><th>PredictedTimeToDrain(HH:MM:SS.mmmmmm)</th><th>PercentageFull</th></tr>
<tr><td>0</td><td>0</td><td>23642112</td><td>0.0</td><td>0.0</td><td>"00:00:00.000000"</td><td>0</td>
</tr></table><H2>File System Information</H2><table border=1 style="border-collapse: collapse;"><tr>
<th colspan=3>File</th><th colspan=7>Async Operations</th><th colspan=3>BranchSetManager</th></tr><tr>
<th>File Name</th><th>File Number</th><th>Pushdown Count</th><th>List Size</th><th>Max List Size</th>
<th>Num Ops</th><th>Max Num Ops</th><th>Max Single Op Size</th><th>Trim Merges</th><th>Share Merges</th>
<th>Index Count</th><th>Branch Count</th><th>Reference Count</th></tr><tr><td>"/slicefiles/0.log"</td>
<td>170</td><td>0</td><td>0</td><td>0</td><td>0</td><td>0</td><td>0</td><td>0</td><td>0</td><td>1</td>
<td>1</td><td>3</td></tr><tr><td>"/failed/50.failed"</td><td>175</td><td>0</td><td>0</td><td>0</td>
<td>0</td><td>0</td><td>0</td><td>0</td><td>0</td><td>0</td><td>1</td><td>0</td></tr></table><br>
<table border=1 style="border-collapse: collapse;"><tr><th>Filename</th><th>Branch</th><th>Size</th></tr
><tr><td>/slicefiles/0.log</td><td>All</td><td>12288(12.29 KB)</td></tr><tr><td></td><td>0</td>
<td>12288(12.29 KB)</td></tr><tr><td>/failed/50.failed</td><td>All</td><td>0(0 B)</td></tr></table>
<H3>Disk Stats</H3><pre>Total space: 3839.70 GB</body></html>
"""


@mock.patch('shutil.rmtree')
@mock.patch('tempfile.mkdtemp')
@mock.patch.object(_cluster, 'open', create=True)
@mock.patch.object(sf_api.ClusterApi, 'http_get')
class TestStoreReports(unittest.TestCase):

    def test_store_reports(self, mock_http_get, mock_open, mock_mkdtemp,
                           mock_rmtree):
        tmp_dir = '/tmp/dir_should_not_be_made'
        mock_mkdtemp.return_value = tmp_dir
        artifacts = mock.MagicMock()
        _cluster.store_reports(CLUSTER_FM, artifacts)
        assert artifacts.store_artifacts_local.called
        mock_rmtree.assert_called_with(tmp_dir)


class TestAddNodesHelpers(unittest.TestCase):

    def setUp(self):
        self.pending_node_dms = models.PendingNode.get_mock_object_list(3)
        async_handle_dm = models.AsyncHandle.get_mock_object()
        node_static_id = models.PendingNode.static_id
        self.async_handler = functional_models.AddNodesAsyncHandler(CLUSTER_FM,
                                                                    node_static_id, async_handle_dm)

    @mock.patch('valence.functional_models.AsyncHandler.get_models')
    @mock.patch.object(functional_models.Cluster, 'add_nodes')
    def test_add_nodes(self, mock_add_nodes, mock_get_models):
        mock_add_nodes.return_value = [self.async_handler]
        _cluster.add_nodes(CLUSTER_FM, self.pending_node_dms)


class TestRemoveNodesHelpers(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.node_dms = models.ActiveNode.get_mock_object_list(2)
        cls.pending_node_dms = models.PendingNode.get_mock_object_list(2)
        async_handle_dm = models.AsyncHandle.get_mock_object()
        node_static_id = models.ActiveNode.static_id
        cls.async_handler = functional_models.RemoveNodesAsyncHandler(
            CLUSTER_FM, node_static_id, async_handle_dm)

    @mock.patch('valence.functional_models._async_handler.AsyncHandler.get_models')
    @mock.patch.object(functional_models.Cluster, 'remove_nodes')
    def test_remove_nodes(self, mock_remove_nodes, mock_get_models):
        mock_remove_nodes.return_value = [self.async_handler]
        mock_get_models.return_value = self.pending_node_dms
        ret = _cluster.remove_nodes(CLUSTER_FM, self.node_dms)
        assert ret == self.pending_node_dms

    @mock.patch.object(functional_models.Cluster, 'list_drives')
    @mock.patch.object(vl_helpers._drive, 'remove_drives')
    @mock.patch.object(vl_helpers._cluster, 'remove_nodes')
    def test_remove_nodes_and_drives(self, mock_remove_nodes, mock_remove_drives, mock_list_drives):
        drives = [models.Drive.create(drive_id=1)]
        mock_list_drives.return_value = drives
        vl_helpers.remove_nodes_and_drives(CLUSTER_FM, self.node_dms, timeout=666)
        mock_remove_nodes.assert_called_with(CLUSTER_FM, node_dms=self.node_dms, timeout=666)
        mock_remove_drives.assert_called_with(CLUSTER_FM, drive_dms=drives, timeout=666)


@unittest_utils.patch_sleep()
class TestOrphanPendingNodes(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.pending_node_dms = models.PendingNode.get_mock_object_list(2)

    @mock.patch.object(functional_models.Node, 'set_cluster_config', new=mock.Mock())
    @mock.patch('valence.functional_models.Cluster.list_pending_nodes')
    def test_orphan_pending_nodes(self, mock_list_pending_nodes):
        mock_list_pending_nodes.side_effect = [self.pending_node_dms, self.pending_node_dms, []]
        _cluster.orphan_pending_nodes(CLUSTER_FM, self.pending_node_dms)

    @mock.patch.object(utils, 'timeout_timer')
    @mock.patch.object(functional_models.Node, 'set_cluster_config', new=mock.Mock())
    @mock.patch('valence.functional_models.Cluster.list_pending_nodes')
    def test_orphan_pending_nodes_timeout_error(self, mock_list_pending_nodes, mock_timeout_timer):
        mock_list_pending_nodes.side_effect = [self.pending_node_dms]
        mock_timeout_timer.return_value = iter([False])
        with pytest.raises(errors.TimeoutError):
            _cluster.orphan_pending_nodes(CLUSTER_FM, self.pending_node_dms)


class TestOrphanActiveNodes(unittest.TestCase):

    @mock.patch.object(vl_helpers._cluster, 'remove_nodes_and_drives')
    @mock.patch.object(vl_helpers._cluster, 'orphan_pending_nodes')
    def test_orphan_active_nodes(self, mock_orphan_pending, mock_remove_nodes):
        # these mocked nodes are unnecessary, we could just use objects, but whatever
        active_node_dms = models.ActiveNode.get_mock_object_list(2)
        rs_node_dms = models.ResourceNode.get_mock_object_list(2)

        mock_remove_nodes.return_value = models.PendingNode.get_mock_object_list(2)
        mock_orphan_pending.return_value = rs_node_dms

        assert rs_node_dms == vl_helpers.orphan_active_nodes(CLUSTER_FM, active_node_dms)


@unittest_utils.patch_sleep()
class TestAssociateOrphanedNodes(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.node_fm = functional_models.Node.get_mock_fm()
        cls.pending_node_dms = models.PendingNode.get_mock_object_list(1)

    @mock.patch.object(functional_models.Node, 'set_cluster_config')
    @mock.patch.object(functional_models.Cluster, 'list_pending_nodes')
    @mock.patch.object(functional_models.Cluster, 'get_cluster_info')
    def test_associate_orphaned_nodes(self, mock_get_cluster_info, mock_list_pending_nodes,
                                      mock_set_cluster_config):
        mock_list_pending_nodes.side_effect = [[], self.pending_node_dms]
        _cluster.associate_orphaned_nodes(CLUSTER_FM, self.pending_node_dms)
        assert mock_list_pending_nodes.call_count == 2

    @mock.patch.object(utils, 'timeout_timer')
    @mock.patch.object(functional_models.Node, 'set_cluster_config')
    @mock.patch('valence.functional_models.Cluster.list_pending_nodes')
    @mock.patch.object(functional_models.Cluster, 'get_cluster_info')
    def test_associate_orphaned_nodes_timeout_error(self, mock_get_cluster_info,
                                                    mock_list_pending_nodes, mock_set_cluster_config,
                                                    mock_timeout_timer):
        mock_timeout_timer.return_value = iter([False])
        with pytest.raises(errors.TimeoutError):
            _cluster.associate_orphaned_nodes(CLUSTER_FM, self.pending_node_dms)


class TestDeletePurgeVolumes(unittest.TestCase):

    @mock.patch.object(functional_models.Cluster, 'purge_all_deleted_volumes')
    @mock.patch.object(functional_models.Cluster, 'purge_deleted_volumes')
    def test_purge_volumes(self, mock_purge, mock_purge_all):
        volumes = models.Volume.get_mock_object_list(2)
        mock_purge.return_value = None
        ret_vol = vl_helpers.purge_deleted_volumes(CLUSTER_FM, volumes)
        assert ret_vol is None
        assert not mock_purge_all.called

    @mock.patch.object(functional_models.Cluster, 'purge_all_deleted_volumes')
    @mock.patch.object(functional_models.Cluster, 'purge_deleted_volumes')
    def test_purge_all_volumes(self, mock_purge, mock_purge_all):
        mock_purge_all.return_value = None
        ret_vol = vl_helpers.purge_deleted_volumes(CLUSTER_FM)
        assert ret_vol is None
        assert not mock_purge.called

    @mock.patch.object(functional_models.Cluster, 'delete_volumes')
    @mock.patch.object(vl_helpers._volume, 'purge_deleted_volumes')
    def test_delete_and_purge(self, mock_purge, mock_del):
        volumes = models.Volume.get_mock_object_list(2)
        mock_del.return_value = volumes
        mock_purge.return_value = None
        vl_helpers.delete_and_purge_volumes(CLUSTER_FM, volumes)


class TestCreateVolumes(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.volume_dms = models.Volume.get_mock_object_list(2)

    @mock.patch.object(functional_models.Cluster, 'create_multiple_volumes')
    def test_create_volumes_single_profile(self, mock_create):
        mock_create.return_value = self.volume_dms
        vl_helpers.create_volumes_single_profile(CLUSTER_FM, self.volume_dms[0], 2)

    @mock.patch.object(functional_models.Cluster, 'create_volume')
    def test_create_volumes_individual_profiles(self, mock_create):
        mock_create.return_value = self.volume_dms[0]
        vl_helpers.create_volumes_individual_profiles(CLUSTER_FM, self.volume_dms)


@mock.patch.object(functional_models.Cluster, 'authenticate')
@mock.patch('sf_ssh.Ssh')
@mock.patch.object(functional_models.Cluster, 'list_active_nodes')
class TestCheckStorageNetwork(unittest.TestCase):
    def setUp(self):
        os = at2_enums.ClientOS.ubuntu_12_04
        client_dms = models.ResourceClient.get_mock_object_list(2, overrides={'os': os})
        self.client_fms = [functional_models.get_client_from_dm(c) for c in client_dms]
        self.node_dms = models.ActiveNode.get_mock_object_list(2)

    def test_check_storage_network(self, mock_list_active_nodes, mock_ssh, mock_auth):
        cluster_fm = functional_models.Cluster(RS_CLUSTER_DM)
        inst = mock_ssh.return_value.__enter__()
        inst.run_command.return_value = ssh.RunCommandOutput(stdout='', stderr='', return_code=0)
        mock_list_active_nodes.return_value = self.node_dms
        _cluster.check_storage_network(cluster_fm, self.client_fms)

    def test_check_storage_network_fail(self, mock_list_active_nodes, mock_ssh, mock_auth):
        cluster_fm = functional_models.Cluster(RS_CLUSTER_DM)
        inst = mock_ssh.return_value.__enter__()
        inst.run_command.return_value = ssh.RunCommandOutput(stderr='', stdout='', return_code=1)
        mock_list_active_nodes.return_value = self.node_dms
        with pytest.raises(functional_models._errors.FunctionalClientError):
            _cluster.check_storage_network(cluster_fm, self.client_fms)


@mock.patch('valence.functional_models.Cluster.get_gc_status')
class TestWaitForGarbageCollectionToComplete(unittest.TestCase):
    def test_wait_for_gc_defaults_complete(self, mock_get_gc_status):
        cluster_fm = functional_models.Cluster(RS_CLUSTER_DM)
        mock_get_gc_status.return_value = result_models.GetGCStatus.get_mock_object(
            overrides={'status': enums.GcStatus.complete})
        _cluster.wait_for_garbage_collection_to_complete(cluster_fm)

    @unittest_utils.patch_sleep()
    @mock.patch('time.time')
    def test_wait_for_gc_running_timeout(self, mock_time, mock_get_gc_status):
        cluster_fm = functional_models.Cluster(RS_CLUSTER_DM)
        mock_time.side_effect = [1000, 1002]
        mock_get_gc_status.return_value = result_models.GetGCStatus.get_mock_object(
            overrides={'status': enums.GcStatus.in_progress})
        with pytest.raises(errors.TimeoutError):
            _cluster.wait_for_garbage_collection_to_complete(cluster_fm, timeout=1)

    @mock.patch('time.time')
    def test_wait_for_gc_complete_before_timeout(self, mock_time, mock_get_gc_status):
        cluster_fm = functional_models.Cluster(RS_CLUSTER_DM)
        mock_time.side_effect = [1000, 1002]
        mock_get_gc_status.side_effect = [
            result_models.GetGCStatus.get_mock_object(
                overrides={'status': enums.GcStatus.in_progress}),
            result_models.GetGCStatus.get_mock_object(
                overrides={'status': enums.GcStatus.complete})
        ]
        _cluster.wait_for_garbage_collection_to_complete(cluster_fm, timeout=6, poll_period=0)


@mock.patch('sf_ssh.Ssh')
@mock.patch('valence.functional_models.Cluster.get_drive_hardware_info')
@mock.patch('valence.functional_models.Cluster.list_drives')
@mock.patch('valence.functional_models.Cluster.list_active_nodes')
class TestIsClusterIdle(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.cluster_fm = functional_models.Cluster(RS_CLUSTER_DM)
        cls.node_dms = [models.ActiveNode.get_mock_object() for _ in range(2)]
        cls.drive_dms = [models.Drive.get_mock_object() for _ in range(2)]
        cls.mock_drives_hw_info = [
            models.DriveHardwareInfo.create(logicalname='mock1-1'),
            models.DriveHardwareInfo.create(logicalname='mock1-2'),
            models.DriveHardwareInfo.create(logicalname='mock2-1'),
            models.DriveHardwareInfo.create(logicalname='mock2-2')
        ]

    def test_idle(self, mock_list_nodes, mock_list_drives, mock_get_drive_hw_info, mock_ssh):
        mock_list_nodes.return_value = self.node_dms
        mock_list_drives.return_value = self.drive_dms
        mock_get_drive_hw_info.side_effect = self.mock_drives_hw_info
        mock_ssh_conn = mock_ssh.return_value.__enter__.return_value
        mock_ssh_conn.run_command.return_value = ssh.RunCommandOutput(
            stdout='0', stderr='', return_code=0
        )

        assert _cluster.is_cluster_idle(self.cluster_fm)
        assert mock_list_drives.call_count == 2
        assert mock_get_drive_hw_info.call_count == 4
        assert mock_ssh_conn.run_command.call_count == 2
        assert 'mock1-1' in mock_ssh_conn.run_command.call_args_list[0][0][0]
        assert 'mock1-2' in mock_ssh_conn.run_command.call_args_list[0][0][0]
        assert 'mock2-1' in mock_ssh_conn.run_command.call_args_list[1][0][0]
        assert 'mock2-2' in mock_ssh_conn.run_command.call_args_list[1][0][0]

    def test_not_idle(self, mock_list_nodes, mock_list_drives, mock_get_drive_hw_info, mock_ssh):
        mock_list_nodes.return_value = self.node_dms
        mock_list_drives.return_value = self.drive_dms
        mock_get_drive_hw_info.side_effect = self.mock_drives_hw_info
        mock_ssh_conn = mock_ssh.return_value.__enter__.return_value
        mock_ssh_conn.run_command.return_value = ssh.RunCommandOutput(
            stdout='100', stderr='', return_code=0
        )

        assert not _cluster.is_cluster_idle(self.cluster_fm)
        assert mock_list_drives.call_count == 1
        assert mock_get_drive_hw_info.call_count == 2
        assert mock_ssh_conn.run_command.call_count == 1


@mock.patch('valence.functional_models.Cluster.get_cluster_master')
class TestAssertClusterMasterMoved(unittest.TestCase):
    """Tests assert_cluster_master_moved helper function."""

    @classmethod
    def setUpClass(cls):
        """Setup model objects for the individual test cases."""
        cls.cluster_fm = functional_models.Cluster(RS_CLUSTER_DM)
        cls.node_dms = [models.ActiveNode.get_mock_object() for _ in range(2)]

    def test_cluster_master_moved(self, mock_get_cm):
        """Test assert_cluster_master scenario where node DM was perturbed and CM changed."""
        orig_cm = self.node_dms[0]
        mock_get_cm.return_value = self.node_dms[1]

        _cluster.assert_cluster_master_moved(
            self.cluster_fm,
            orig_cm,
            perturbed_node_dms=[orig_cm]
        )

    def test_cluster_master_assert_not_moved(self, mock_get_cm):
        """Test assert_cluster_master scenario where we assert because the node DM was perturbed and CM
        didn't change.
        """

        orig_cm = self.node_dms[0]
        mock_get_cm.return_value = self.node_dms[0]

        with pytest.raises(vl_helpers._errors.HelperError):
            _cluster.assert_cluster_master_moved(
                self.cluster_fm,
                orig_cm,
                perturbed_node_dms=[orig_cm]
            )

    def test_cluster_master_assert_not_perturbed(self, mock_get_cm):
        """Test assert_cluster_master scenario where we don't assert when the node DM wasn't perturbed.
        """

        orig_cm = self.node_dms[0]
        mock_get_cm.return_value = self.node_dms[0]

        # In this case, because we don't provide a perturbed list we won't assert if the master node
        # didn't change.
        _cluster.assert_cluster_master_moved(
            self.cluster_fm,
            orig_cm,
        )


class TestWaitForHealthyServices(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        """Setup model objects for the individual test cases."""
        cls.cluster_fm = functional_models.Cluster(RS_CLUSTER_DM)

    def setUp(self):
        self.service_status_fail = result_models.GetServiceStatus.get_mock_response().result_dm
        self.service_status_fail.failed = {1: 'abc'}
        self.service_status_success = result_models.GetServiceStatus.get_mock_response().result_dm
        self.service_status_success.failed = None

    @unittest_utils.patch_sleep()
    @mock.patch.object(time, 'time')
    @mock.patch.object(functional_models.Cluster, 'get_service_status')
    def test_poll(self, mock_service_status, mock_time):
        mock_time.return_value = 0
        mock_service_status.side_effect = [self.service_status_fail, self.service_status_success]
        vl_helpers.wait_for_healthy_services(self.cluster_fm)

    @mock.patch.object(functional_models.Cluster, 'get_service_status')
    def test_specify_services(self, mock_service_status):
        mock_service_status.return_value = self.service_status_fail
        vl_helpers.wait_for_healthy_services(self.cluster_fm, service_ids=[2])

    @mock.patch.object(utils, 'timeout_timer')
    @mock.patch.object(functional_models.Cluster, 'get_service_status')
    def test_timeout(self, mock_service_status, mock_timer):
        mock_timer.return_value = iter([False])
        mock_service_status.return_value = self.service_status_fail
        with pytest.raises(errors.TimeoutError):
            vl_helpers.wait_for_healthy_services(self.cluster_fm)


@mock.patch('valence.vl_helpers._bin_sync.wait_for_bin_sync_to_start')
@mock.patch('valence.vl_helpers._bin_sync.wait_for_bin_sync_to_finish_with_timeout')
@mock.patch('valence.vl_helpers._slice.wait_for_slice_sync_to_start')
@mock.patch('valence.vl_helpers._slice.wait_for_slice_sync_to_finish')
@mock.patch('valence.vl_helpers._bin_sync.is_bin_syncing')
@mock.patch('valence.vl_helpers._slice.is_slice_syncing')
class TestWaitForBinAndSliceSyncComplete(unittest.TestCase):
    """Tests wait_for_bin_and_slice_sync_complete helper function."""

    def test_wait_for_bin_and_slice_sync_complete_default(self, mock_is_slice_syncing,
                                                          mock_is_bin_syncing,
                                                          mock_wait_for_slice_sync_to_finish,
                                                          mock_wait_for_slice_sync_to_start,
                                                          mock_wait_for_bin_sync_to_finish,
                                                          mock_wait_for_bin_sync_to_start):
        mock_is_slice_syncing.return_value = True
        mock_is_bin_syncing.return_value = True

        _cluster.wait_for_bin_and_slice_sync_complete(CLUSTER_FM)
        assert mock_wait_for_bin_sync_to_start.called
        assert mock_wait_for_slice_sync_to_start.called
        assert mock_is_slice_syncing.called
        assert mock_is_bin_syncing.called
        assert mock_wait_for_slice_sync_to_finish.called
        assert mock_wait_for_bin_sync_to_finish.called

    def test_wait_for_bin_and_slice_sync_complete_skip_start(self, mock_is_slice_syncing,
                                                             mock_is_bin_syncing,
                                                             mock_wait_for_slice_sync_to_finish,
                                                             mock_wait_for_slice_sync_to_start,
                                                             mock_wait_for_bin_sync_to_finish,
                                                             mock_wait_for_bin_sync_to_start):
        mock_is_slice_syncing.return_value = True
        mock_is_bin_syncing.return_value = True

        _cluster.wait_for_bin_and_slice_sync_complete(CLUSTER_FM, skip_sync_start=True)
        assert mock_wait_for_bin_sync_to_start.not_called
        assert mock_wait_for_slice_sync_to_start.not_called
        assert mock_is_slice_syncing.called
        assert mock_is_bin_syncing.called
        assert mock_wait_for_slice_sync_to_finish.called
        assert mock_wait_for_bin_sync_to_finish.called

    def test_wait_for_bin_and_slice_sync_complete_skip_finish(self, mock_is_slice_syncing,
                                                              mock_is_bin_syncing,
                                                              mock_wait_for_slice_sync_to_finish,
                                                              mock_wait_for_slice_sync_to_start,
                                                              mock_wait_for_bin_sync_to_finish,
                                                              mock_wait_for_bin_sync_to_start):
        mock_is_slice_syncing.return_value = True
        mock_is_bin_syncing.return_value = True

        _cluster.wait_for_bin_and_slice_sync_complete(CLUSTER_FM, skip_sync_finish=True)
        assert mock_wait_for_bin_sync_to_start.called
        assert mock_wait_for_slice_sync_to_start.called
        assert mock_is_slice_syncing.not_called
        assert mock_is_bin_syncing.not_called
        assert mock_wait_for_slice_sync_to_finish.not_called
        assert mock_wait_for_bin_sync_to_finish.not_called

    def test_wait_for_bin_and_slice_sync_complete_skip_start_finish(self, mock_is_slice_syncing,
                                                                    mock_is_bin_syncing,
                                                                    mock_wait_for_slice_sync_to_finish,
                                                                    mock_wait_for_slice_sync_to_start,
                                                                    mock_wait_for_bin_sync_to_finish,
                                                                    mock_wait_for_bin_sync_to_start):
        mock_is_slice_syncing.return_value = True
        mock_is_bin_syncing.return_value = True

        _cluster.wait_for_bin_and_slice_sync_complete(CLUSTER_FM, skip_sync_start=True,
                                                      skip_sync_finish=True)
        assert mock_wait_for_bin_sync_to_start.not_called
        assert mock_wait_for_slice_sync_to_start.not_called
        assert mock_is_slice_syncing.not_called
        assert mock_is_bin_syncing.not_called
        assert mock_wait_for_slice_sync_to_finish.not_called
        assert mock_wait_for_bin_sync_to_finish.not_called

    def test_wait_for_bin_and_slice_sync_complete_only_bin_syncing(self, mock_is_slice_syncing,
                                                                   mock_is_bin_syncing,
                                                                   mock_wait_for_slice_sync_to_finish,
                                                                   mock_wait_for_slice_sync_to_start,
                                                                   mock_wait_for_bin_sync_to_finish,
                                                                   mock_wait_for_bin_sync_to_start):
        mock_is_slice_syncing.return_value = False
        mock_is_bin_syncing.return_value = True

        _cluster.wait_for_bin_and_slice_sync_complete(CLUSTER_FM, skip_sync_start=True)
        assert mock_wait_for_slice_sync_to_finish.called
        assert mock_wait_for_bin_sync_to_finish.called

    def test_wait_for_bin_and_slice_sync_complete_no_syncing(self, mock_is_slice_syncing,
                                                             mock_is_bin_syncing,
                                                             mock_wait_for_slice_sync_to_finish,
                                                             mock_wait_for_slice_sync_to_start,
                                                             mock_wait_for_bin_sync_to_finish,
                                                             mock_wait_for_bin_sync_to_start):
        mock_is_slice_syncing.return_value = False
        mock_is_bin_syncing.return_value = False

        _cluster.wait_for_bin_and_slice_sync_complete(CLUSTER_FM, skip_sync_start=True)
        assert mock_wait_for_slice_sync_to_finish.not_called
        assert mock_wait_for_bin_sync_to_finish.not_called


class TestRecreateCluster(unittest.TestCase):
    """Tests for recreate_cluster vl_helper."""

    @mock.patch.object(vl_helpers._node, 'create_cluster')
    @mock.patch.object(functional_models.AsyncHandler, 'join')
    @mock.patch.object(functional_models.Node, 'get_version_info')
    @mock.patch.object(functional_models.Node, 'start_rtfi')
    @mock.patch.object(functional_models.Cluster, 'list_active_nodes')
    def test_recreate_cluster(self, mock_list_nodes, mock_start_rtfi, mock_get_version_info, mock_join,
                              mock_create_cluster):
        mock_list_nodes.return_value = [models.ActiveNode.get_mock_object()]
        mock_get_version_info.return_value = result_models.GetVersionInfo.get_mock_object()
        mock_repo_name = 'sodium-patch7-dev'
        vl_helpers.recreate_cluster(functional_models.Cluster.get_mock_fm(),
                                    dev_image_repo=mock_repo_name)
        assert mock_start_rtfi.called
        assert mock_create_cluster.called


@mock.patch.object(time, 'time')
@mock.patch.object(time, 'sleep')
@mock.patch.object(functional_models.Cluster, 'api_is_responsive')
class TestWaitForApiUp(unittest.TestCase):
    """Tests for wait_for_api_up vl_helper."""
    @classmethod
    def setUpClass(cls):
        cls.cluster_fm = functional_models.Cluster.get_mock_fm()

    def test_wait_for_api_up_success(self, mock_responsive, mock_sleep, mock_time):
        mock_time.return_value = 0
        mock_responsive.side_effect = [False, True]
        vl_helpers.wait_for_api_up(self.cluster_fm)

    def test_wait_for_api_up_fail(self, mock_responsive, mock_sleep, mock_time):
        mock_time.side_effect = [0, 2]
        mock_responsive.return_value = False
        with pytest.raises(errors.TimeoutError):
            vl_helpers.wait_for_api_up(self.cluster_fm, timeout=1)


class TestEnableFeature(unittest.TestCase):
    """Tests for vl_heleprs.enable_feature"""
    @mock.patch.object(functional_models.Cluster, 'enable_feature')
    def test_enable_feature(self, mock_enable_feature):
        cluster_fm = functional_models.Cluster.get_mock_fm()
        vl_helpers.enable_feature(cluster_fm=cluster_fm, feature=enums.SfFeature.fips_drives)
        assert mock_enable_feature.called


@mock.patch.object(functional_models.Cluster, 'get_fips_report')
class TestValidateFipsDrivesReport(unittest.TestCase):
    """Tests for wait_for_fips_drives_report vl_helper."""
    @classmethod
    def setUpClass(cls):
        """Setup model objects for the individual test cases."""
        cls.cluster_fm = functional_models.Cluster.get_mock_fm()

    def setUp(self):
        fips_drives_ready = {'node_id': 1, 'fips_drives': enums.FipsDrivesStatus.ready}
        node_report_ready = models.FipsNodeReport.get_mock_object(overrides=dict(fips_drives_ready))
        self.report_pass = result_models.GetFipsReport.get_mock_object(
                                overrides=dict(nodes=[node_report_ready]))
        self.report_pass.nodes[0].fips_drives = enums.FipsDrivesStatus.ready

        fips_drives_none = {'node_id': 1, 'fips_drives': enums.FipsDrivesStatus.none}
        node_report_none = models.FipsNodeReport.get_mock_object(overrides=dict(fips_drives_none))
        self.report_fail = result_models.GetFipsReport.get_mock_object(
                                overrides=dict(nodes=[node_report_none]))
        self.report_fail.nodes[0].fips_drives = enums.FipsDrivesStatus.none

        self.report_ready_dup = result_models.GetFipsReport.get_mock_object(
                               overrides=dict(nodes=[node_report_ready, node_report_ready]))
        self.report_ready_dup.nodes[0].fips_drives = enums.FipsDrivesStatus.ready
        self.report_ready_dup.nodes[1].fips_drives = enums.FipsDrivesStatus.ready

        fips_drives_info = {'node_id': 2, 'fips_drives': enums.FipsDrivesStatus.ready}
        node_report_no_node_id = models.FipsNodeReport.get_mock_object(
                                overrides=dict(fips_drives_info))
        self.report_no_node_id = result_models.GetFipsReport.get_mock_object(
                                overrides=dict(nodes=[node_report_no_node_id]))
        self.report_no_node_id.nodes[0].fips_drives = enums.FipsDrivesStatus.ready

        self.node_dm = models.ActiveNode.get_mock_object(overrides=dict(node_id=1))

    def test_validate_fips_drives_report(self, mock_fips_report):
        mock_fips_report.return_value = self.report_pass
        vl_helpers.validate_fips_drives_report(self.cluster_fm, [self.node_dm],
                                               enums.FipsDrivesStatus.ready)
        mock_fips_report.assert_called()

    def test_validate_fips_drives_report_fail(self, mock_fips_report):
        mock_fips_report.return_value = self.report_fail
        assert vl_helpers.validate_fips_drives_report(self.cluster_fm, [self.node_dm],
                                                      enums.FipsDrivesStatus.ready) is False
        mock_fips_report.assert_called()

    def test_validate_fail_dup_node_id(self, mock_fips_report):
        mock_fips_report.return_value = self.report_ready_dup
        with pytest.raises(ValueError):
            vl_helpers.validate_fips_drives_report(self.cluster_fm, [self.node_dm],
                                                   enums.FipsDrivesStatus.ready)

    def test_validate_no_node_id_found(self, mock_fips_report):
        mock_fips_report.return_value = self.report_no_node_id
        with pytest.raises(ValueError):
            vl_helpers.validate_fips_drives_report(self.cluster_fm, [self.node_dm],
                                                   enums.FipsDrivesStatus.ready)


@unittest_utils.patch_sleep()
class TestWaitForClusterIdle(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.cluster_fm = functional_models.Cluster(RS_CLUSTER_DM)

    @mock.patch('valence.functional_models.Cluster.get_cluster_stats')
    def test_wait_for_cluster_idle(self, mock_get_cluster_stats):
        stats = result_models.GetClusterStats.get_mock_object(
            overrides=dict(cluster_stats=dict(write_ops=10, read_ops=10)))
        stats2 = result_models.GetClusterStats.get_mock_object(
            overrides=dict(cluster_stats=dict(write_ops=100, read_ops=100)))
        mock_get_cluster_stats.side_effect = [stats, stats2, stats2]
        _cluster.wait_for_cluster_idle(self.cluster_fm, timeout=1)
        assert mock_get_cluster_stats.call_count == 3

    @mock.patch('time.time')
    def test_wait_for_cluster_idle_timeout(self, mock_time):
        mock_time.side_effect = [1, 601]
        with pytest.raises(errors.TimeoutError):
            _cluster.wait_for_cluster_idle(self.cluster_fm, timeout=1)


@mock.patch('valence.functional_models.Cluster.get_report_raw')
class TestIsPcacheEmpty(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.cluster_fm = functional_models.Cluster(RS_CLUSTER_DM)

    def test_is_pcache_empty(self, mock_get_report_raw):
        empty_return_value = SS_REPORT.format(active_segments=0)
        mock_get_report_raw.return_value = empty_return_value
        assert _cluster.is_pcache_empty(self.cluster_fm)
        assert mock_get_report_raw.call_count == 1

    def test_is_pcache_not_empty(self, mock_get_report_raw):
        in_use_return_value = SS_REPORT.format(active_segments=42)

        mock_get_report_raw.return_value = in_use_return_value
        assert not _cluster.is_pcache_empty(self.cluster_fm)

    def test_no_valid_cache_headers(self, mock_get_report_raw):
        no_valid_cache_headers_return_value = '<table>empty</table>'
        mock_get_report_raw.return_value = no_valid_cache_headers_return_value
        assert _cluster.is_pcache_empty(self.cluster_fm)
